@@ -1,9 +1,12 @@
 /******************************************************************************
+ * $Id: cgiutil.c 9172 2009-07-13 20:26:41Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  cgiRequestObj and CGI parameter parsing. 
  * Author:   Steve Lime and the MapServer team.
  *
+ * Notes: Portions derived from NCSA HTTPd Server's example CGI programs (util.c).
+ *
  ******************************************************************************
  * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
@@ -33,7 +36,7 @@
 #include "cgiutil.h"
 #include "mapserver.h"
 
-MS_CVSID("$Id: cgiutil.c 6764 2007-08-31 21:40:06Z tamas $")
+MS_CVSID("$Id: cgiutil.c 9172 2009-07-13 20:26:41Z dmorissette $")
 
 #define LF 10
 #define CR 13
@@ -41,7 +44,8 @@ MS_CVSID("$Id: cgiutil.c 6764 2007-08-31 21:40:06Z tamas $")
 static char *readPostBody( cgiRequestObj *request ) 
 {
   char *data; 
-  int data_max, data_len, chunk_size;
+  size_t data_max, data_len;
+  int chunk_size;
 
   msIO_needBinaryStdin();
 
@@ -49,11 +53,17 @@ static char *readPostBody( cgiRequestObj *request )
   /*      If the length is provided, read in one gulp.                    */
   /* -------------------------------------------------------------------- */
   if( getenv("CONTENT_LENGTH") != NULL ) {
-    data_max = atoi(getenv("CONTENT_LENGTH"));
+    data_max = (size_t) atoi(getenv("CONTENT_LENGTH"));
+    /* Test for suspicious CONTENT_LENGTH (negative value or SIZE_MAX) */
+    if( data_max >= SIZE_MAX ) {
+      msIO_printf("Content-type: text/html%c%c",10,10);
+      msIO_printf("Suspicious Content-Length.\n");
+      exit( 1 );
+    }
     data = (char *) malloc(data_max+1);
     if( data == NULL ) {
       msIO_printf("Content-type: text/html%c%c",10,10);
-      msIO_printf("malloc() failed, Content-Length: %d unreasonably large?\n", data_max );
+      msIO_printf("malloc() failed, Content-Length: %u unreasonably large?\n", data_max );
       exit( 1 );
     }
 
@@ -70,7 +80,9 @@ static char *readPostBody( cgiRequestObj *request )
   /* -------------------------------------------------------------------- */
   /*      Otherwise read in chunks to the end.                            */
   /* -------------------------------------------------------------------- */
-  data_max = 10000;
+#define DATA_ALLOC_SIZE 10000
+
+  data_max = DATA_ALLOC_SIZE;
   data_len = 0;
   data = (char *) malloc(data_max+1);
 
@@ -78,12 +90,19 @@ static char *readPostBody( cgiRequestObj *request )
     data_len += chunk_size;
 
     if( data_len == data_max ) {
-      data_max = data_max + 10000;
+      /* Realloc buffer, making sure we check for possible size_t overflow */
+        if ( data_max > SIZE_MAX - (DATA_ALLOC_SIZE+1) ) {
+        msIO_printf("Content-type: text/html%c%c",10,10);
+        msIO_printf("Possible size_t overflow, cannot reallocate input buffer, POST body too large?\n" );
+        exit(1);
+      }
+
+      data_max = data_max + DATA_ALLOC_SIZE;
       data = (char *) realloc(data, data_max+1);
 
       if( data == NULL ) {
         msIO_printf("Content-type: text/html%c%c",10,10);
-        msIO_printf("out of memory trying to allocate %d input buffer, POST body too large?\n", data_max+1 );
+        msIO_printf("out of memory trying to allocate %u input buffer, POST body too large?\n", data_max+1 );
         exit(1);
       }
     }
@@ -201,7 +220,8 @@ int loadParams(cgiRequestObj *request) {
 
   /* check for any available cookies */
   s = getenv("HTTP_COOKIE");
-  if(s != NULL) {    
+  if(s != NULL) {
+    request->httpcookiedata = strdup(s);
     for(x=0;s[0] != '\0';x++) {
       if(m >= MS_MAX_CGI_PARAMS) {
         msIO_printf("Too many name/value pairs, aborting.\n");
@@ -395,6 +415,7 @@ cgiRequestObj *msAllocCgiObj() {
   request->type = -1;
   request->contenttype = NULL;
   request->postrequest = NULL;
+  request->httpcookiedata = NULL;
 
   return request;
 }
@@ -408,8 +429,10 @@ void msFreeCgiObj(cgiRequestObj *request) {
   request->type = -1;
   msFree(request->contenttype);
   msFree(request->postrequest);
+  msFree(request->httpcookiedata);
   request->contenttype = NULL;
   request->postrequest = NULL;
+  request->httpcookiedata = NULL;
 
   msFree(request);
 }
