@@ -1,9 +1,13 @@
 /******************************************************************************
+ * $Id: mapbits.c 7801 2008-07-09 05:02:01Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Implementation of bit array functions.
  * Author:   Steve Lime and the MapServer team.
  *
+ * Notes: Derived from code placed in the public domain by Bob Stout, for more 
+ * information see http://c.snippets.org/snip_lister.php?fname=bitarray.c.
+ *
  ******************************************************************************
  * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
@@ -28,11 +32,11 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapbits.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: mapbits.c 7801 2008-07-09 05:02:01Z sdlime $")
 
 #include <limits.h>
 
-/* originally found at http://www.snippets.org/ */
+/* #define msGetBit(array, index) (*((array) + (index)/CHAR_BIT) & ( 1 << ((index) % CHAR_BIT))) */
 
 size_t msGetBitArraySize(int numbits)
 {
@@ -52,6 +56,40 @@ int msGetBit(char *array, int index)
   return (*array & (1 << (index % CHAR_BIT))) != 0;    /* 0 or 1 */
 }
 
+/*
+** msGetNextBit( status, start, size)
+**
+** Quickly find the next bit set. If start == 0 and 0 is set, will return 0.
+** If hits end of bitmap without finding set bit, will return -1.
+**
+*/
+int msGetNextBit(char *array, int i, int size) { 
+
+  register char b;
+
+  while(i < size) {
+    b = *(array + (i/CHAR_BIT));
+    if( b && (b >> (i % CHAR_BIT)) ) {
+      /* There is something in this byte */
+      /* And it is not to the right of us */
+      if( b & ( 1 << (i % CHAR_BIT)) ) {
+        /* There is something at this bit! */
+        return i;
+      }
+      else {
+        i++;
+      }
+    }
+    else {
+      /* Nothing in this byte, move to start of next byte */
+      i += CHAR_BIT - (i % CHAR_BIT);
+    }
+  }
+  
+  /* Got to the last byte with no hits! */
+  return -1;
+}
+  
 void msSetBit(char *array, int index, int value)
 {
   array += index / CHAR_BIT;
