@@ -1,9 +1,7 @@
 /**********************************************************************
- * $Id: mapcontext.c 6983 2007-10-30 21:37:36Z assefa $
+ * $Id: mapcontext.c 8461 2009-01-28 15:26:35Z tomkralidis $
  *
- * Name:     mapcontext.c
  * Project:  MapServer
- * Language: C
  * Purpose:  OGC Web Map Context implementation
  * Author:   Julien-Samuel Lacroix, DM Solutions Group (lacroix@dmsolutions.ca)
  *
@@ -30,7 +28,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapcontext.c 6983 2007-10-30 21:37:36Z assefa $")
+MS_CVSID("$Id: mapcontext.c 8461 2009-01-28 15:26:35Z tomkralidis $")
 
 #if defined(USE_WMS_LYR) && defined(USE_OGR)
 
@@ -885,7 +883,7 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
   char *pszProj=NULL;
   char *pszValue;
   char *pszHash, *pszName=NULL;
-  CPLXMLNode *psFormatList, *psFormat, *psStyleList, *psStyle;
+  CPLXMLNode *psFormatList, *psFormat, *psStyleList, *psStyle, *psExtension;
   CPLXMLNode *psDimensionList, *psDimension;
   int nStyle;
   layerObj *layer;
@@ -893,6 +891,7 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
   /* Init new layer */
   if(msGrowMapLayers(map) == NULL)
     return MS_FAILURE;
+  
   layer = (GET_LAYER(map, map->numlayers));
   initLayer(layer, map);
   layer->map = (mapObj *)map;
@@ -951,6 +950,9 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
       }
   }
 
+  /* Server Title */
+  msGetMapContextXMLHashValue(psLayer, "Server.title", &(layer->metadata),  "wms_server_title");
+
   /* Abstract */
   msGetMapContextXMLHashValue(psLayer, "Abstract", &(layer->metadata), 
                               "wms_abstract");
@@ -1152,7 +1154,16 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
           }
       }
   }
-  
+ 
+  /* Extension */
+  psExtension = CPLGetXMLNode(psLayer, "Extension");
+  if (psExtension != NULL) {
+      pszValue = (char*)CPLGetXMLValue(psExtension, "ol:opacity", NULL);
+      if(pszValue != NULL)
+      {
+          layer->opacity = atof(pszValue)*100;
+      } 
+  }
 
   return MS_SUCCESS;
 }
@@ -1440,7 +1451,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
   char * tabspace=NULL, *pszValue, *pszChar,*pszSLD=NULL,*pszURL,*pszSLD2=NULL;
   char *pszStyle, *pszCurrent, *pszStyleItem, *pszSLDBody;
   char *pszEncodedVal;
-  int i, nValue, nVersion=-1;
+  int i, nValue, nVersion=OWS_VERSION_NOTSET;
   /* Dimension element */
   char *pszDimension;
   const char *pszDimUserValue=NULL, *pszDimUnits=NULL, *pszDimDefault=NULL;
@@ -1454,7 +1465,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
     version = "1.1.0";
 
   nVersion = msOWSParseVersionString(version);
-  if (nVersion < 0)
+  if (nVersion == OWS_VERSION_BADFORMAT)
       return MS_FAILURE;  /* msSetError() already called. */
 
   /* Make sure this is a supported version */
@@ -1690,7 +1701,13 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                                        NULL, "wms_server_version", OWS_WARN,
                           "      <Server service=\"OGC:WMS\" version=\"%s\" ",
                                        "1.0.0");
-          if(GET_LAYER(map, i)->name)
+
+          if(msOWSLookupMetadata(&(GET_LAYER(map, i)->metadata), "MO", "server_title"))
+              msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata),
+                                       NULL, "wms_server_title", OWS_NOERR,
+                                       "title=\"%s\">\n", "");
+
+          else if(GET_LAYER(map, i)->name)
               msOWSPrintEncodeMetadata(stream, &(GET_LAYER(map, i)->metadata), 
                                        NULL, "wms_title", OWS_NOERR, 
                                        "title=\"%s\">\n", GET_LAYER(map, i)->name);
