@@ -1,9 +1,12 @@
 /******************************************************************************
+ * $Id: mapcopy.c 8207 2008-12-08 14:50:17Z tbonfort $
+ *
  * Project: MapServer
  * Purpose: Functions to allow copying/cloning of maps
  * Author:  Sean Gillies, sgillies@frii.com
  *
- * Notes: These functions are not in mapfile.c because that file is 
+ * Notes: 
+ * These functions are not in mapfile.c because that file is 
  * cumbersome enough as it is.  There is agreement that this code and
  * that in mapfile.c should eventually be split up by object into 
  * mapobj.c, layerobj.c, etc.  Or something like that.  
@@ -42,7 +45,7 @@
 #include "mapserver.h"
 #include "mapsymbol.h"
 
-MS_CVSID("$Id: mapcopy.c 7255 2008-01-10 15:32:46Z sdlime $")
+MS_CVSID("$Id: mapcopy.c 8207 2008-12-08 14:50:17Z tbonfort $")
 
 #include "mapcopy.h"
 
@@ -295,6 +298,8 @@ int msCopyLabel(labelObj *dst, labelObj *src)
 
     MS_COPYSTRING(dst->encoding, src->encoding);
 
+    MS_COPYSTELEM(outlinewidth);
+
     return MS_SUCCESS;
 }
 
@@ -381,7 +386,8 @@ int msCopyStyle(styleObj *dst, styleObj *src)
     MS_COPYSTELEM(minvalue);
     MS_COPYSTELEM(maxvalue);
     MS_COPYSTELEM(opacity);
-
+    MS_COPYSTRING(dst->_geomtransformexpression,src->_geomtransformexpression);
+    MS_COPYSTELEM(_geomtransform);
     MS_COPYSTRING(dst->rangeitem, src->rangeitem);
     MS_COPYSTELEM(rangeitemindex);
 
@@ -443,7 +449,8 @@ int msCopyClass(classObj *dst, classObj *src, layerObj *layer)
     MS_COPYSTRING(dst->keyimage, src->keyimage);
     MS_COPYSTRING(dst->name, src->name);
     MS_COPYSTRING(dst->title, src->title);
-
+    MS_COPYSTRING(dst->group, src->group);
+    
     if (msCopyExpression(&(dst->text), &(src->text)) != MS_SUCCESS) {
         msSetError(MS_MEMERR, "Failed to copy text.", "msCopyClass()");
         return MS_FAILURE;
@@ -656,6 +663,7 @@ int msCopyScalebar(scalebarObj *dst, scalebarObj *src)
     MS_COPYSTELEM(transparent);
     MS_COPYSTELEM(interlace);
     MS_COPYSTELEM(postlabelcache);
+    MS_COPYSTELEM(align);
 
     return MS_SUCCESS;
 }
@@ -708,7 +716,7 @@ int msCopyLegend(legendObj *dst, legendObj *src, mapObj *map)
  * msCopyLayer()                                                       *
  *                                                                     *
  * Copy a layerObj, using mapfile.c:initClass(), msCopyClass(),        *
- * msCopyColor(), msCopyProjection(), msSHPOpenFile(),                 *
+ * msCopyColor(), msCopyProjection(), msShapefileOpen(),                 *
  * msCreateHashTable(), msCopyHashTable(), msCopyExpression()          *
  *                                                                     *
  * As it stands, we are not copying a layer's resultcache              *
@@ -764,6 +772,8 @@ int msCopyLayer(layerObj *dst, layerObj *src)
 
     MS_COPYSTELEM(labelminscaledenom);
     MS_COPYSTELEM(labelmaxscaledenom);
+    MS_COPYSTELEM(mingeowidth);
+    MS_COPYSTELEM(maxgeowidth);
 
     MS_COPYSTELEM(sizeunits);
     MS_COPYSTELEM(maxfeatures);
@@ -845,8 +855,10 @@ int msCopyLayer(layerObj *dst, layerObj *src)
             return MS_FAILURE;
     }
 
-		MS_COPYRECT(&(dst->extent), &(src->extent));
+    MS_COPYRECT(&(dst->extent), &(src->extent));
     
+    MS_COPYSTRING(dst->classgroup, src->classgroup); 
+
     return MS_SUCCESS;
 }
 
