@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapdrawgdal.c 7418 2008-02-29 00:02:49Z nsavard $
  *
  * Project:  MapServer
  * Purpose:  Code for drawing GDAL raster layers.  Called from 
@@ -32,7 +33,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapdrawgdal.c 6944 2007-10-15 03:42:24Z warmerdam $")
+MS_CVSID("$Id: mapdrawgdal.c 7418 2008-02-29 00:02:49Z nsavard $")
 
 extern int InvGeoTransform( double *gt_in, double *gt_out );
 
@@ -1594,6 +1595,7 @@ int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer,
                            double *padfGeoTransform )
 
 {
+    const char *extent_priority = NULL;
 #if defined(USE_WMS_SVR) || defined (USE_WFS_SVR)
     rectObj  rect;
 #endif
@@ -1608,6 +1610,23 @@ int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer,
     padfGeoTransform[4] = 0.0;
     padfGeoTransform[5] = -1.0;
     
+/* -------------------------------------------------------------------- */
+/*      Do we want to override GDAL with a worldfile if one is present? */
+/* -------------------------------------------------------------------- */
+    extent_priority = CSLFetchNameValue( layer->processing, 
+                                         "EXTENT_PRIORITY" );
+
+    if( extent_priority != NULL
+        && EQUALN(extent_priority,"WORLD",5) )
+    {
+	        if( GDALGetDescription(hDS) != NULL 
+            && GDALReadWorldFile(GDALGetDescription(hDS), "wld", 
+                                 padfGeoTransform) )
+        {
+            return MS_SUCCESS;
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Try GDAL.                                                       */
 /*                                                                      */
