@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: maperror.c 8491 2009-02-03 21:27:07Z assefa $
  *
  * Project:  MapServer
  * Purpose:  Implementation of msSetError(), msDebug() and related functions.
@@ -40,7 +41,7 @@
 #endif
 #include <stdarg.h>
 
-MS_CVSID("$Id: maperror.c 6757 2007-08-31 03:30:07Z tomkralidis $")
+MS_CVSID("$Id: maperror.c 8491 2009-02-03 21:27:07Z assefa $")
 
 static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"Unable to access file.",
@@ -81,7 +82,8 @@ static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"GML encoding error.",
             "SOS server error.",
 						"NULL parent pointer error.",
-            "AGG library error."
+            "AGG library error.",
+            "OWS error."
 };
 
 #ifndef USE_THREAD
@@ -333,7 +335,7 @@ void msSetError(int code, const char *message_fmt, const char *routine, ...)
   else
   {
     va_start(args, routine);
-    vsprintf( ms_error->message, message_fmt, args );
+    vsnprintf( ms_error->message, MESSAGELENGTH, message_fmt, args );
     va_end(args);
   }
 
@@ -391,6 +393,8 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
   int nBlack = 0;   
   outputFormatObj *format = NULL;
   char *errormsg = msGetErrorString("; ");
+  char *pFormatBuffer;
+  char cGDFormat[128];
   if (map) {
       if( map->width != -1 && map->height != -1 )
       {
@@ -400,7 +404,9 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
       format = map->outputformat;
   }
 
-  if (format == NULL) format = msCreateDefaultOutputFormat( NULL, "GD/PC256" );
+  /* Default to GIF if no suitable GD output format set */
+  if (format == NULL || (!MS_DRIVER_GD(format) && !MS_DRIVER_AGG(format))) 
+    format = msCreateDefaultOutputFormat( NULL, "GD/PC256" );
 
   img = gdImageCreate(width, height);
   color = gdImageColorAllocate(img, map->imagecolor.red, 
@@ -462,7 +468,18 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
   /* actually write the image */
   if(!filename) 
       msIO_printf("Content-type: %s%c%c", MS_IMAGE_MIME_TYPE(format), 10,10);
-  msSaveImageGD(img, filename, format);
+  if (MS_DRIVER_GD(format))
+    msSaveImageGD(img, filename, format);
+  else
+  {
+      pFormatBuffer = format->driver;
+      strcpy(cGDFormat, "gd/");
+      strcat(cGDFormat, &(format->driver[4]));
+      format->driver = &cGDFormat[0];
+      msSaveImageGD(img, filename, format);
+      format->driver = pFormatBuffer;
+  }
+
   gdImageDestroy(img);
 
   if (format->refcount == 0)
@@ -503,6 +520,12 @@ char *msGetVersion() {
 #ifdef USE_GD_FT
   strcat(version, " SUPPORTS=FREETYPE");
 #endif
+#ifdef USE_ICONV
+  strcat(version, " SUPPORTS=ICONV");
+#endif
+#ifdef USE_FRIBIDI
+  strcat(version, " SUPPORTS=FRIBIDI");
+#endif
 #ifdef USE_WMS_SVR
   strcat(version, " SUPPORTS=WMS_SERVER");
 #endif
@@ -533,6 +556,9 @@ char *msGetVersion() {
 #ifdef USE_POINT_Z_M
   strcat(version, " SUPPORTS=POINT_Z_M");
 #endif
+#ifdef USE_RGBA_PNG
+  strcat(version, " SUPPORTS=RGBA_PNG");
+#endif
 #ifdef USE_TIFF
   strcat(version, " INPUT=TIFF");
 #endif
@@ -566,25 +592,5 @@ char *msGetVersion() {
 
 int msGetVersionInt() 
 {
-    static int version = 0;
-
-    /* Parse MS_VERSION only once */
-    if (version == 0)
-    {
-        char **tokens = NULL;
-        int n = 0;
-        tokens = msStringSplit(MS_VERSION, '.', &n);
-        if (n<3)
-        {
-            msSetError(MS_MISCERR, "Invalid version string: %s", "msGetVersionInt()", MS_VERSION);
-        }
-        else
-        {
-            version = atoi(tokens[0])*10000 + atoi(tokens[1])*100 + atoi(tokens[2]);
-        }
-        if (tokens)
-            msFreeCharArray(tokens, n);
-    }
-
-    return version;
+    return MS_VERSION_NUM;
 }
