@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapgeos.c 7418 2008-02-29 00:02:49Z nsavard $
  *
  * Project:  MapServer
  * Purpose:  MapServer-GEOS integration.
@@ -302,6 +303,9 @@ static shapeObj *msGEOSGeometry2Shape_point(GEOSGeom g)
   GEOSCoordSeq_getX(coords, 0, &(shape->line[0].point[0].x));
   GEOSCoordSeq_getY(coords, 0, &(shape->line[0].point[0].y));
   /* GEOSCoordSeq_getZ(coords, 0, &(shape->line[0].point[0].z)); */
+
+  shape->bounds.minx = shape->bounds.maxx = shape->line[0].point[0].x;
+  shape->bounds.miny = shape->bounds.maxy = shape->line[0].point[0].y;
  
   return shape;
 }
@@ -336,7 +340,9 @@ static shapeObj *msGEOSGeometry2Shape_multipoint(GEOSGeom g)
     GEOSCoordSeq_getY(coords, 0, &(shape->line[0].point[i].y));
     /* GEOSCoordSeq_getZ(coords, 0, &(shape->line[0].point[i].z)); */
   }
-  
+ 
+  msComputeBounds(shape); 
+
   return shape;
 }
 
@@ -368,6 +374,8 @@ static shapeObj *msGEOSGeometry2Shape_line(GEOSGeom g)
     /* GEOSCoordSeq_getZ(coords, i, &(shape->line[0].point[i].z)); */
   }
 
+  msComputeBounds(shape); 
+
   return shape;
 }
 
@@ -407,6 +415,8 @@ static shapeObj *msGEOSGeometry2Shape_multiline(GEOSGeom g)
     msAddLineDirectly(shape, &line);
   }
 
+  msComputeBounds(shape); 
+
   return shape;
 }
 
@@ -462,6 +472,8 @@ static shapeObj *msGEOSGeometry2Shape_polygon(GEOSGeom g)
     msAddLineDirectly(shape, &line);
   }
 
+  msComputeBounds(shape); 
+
   return shape;
 }
 
@@ -523,6 +535,8 @@ static shapeObj *msGEOSGeometry2Shape_multipolygon(GEOSGeom g)
     }
   } /* next polygon */
 
+  msComputeBounds(shape); 
+
   return shape; 
 }
 
