@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapgml.c 7736 2008-06-24 03:54:41Z tomkralidis $
  *
  * Project:  MapServer
  * Purpose:  shapeObj to GML output via MapServer queries.
@@ -28,8 +29,9 @@
 
 #include "mapserver.h"
 #include "maperror.h"
+#include "mapgml.h"
 
-MS_CVSID("$Id: mapgml.c 6580 2007-08-14 14:53:32Z sdlime $")
+MS_CVSID("$Id: mapgml.c 7736 2008-06-24 03:54:41Z tomkralidis $")
 
 /* Use only mapgml.c if WMS or WFS is available */
 
@@ -84,8 +86,8 @@ static int gmlWriteBounds_GML3(FILE *stream, rectObj *rect, const char *srsname,
   } else
     msIO_fprintf(stream, "%s\t<gml:Envelope>\n", tab);
 
-  msIO_fprintf(stream, "%s\t\t<gml:pos>%.6f %.6f</gml:pos>\n", tab, rect->minx, rect->miny);
-  msIO_fprintf(stream, "%s\t\t<gml:pos>%.6f %.6f</gml:pos>\n", tab, rect->maxx, rect->maxy);
+  msIO_fprintf(stream, "%s\t\t<gml:lowerCorner>%.6f %.6f</gml:lowerCorner>\n", tab, rect->minx, rect->miny);
+  msIO_fprintf(stream, "%s\t\t<gml:upperCorner>%.6f %.6f</gml:upperCorner>\n", tab, rect->maxx, rect->maxy);
   
   msIO_fprintf(stream, "%s\t</gml:Envelope>\n", tab);
   msIO_fprintf(stream, "%s</gml:boundedBy>\n", tab);
@@ -1295,10 +1297,10 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
 
 #ifdef USE_PROJ
           if(msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), namespaces, MS_TRUE)) { /* use the map projection first */
-            gmlWriteBounds(stream, OWS_GML2, &(shape.bounds), msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), namespaces, MS_TRUE), "\t\t\t");
+            gmlWriteBounds(stream, OWS_GML2, &(shape.bounds), msOWSGetEPSGProj(&(map->projection), NULL, namespaces, MS_TRUE), "\t\t\t");
             gmlWriteGeometry(stream, NULL, OWS_GML2, &(shape), msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), namespaces, MS_TRUE), NULL, "\t\t\t");
           } else { /* then use the layer projection and/or metadata */
-            gmlWriteBounds(stream, OWS_GML2, &(shape.bounds), msOWSGetEPSGProj(&(lp->projection), &(lp->metadata), namespaces, MS_TRUE), "\t\t\t");
+            gmlWriteBounds(stream, OWS_GML2, &(shape.bounds), msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), namespaces, MS_TRUE), "\t\t\t");
             gmlWriteGeometry(stream, NULL, OWS_GML2, &(shape), msOWSGetEPSGProj(&(lp->projection), &(lp->metadata), namespaces, MS_TRUE), NULL, "\t\t\t");
           }
 #else
@@ -1538,3 +1540,160 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
   return MS_FAILURE;
 #endif /* USE_WFS_SVR */
 }
+
+
+#ifdef USE_LIBXML2
+
+/**
+ * msGML3BoundedBy()
+ *
+ * returns an object of BoundedBy as per GML 3
+ *
+ * @param xmlNsPtr psNs the namespace object
+ * @param minx minx
+ * @param miny miny
+ * @param maxx maxx
+ * @param mayy mayy
+ * @param psEpsg epsg code
+ * @param dimension number of dimensions
+ *
+ * @return psNode xmlNodePtr of XML construct
+ *
+ */
+
+xmlNodePtr msGML3BoundedBy(xmlNsPtr psNs, double minx, double miny, double maxx, double maxy, const char *psEpsg) {
+  xmlNodePtr psNode = NULL, psSubNode = NULL, psSubSubNode = NULL;
+  char *pszTmp = NULL;
+  char *pszTmp2 = NULL;
+  char pszEpsg[11];
+
+  psNode = xmlNewNode(psNs, BAD_CAST "boundedBy");
+  psSubNode = xmlNewChild(psNode, NULL, BAD_CAST "Envelope", NULL);
+
+  if (psEpsg) {
+    sprintf(pszEpsg, "%s", psEpsg);
+    msStringToLower(pszEpsg);
+    pszTmp = msStringConcatenate(pszTmp, "urn:ogc:crs:");
+    pszTmp = msStringConcatenate(pszTmp, pszEpsg);
+    xmlNewProp(psSubNode, BAD_CAST "srsName", BAD_CAST pszTmp);
+    free(pszTmp);
+    pszTmp = msIntToString(2);
+    xmlNewProp(psSubNode, BAD_CAST "srsDimension", BAD_CAST pszTmp);
+    free(pszTmp);
+  }
+
+  pszTmp = msDoubleToString(minx, MS_TRUE);
+  pszTmp = msStringConcatenate(pszTmp, " ");
+  pszTmp2 = msDoubleToString(miny, MS_TRUE);
+  pszTmp = msStringConcatenate(pszTmp, pszTmp2);
+  psSubSubNode = xmlNewChild(psSubNode, NULL, BAD_CAST "lowerCorner", BAD_CAST pszTmp);
+  free(pszTmp);
+  free(pszTmp2);
+
+  pszTmp = msDoubleToString(maxx, MS_TRUE);
+  pszTmp = msStringConcatenate(pszTmp, " ");
+  pszTmp2 = msDoubleToString(maxy,MS_TRUE); 
+  pszTmp = msStringConcatenate(pszTmp, pszTmp2);
+  psSubSubNode = xmlNewChild(psSubNode, NULL, BAD_CAST "upperCorner", BAD_CAST pszTmp);
+  free(pszTmp);
+  free(pszTmp2);
+  return psNode;
+}
+
+/**
+ *msGML3Point()
+ *
+ * returns an object of Point as per GML 3
+ *
+ * @param xmlNsPtr psNs the gml namespace object
+ * @param pszSrsName EPSG code of geometry
+ * @param id
+ * @param x x coordinate
+ * @param y y coordinate
+ * @param z z coordinate
+ *
+ * @return psNode xmlNodePtr of XML construct
+ *
+ */
+
+xmlNodePtr msGML3Point(xmlNsPtr psNs, const char *psSrsName, const char *id, double x, double y) {
+  xmlNodePtr psNode = NULL, psSubNode = NULL;
+  char *pszTmp = NULL;
+  int dimension = 2;
+  char pszSrsName[11];
+
+  psNode = xmlNewNode(psNs, BAD_CAST "Point");
+
+  if (id) {
+    xmlNewNsProp(psNode, psNs, BAD_CAST "id", BAD_CAST id);
+  }
+
+  if (psSrsName) {
+    sprintf(pszSrsName, "%s", psSrsName);
+    msStringToLower(pszSrsName);
+    pszTmp = msStringConcatenate(pszTmp, "urn:ogc:crs:");
+    pszTmp = msStringConcatenate(pszTmp, pszSrsName);
+    xmlNewProp(psNode, BAD_CAST "srsName", BAD_CAST pszTmp);
+    free(pszTmp);
+    pszTmp = msIntToString(dimension);
+    xmlNewProp(psNode, BAD_CAST "srsDimension", BAD_CAST pszTmp);
+    free(pszTmp);
+  }
+
+  pszTmp = msDoubleToString(x, MS_TRUE);
+  pszTmp = msStringConcatenate(pszTmp, " ");
+  pszTmp = msStringConcatenate(pszTmp, msDoubleToString(y, MS_TRUE));
+  psSubNode = xmlNewChild(psNode, NULL, BAD_CAST "pos", BAD_CAST pszTmp);
+
+  free(pszTmp);
+  return psNode;
+}
+
+/**
+ * msGML3TimePeriod()
+ *
+ * returns an object of TimePeriod as per GML 3
+ *
+ * @param xmlNsPtr psNs the gml namespace object
+ * @param pszStart start time
+ * @param pszEnd end time
+ * 
+ * @return psNode xmlNodePtr of XML construct
+ *
+ */
+
+xmlNodePtr msGML3TimePeriod(xmlNsPtr psNs, char *pszStart, char *pszEnd) {
+  xmlNodePtr psNode=NULL,psSubNode=NULL;
+
+  psNode = xmlNewNode(psNs, BAD_CAST "TimePeriod");
+  psSubNode = xmlNewChild(psNode, NULL, BAD_CAST "beginPosition", BAD_CAST pszStart);
+  if (pszEnd)
+    psSubNode = xmlNewChild(psNode, NULL, BAD_CAST "endPosition", BAD_CAST pszEnd);
+  else {
+    psSubNode = xmlNewChild(psNode, NULL, BAD_CAST "endPosition", NULL);
+    xmlNewProp(psNode, BAD_CAST "indeterminatePosition", BAD_CAST "now");
+  }
+  return psNode;
+}
+
+/**
+ * msGML3TimeInstant()
+ *
+ * returns an object of TimeInstant as per GML 3
+ *
+ * @param xmlNsPtr psNs the gml namespace object
+ * @param timeInstant time instant
+ *
+ * @return psNode xmlNodePtr of XML construct
+ *
+ */
+
+xmlNodePtr msGML3TimeInstant(xmlNsPtr psNs, char *pszTime) {
+  xmlNodePtr psNode=NULL,psSubNode=NULL;
+
+  psNode = xmlNewNode(psNs, BAD_CAST "TimeInstant");
+  psSubNode = xmlNewChild(psNode, NULL, BAD_CAST "timePosition", BAD_CAST pszTime);
+  return psNode;
+}
+
+#endif /* USE_LIBXML2 */
