@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: maphttp.c 6384 2007-07-24 16:35:35Z dmorissette $
+ * $Id: maphttp.c 9017 2009-05-15 11:43:56Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  Utility functions to access files via HTTP (requires libcurl)
@@ -37,7 +37,7 @@
 #include "mapows.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: maphttp.c 6384 2007-07-24 16:35:35Z dmorissette $")
+MS_CVSID("$Id: maphttp.c 9017 2009-05-15 11:43:56Z dmorissette $")
 
 #include <time.h>
 #ifndef _WIN32
@@ -121,9 +121,7 @@ void msHTTPCleanup()
 void msHTTPInitRequestObj(httpRequestObj *pasReqInfo, int numRequests) 
 {
     int i;
-    if (!gbCurlInitialized)
-        msHTTPInit();
-  
+
     for(i=0; i<numRequests; i++)
     {
         pasReqInfo[i].pszGetUrl = NULL;
@@ -136,6 +134,12 @@ void msHTTPInitRequestObj(httpRequestObj *pasReqInfo, int numRequests)
         pasReqInfo[i].pszContentType = NULL;
         pasReqInfo[i].pszErrBuf = NULL;
         pasReqInfo[i].pszUserAgent = NULL;
+        pasReqInfo[i].pszHTTPCookieData = NULL;
+        pasReqInfo[i].pszProxyAddress = NULL;
+        pasReqInfo[i].pszProxyUsername = NULL;
+        pasReqInfo[i].pszProxyPassword = NULL;
+        pasReqInfo[i].pszHttpUsername = NULL;
+        pasReqInfo[i].pszHttpPassword = NULL;
 
         pasReqInfo[i].debug = MS_FALSE;
 
@@ -183,6 +187,10 @@ void msHTTPFreeRequestObj(httpRequestObj *pasReqInfo, int numRequests)
             free(pasReqInfo[i].pszUserAgent);
         pasReqInfo[i].pszUserAgent = NULL;
 
+        if (pasReqInfo[i].pszHTTPCookieData)
+            free(pasReqInfo[i].pszHTTPCookieData);
+        pasReqInfo[i].pszHTTPCookieData = NULL;
+
         pasReqInfo[i].curl_handle = NULL;
     }
 }
@@ -214,6 +222,30 @@ static size_t msHTTPWriteFct(void *buffer, size_t size, size_t nmemb,
     return fwrite(buffer, size, nmemb, psReq->fp);
 }
 
+/**********************************************************************
+ *                          msGetCURLAuthType()
+ *
+ * Returns the equivalent CURL CURLAUTH_ constant given a
+ * MS_HTTP_AUTH_TYPE, or CURLAUTH_BASIC if no match is found.
+ **********************************************************************/
+long msGetCURLAuthType(enum MS_HTTP_AUTH_TYPE authType)
+{
+    switch (authType)
+    {
+        case MS_BASIC:
+            return CURLAUTH_BASIC;
+        case MS_DIGEST:
+            return CURLAUTH_DIGEST;
+        case MS_NTLM:
+            return CURLAUTH_NTLM;
+        case MS_ANY:
+            return CURLAUTH_ANY;
+        case MS_ANYSAFE:
+            return CURLAUTH_ANYSAFE;
+        default:
+            return CURLAUTH_BASIC;    
+    }
+}
 
 /**********************************************************************
  *                          msHTTPExecuteRequests()
@@ -235,10 +267,14 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
     CURLM   *multi_handle;
     CURLMsg *curl_msg;
     char     debug = MS_FALSE;
+    const char *pszCurlCABundle = NULL;
 
     if (numRequests == 0)
         return MS_SUCCESS;  /* Nothing to do */
 
+    if (!gbCurlInitialized)
+        msHTTPInit();
+  
     /* Establish the timeout (seconds) for how long we are going to wait 
      * for a response. 
      * We use the longest timeout value in the array of requests 
@@ -256,9 +292,17 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
     if (nTimeout <= 0)
         nTimeout = 30;
 
+    /* Check if we've got a CURL_CA_BUNDLE env. var.
+     * If set then the value is the full path to the ca-bundle.crt file
+     * e.g. CURL_CA_BUNDLE=/usr/local/share/curl/curl-ca-bundle.crt
+     */
+    pszCurlCABundle = getenv("CURL_CA_BUNDLE");
+
     if (debug)
     {
         msDebug("HTTP: Starting to prepare HTTP requests.\n");
+        if (pszCurlCABundle)
+            msDebug("Using CURL_CA_BUNDLE=%s\n", pszCurlCABundle);
     }
 
     /* Alloc a curl-multi handle, and add a curl-easy handle to it for each
@@ -360,6 +404,82 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
         /* Set timeout.*/
         curl_easy_setopt(http_handle, CURLOPT_TIMEOUT, nTimeout );
 
+        /* Pass CURL_CA_BUNDLE if set */
+        if (pszCurlCABundle)
+            curl_easy_setopt(http_handle, CURLOPT_CAINFO, pszCurlCABundle );
+
+        /* Set proxying settings */
+        if (pasReqInfo[i].pszProxyAddress != NULL
+            && strlen(pasReqInfo[i].pszProxyAddress) > 0)
+        {
+            long    nProxyType     = CURLPROXY_HTTP;
+            
+            curl_easy_setopt(http_handle, CURLOPT_PROXY,
+                             pasReqInfo[i].pszProxyAddress);
+            
+            if (pasReqInfo[i].nProxyPort > 0
+                && pasReqInfo[i].nProxyPort < 65535)
+            {
+                curl_easy_setopt(http_handle, CURLOPT_PROXYPORT,
+                                 pasReqInfo[i].nProxyPort);
+            }
+            
+            switch (pasReqInfo[i].eProxyType)
+            {
+                case MS_HTTP:
+                    nProxyType = CURLPROXY_HTTP;
+                    break;
+                case MS_SOCKS5:
+                    nProxyType = CURLPROXY_SOCKS5;
+                    break;
+            }
+            curl_easy_setopt(http_handle, CURLOPT_PROXYTYPE, nProxyType);
+            
+            /* If there is proxy authentication information, set it */
+            if (pasReqInfo[i].pszProxyUsername != NULL
+                && pasReqInfo[i].pszProxyPassword != NULL
+                && strlen(pasReqInfo[i].pszProxyUsername) > 0
+                && strlen(pasReqInfo[i].pszProxyPassword) > 0)
+            {
+                char    szUsernamePasswd[128];    
+#ifdef CURLOPT_PROXYAUTH
+                long    nProxyAuthType = CURLAUTH_BASIC;
+                /* CURLOPT_PROXYAUTH available only in Curl 7.10.7 and up */
+                nProxyAuthType = msGetCURLAuthType(pasReqInfo[i].eProxyAuthType);
+                curl_easy_setopt(http_handle, CURLOPT_PROXYAUTH, nProxyAuthType);
+#else
+                /* We log an error but don't abort processing */
+                msSetError(MS_HTTPERR, "CURLOPT_PROXYAUTH not supported. Requires Curl 7.10.7 and up. *_proxy_auth_type setting ignored.", 
+                           "msHTTPExecuteRequests()");
+#endif /* CURLOPT_PROXYAUTH */ 
+                
+                snprintf(szUsernamePasswd, 127, "%s:%s",
+                         pasReqInfo[i].pszProxyUsername,
+                         pasReqInfo[i].pszProxyPassword);
+                curl_easy_setopt(http_handle, CURLOPT_PROXYUSERPWD,
+                                 szUsernamePasswd);
+            }
+        }
+        
+        /* Set HTTP Authentication settings */
+        if (pasReqInfo[i].pszHttpUsername != NULL
+            && pasReqInfo[i].pszHttpPassword != NULL
+            && strlen(pasReqInfo[i].pszHttpUsername) > 0
+            && strlen(pasReqInfo[i].pszHttpPassword) > 0)
+        {
+            char    szUsernamePasswd[128];
+            long    nHttpAuthType = CURLAUTH_BASIC;
+            
+            snprintf(szUsernamePasswd, 127, "%s:%s",
+                     pasReqInfo[i].pszHttpUsername,
+                     pasReqInfo[i].pszHttpPassword);
+            curl_easy_setopt(http_handle, CURLOPT_USERPWD,
+                             szUsernamePasswd);
+                                 
+            nHttpAuthType = msGetCURLAuthType(pasReqInfo[i].eHttpAuthType);
+            curl_easy_setopt(http_handle, CURLOPT_HTTPAUTH, nHttpAuthType);
+        }
+
         /* NOSIGNAL should be set to true for timeout to work in multithread
          * environments on Unix, requires libcurl 7.10 or more recent.
          * (this force avoiding the use of sgnal handlers)
@@ -405,6 +525,29 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
             curl_easy_setopt(http_handle, CURLOPT_HTTPHEADER, headers);
             /* curl_slist_free_all(headers); */ /* free the header list */
         }
+
+        /* Added by RFC-42 HTTP Cookie Forwarding */
+        if(pasReqInfo[i].pszHTTPCookieData != NULL)
+        {
+            /* Check if there's no end of line in the Cookie string */
+            /* This could break the HTTP Header */
+            int nPos;
+
+            for(nPos=0; nPos<strlen(pasReqInfo[i].pszHTTPCookieData); nPos++)
+            {
+                if(pasReqInfo[i].pszHTTPCookieData[nPos] == '\n')
+                {
+                    msSetError(MS_HTTPERR, "Can't open output file %s.", 
+                       "msHTTPExecuteRequests()", pasReqInfo[i].pszOutputFile);
+                    return(MS_FAILURE);
+                }
+            }
+
+            /* Set the Curl option to send Cookie */
+            curl_easy_setopt(http_handle, CURLOPT_COOKIE, 
+                             pasReqInfo[i].pszHTTPCookieData);
+        }
+
         /* Add to multi handle */
         curl_multi_add_handle(multi_handle, http_handle);
 
