@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapimagemap.c 8482 2009-01-29 23:34:29Z tamas $
  *
  * Project:  MapServer
  * Purpose:  Implements imagemap outputformat support.
@@ -37,7 +38,7 @@
 #include <io.h>
 #endif
 
-MS_CVSID("$Id: mapimagemap.c 7425 2008-03-03 20:05:24Z dmorissette $")
+MS_CVSID("$Id: mapimagemap.c 8482 2009-01-29 23:34:29Z tamas $")
 
 #define MYDEBUG 0
 #define DEBUG_IF if (MYDEBUG > 2) 
@@ -1847,9 +1848,9 @@ int msDrawLabelCacheIM(imageObj* img, mapObj *map)
       continue; /* not an error, just don't want to do anything */
 
     if(cachePtr->label.type == MS_TRUETYPE)
-      cachePtr->label.size = (int)(cachePtr->label.size*layerPtr->scalefactor);
+      cachePtr->label.size = (cachePtr->label.size*layerPtr->scalefactor);
 
-    if(msGetLabelSize(cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor, MS_TRUE) == -1)
+    if(msGetLabelSize(img,cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor, MS_TRUE,NULL) == -1)
       return(-1);
 
     if(labelPtr->autominfeaturesize && ((r.maxx-r.minx) > cachePtr->featuresize))
