@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: maplabel.c 8904 2009-04-14 13:29:10Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  Labeling Implementation.
@@ -37,12 +38,279 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: maplabel.c 7332 2008-01-30 22:17:23Z tamas $")
+MS_CVSID("$Id: maplabel.c 8904 2009-04-14 13:29:10Z dmorissette $")
 
-int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int tileindex, pointObj *point, labelPathObj *labelpath, char *string, double featuresize, labelObj *label )
+
+/**
+ * replace wrap characters with \n , respecting maximal line length.
+ *
+ * returns a pointer to the newly allocated text. memory is controlled
+ * inside this function, so the caller MUST use the pointer returned by
+ * the function:
+ * text = msWrapText(label,text);
+ *
+ * TODO/FIXME: function will produce erroneous/crashing? results
+ * if the wrap character is encoded with multiple bytes
+ *
+ * see http://mapserver.gis.umn.edu/development/rfc/ms-rfc-40/
+ * for a summary of how wrap/maxlength interact on the result
+ * of the text transformation
+ */
+char *msWrapText(labelObj *label, char *text) {
+    char wrap;
+    int maxlength;
+    if(!text) /*not an error if no text*/
+        return text;
+    wrap = label->wrap;
+    maxlength = label->maxlength;
+    if(maxlength == 0) {
+        if(wrap!='\0') {
+            /* if maxlength = 0 *and* a wrap character was specified,
+             * replace all wrap characters by \n
+             * this is the traditional meaning of the wrap character
+             */
+            msReplaceChar(text, wrap, '\n');
+        }
+        /* if neither maxlength, nor wrap were specified,
+         * don't transform this text */
+        return text;
+    } else if(maxlength>0) {
+        if(wrap!='\0') {
+            /* split input text at the wrap character, only if
+             * the current line length is over maxlength */
+
+            /* TODO: check if the wrap character is a valid byte
+             * inside a multibyte utf8 glyph. if so, the msCountChars
+             * will return an erroneous value */
+            int numwrapchars = msCountChars(text,wrap);
+
+            if(numwrapchars > 0) {
+                if(label->encoding) {
+                    /* we have to use utf decoding functions here, so as not to
+                     * split a text line on a multibyte character */
+                    
+                    int num_cur_glyph_on_line = 0; /*count for the number of glyphs
+                                                     on the current line*/
+                    char *textptr = text;
+                    char glyph[11]; /*storage for unicode fetching function*/
+                    int glyphlen = 0; /*size of current glyph in bytes*/
+                    while((glyphlen = msGetNextGlyph((const char**)&textptr,glyph))>0) {
+                        num_cur_glyph_on_line++;
+                        if(*glyph == wrap && num_cur_glyph_on_line>=(maxlength)) {
+                            /*FIXME (if wrap becomes something other than char):*/
+                            *(textptr-1)='\n'; /*replace wrap char with a \n*/
+                            num_cur_glyph_on_line=0; /*reset count*/
+                        }
+                    }
+                } else {
+                    int cur_char_on_line = 0;
+                    char *textptr = text;
+                    while(*textptr != 0) {
+                        cur_char_on_line++;
+                        if(*textptr == wrap && cur_char_on_line>=maxlength) {
+                            *textptr='\n'; /*replace wrap char with a \n*/
+                            cur_char_on_line=0; /*reset count*/
+                        }
+                        textptr++;
+                    }
+                }
+                return text;
+            } else {
+                /*there are no characters available for wrapping*/
+                return text;
+            }
+        } else {
+            /* if no wrap character was specified, but a maxlength was,
+             * don't draw this label if it is longer than the specified maxlength*/
+            if(msGetNumGlyphs(text)>maxlength) {
+                free(text);
+                return NULL;
+            } else {
+                return text;
+            }
+        }
+    } else { /* negative maxlength: we split lines unconditionally, i.e. without
+    loooking for a wrap character*/
+        int numglyphs,numlines;
+        maxlength = -maxlength; /* use a positive value*/
+        numglyphs = msGetNumGlyphs(text);
+        numlines = numglyphs / maxlength; /*count total number of lines needed
+                                            after splitting*/
+        if(numlines>1) {
+            char *newtext = malloc(strlen(text)+numlines+1);
+            char *newtextptr = newtext;
+            char *textptr = text;
+            int glyphlen = 0, num_cur_glyph = 0;
+            while((glyphlen = msGetNextGlyph((const char**)&textptr,newtextptr))>0) {
+                num_cur_glyph++;
+                newtextptr += glyphlen;
+                if(num_cur_glyph%maxlength == 0 && num_cur_glyph != numglyphs) {
+                    /*we're at a split location, insert a newline*/
+                    *newtextptr = '\n';
+                    newtextptr++;
+                }
+            }
+            free(text);
+            return newtext;
+        } else {
+            /*no splitting needed, return the original*/
+            return text;
+        }
+    }
+}
+
+char *msAlignText(mapObj *map, imageObj *image, labelObj *label, char *text) {
+    double spacewidth=0.0; /*size of a single space, in fractional pixels*/
+    int numlines;
+    char **textlines,*newtext,*newtextptr;
+    int *textlinelengths,*numspacesforpadding;
+    int numspacestoadd,maxlinelength,i;
+    rectObj label_rect;
+    if(!msCountChars(text,'\n'))
+        return text; /*only one line*/
+    
+    /*split text into individual lines
+     * TODO: check if splitting on \n is utf8 safe*/
+    textlines = msStringSplit(text,'\n',&numlines);
+    
+    /*
+     * label->space_size_10 contains a cache for the horizontal size of a single
+     * 'space' character, at size 10 for the current label
+     * FIXME: in case of attribute binding for the FONT of the label, this cache will
+     * be wrong for labels where the attributed font is different than the first
+     * computed font. This shouldn't happen too often, and hopefully the size of a 
+     * space character shouldn't vary too much between different fonts*/
+    if(label->space_size_10 == 0.0) {
+        /*if the cache hasn't been initialized yet, or with pixmap fonts*/
+        double size=0; /*initialize this here to avoid compiler warning*/
+        if(label->type == MS_TRUETYPE) {
+            size = label->size; /*keep a copy of the original size*/
+            label->size=10;
+        }
+        /* compute the size of 16 adjacent spaces. we can't do this for just one space,
+         * as the labelSize computing functions return integer bounding boxes. we assume
+         * that the integer rounding for such a number of spaces will be negligeable
+         * compared to the actual size of thoses spaces */ 
+        if(msGetLabelSize(image, "                ", label,&label_rect, 
+                    &map->fontset, 1.0, MS_FALSE,NULL)==-1) { 
+            /*error computing label size, we can't continue*/
+
+            /*free the previously allocated split text*/
+            while(numlines--)
+                free(textlines[numlines]);
+            free(textlines);
+            return text;
+        }
+
+        /* this is the size of a single space character. for truetype fonts,
+         * it's the size of a 10pt space. For pixmap fonts, it's the size
+         * for the current label */
+        spacewidth = (label_rect.maxx-label_rect.minx)/16.0;
+        if(label->type == MS_TRUETYPE) {
+            label->size = size; /*restore original label size*/
+            label->space_size_10=spacewidth; /*cache the computed size*/
+
+            /*size of a space for current label size*/
+            spacewidth = spacewidth * (double)label->size/10.0;
+        }
+    } else {
+        spacewidth = label->space_size_10 * (double)label->size/10.0;
+    }
+   
+    
+    /*length in pixels of each line*/
+    textlinelengths = (int*)malloc(numlines*sizeof(int));
+    
+    /*number of spaces that need to be added to each line*/
+    numspacesforpadding = (int*)malloc(numlines*sizeof(int));
+    
+    /*total number of spaces that need to be added*/
+    numspacestoadd=0;
+
+    /*length in pixels of the longest line*/
+    maxlinelength=0;
+    for(i=0;i<numlines;i++) {
+        msGetLabelSize(image, textlines[i], label, &label_rect, &map->fontset, 1.0, MS_FALSE, NULL);
+        textlinelengths[i] = label_rect.maxx-label_rect.minx;
+        if(maxlinelength<textlinelengths[i])
+            maxlinelength=textlinelengths[i];
+    }
+    for(i=0;i<numlines;i++) {
+        /* number of spaces to add so the current line is
+         * as long as the longest line */
+        double nfracspaces = (maxlinelength - textlinelengths[i])/spacewidth;
+
+        if(label->align == MS_ALIGN_CENTER) {
+            numspacesforpadding[i]=MS_NINT(nfracspaces/2.0);
+        } else {
+           if(label->align == MS_ALIGN_RIGHT) {
+               numspacesforpadding[i]=MS_NINT(nfracspaces);
+           }
+        }
+        numspacestoadd+=numspacesforpadding[i];
+    }
+
+    /*allocate new text with room for the additional spaces needed*/
+    newtext = (char*)malloc(strlen(text)+1+numspacestoadd);
+    newtextptr=newtext;
+    for(i=0;i<numlines;i++) {
+        int j;
+        /*padd beginning of line with needed spaces*/
+        for(j=0;j<numspacesforpadding[i];j++) {
+            *(newtextptr++)=' ';
+        }
+        /*copy original line*/
+        strcpy(newtextptr,textlines[i]);
+        /*place pointer at the char right after the current line*/
+        newtextptr+=strlen(textlines[i])+1;
+        if(i!=numlines-1) {
+            /*put the \n back in (was taken away by msStringSplit)*/
+            *(newtextptr-1)='\n';
+        }
+    }
+    /*free the original text*/
+    free(text);
+    for(i=0;i<numlines;i++) {
+        free(textlines[i]);
+    }
+    free(textlines);
+    free(textlinelengths);
+    free(numspacesforpadding);
+
+    /*return the aligned text. note that the terminating \0 was added by the last
+     * call to strcpy */
+    return newtext;
+}
+
+
+/*
+ * this function applies the label encoding and wrap parameters
+ * to the supplied text
+ * Note: it is the caller's responsibility to free the returned 
+ * char array
+ */
+char *msTransformLabelText(mapObj *map, imageObj* image,labelObj *label, char *text)
+{
+    char *newtext = text;
+    if(label->encoding)
+        newtext = msGetEncodedString(text, label->encoding);
+    else
+        newtext=strdup(text);
+    
+    if(newtext && (label->wrap!='\0' || label->maxlength!=0)) {
+        newtext = msWrapText(label, newtext);
+    }
+
+    if(newtext && label->align!=MS_ALIGN_LEFT ) {
+        newtext = msAlignText(map, image,label, newtext);
+    }
+
+    return newtext;
+}
+
+int msAddLabel(mapObj *map, int layerindex, int classindex, shapeObj *shape, pointObj *point, labelPathObj *labelpath, char *string, double featuresize, labelObj *label )
 {
   int i;
-  char wrap[2];
   labelCacheSlotObj *cacheslot;
 
   labelCacheMemberObj *cachePtr=NULL;
@@ -55,13 +323,20 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
   classPtr = GET_LAYER(map, layerindex)->class[classindex];
 
   if( label == NULL )
-      label = &(classPtr->label);
+    label = &(classPtr->label);
+
+  if(map->scaledenom > 0) {
+    if((label->maxscaledenom != -1) && (map->scaledenom >= label->maxscaledenom))
+      return(MS_SUCCESS);
+    if((label->minscaledenom != -1) && (map->scaledenom < label->minscaledenom))
+      return(MS_SUCCESS);
+  }
 
   /* Validate label priority value and get ref on label cache for it */
   if (label->priority < 1)
-      label->priority = 1;
+    label->priority = 1;
   else if (label->priority > MS_MAX_LABEL_PRIORITY)
-      label->priority = MS_MAX_LABEL_PRIORITY;
+    label->priority = MS_MAX_LABEL_PRIORITY;
 
   cacheslot = &(map->labelcache.slots[label->priority-1]);
 
@@ -78,17 +353,22 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
 
   cachePtr->layerindex = layerindex; /* so we can get back to this *raw* data if necessary */
   cachePtr->classindex = classindex;
-  cachePtr->tileindex = tileindex;
-  cachePtr->shapeindex = shapeindex;
+
+  if(shape) {
+    cachePtr->tileindex = shape->tileindex;
+    cachePtr->shapeindex = shape->index;
+    cachePtr->shapetype = shape->type;
+  } else {
+    cachePtr->tileindex = cachePtr->shapeindex = -1;    
+    cachePtr->shapetype = MS_SHAPE_POINT;
+  }
 
   /* Store the label point or the label path (Bug #1620) */
   if ( point ) {
-
-  cachePtr->point = *point; /* the actual label point */
-  cachePtr->point.x = MS_NINT(cachePtr->point.x);
-  cachePtr->point.y = MS_NINT(cachePtr->point.y);
+    cachePtr->point = *point; /* the actual label point */
+    cachePtr->point.x = MS_NINT(cachePtr->point.x);
+    cachePtr->point.y = MS_NINT(cachePtr->point.y);
     cachePtr->labelpath = NULL;
-    
   } else if ( labelpath ) {
     int i;
     cachePtr->labelpath = labelpath;
@@ -100,13 +380,6 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
 
   cachePtr->text = strdup(string); /* the actual text */
 
-  /* GD/Freetype recognizes \r\n as a true line wrap so we must turn the wrap character into that pattern */
-  if(label->type != MS_BITMAP && label->wrap != '\0') {
-    wrap[0] = label->wrap;
-    wrap[1] = '\0';
-    cachePtr->text = msReplaceSubstring(cachePtr->text, wrap, "\r\n");
-  }
-
   /* copy the styles (only if there is an accompanying marker)
    * We cannot simply keeep refs because the rendering code alters some members of the style objects
    */
@@ -154,14 +427,13 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
 
     /* TO DO: at the moment only checks the bottom style, perhaps should check all of them */
     /* #2347: after RFC-24 classPtr->styles could be NULL so we check it */
-    if (classPtr->styles != NULL) {
-	    if(msGetMarkerSize(&map->symbolset, classPtr->styles[0], &w, &h, layerPtr->scalefactor) != MS_SUCCESS)
-    	  return(MS_FAILURE);
+    if(classPtr->styles != NULL) {
+      if(msGetMarkerSize(&map->symbolset, classPtr->styles[0], &w, &h, layerPtr->scalefactor) != MS_SUCCESS) 
+        return(MS_FAILURE);
     } else {
-    	msSetError(MS_MISCERR, "msAddLabel error: missing style definition for layer '%s'", "msAddLabel()", layerPtr->name);
-		return(MS_FAILURE);
-	}
-
+      msSetError(MS_MISCERR, "msAddLabel error: missing style definition for layer '%s'", "msAddLabel()", layerPtr->name);
+      return(MS_FAILURE);
+    }
     rect.minx = MS_NINT(point->x - .5 * w);
     rect.miny = MS_NINT(point->y - .5 * h);
     rect.maxx = rect.minx + (w-1);
@@ -180,7 +452,6 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
   return(MS_SUCCESS);
 }
 
-
 /* msTestLabelCacheCollisions()
 **
 ** Compares current label against labels already drawn and markers from cache and discards it
@@ -342,7 +613,7 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
 
     sscanf(buffer,"%s %s", alias,  file1);
 
-    if (!file1 || !alias || (strlen(file1) <= 0))
+    if (!(*file1) || !(*alias) || (strlen(file1) <= 0))
       continue;
     
     bFullPath = 0;
@@ -384,6 +655,99 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
 #endif
 }
 
+int msGetTruetypeTextBBox(imageObj *img, char *font, double size, char *string, rectObj *rect, double **advances) {
+#ifdef USE_GD_FT
+#ifdef USE_AGG
+    if(img!=NULL && MS_RENDERER_AGG(img->format)) {
+        return msGetTruetypeTextBBoxAGG(img,font,size,string,rect,advances); 
+    } else
+#endif
+    {
+        int bbox[8];
+        char *error;
+        if(advances) {
+#if defined (GD_HAS_FTEX_XSHOW)
+            char *s;
+            int k;
+            gdFTStringExtra strex;
+            strex.flags = gdFTEX_XSHOW;
+            error = gdImageStringFTEx(NULL, bbox, 0, font, size, 0, 0, 0, string, &strex);
+            if(error) {
+                msSetError(MS_TTFERR, error, "gdImageStringFTEx()");
+                return(MS_FAILURE);
+            }
+
+            *advances = (double*)malloc( strlen(string) * sizeof(double) );
+            s = strex.xshow;
+            k = 0;
+            while ( *s && k < strlen(string) ) {
+                (*advances)[k++] = atof(s);      
+                while ( *s && *s != ' ')
+                    s++;
+                if ( *s == ' ' )
+                    s++;
+            }
+
+            gdFree(strex.xshow); /* done with character advances */
+
+            rect->minx = bbox[0];
+            rect->miny = bbox[5];
+            rect->maxx = bbox[2];
+            rect->maxy = bbox[1];
+            return MS_SUCCESS;
+#else
+            msSetError(MS_TTFERR, "gdImageStringFTEx support is not available or is not current enough (requires 2.0.29 or higher).", "msGetTruetypeTextBBox()");
+            return(-1);
+#endif
+        } else {
+            error = gdImageStringFT(NULL, bbox, 0, font, size, 0, 0, 0, string);
+            if(error) {
+                msSetError(MS_TTFERR, error, "msGetTruetypeTextBBox()");
+                return(MS_FAILURE);
+            }
+
+            rect->minx = bbox[0];
+            rect->miny = bbox[5];
+            rect->maxx = bbox[2];
+            rect->maxy = bbox[1];
+            return MS_SUCCESS;
+        }
+    }
+#else
+    /*shouldn't happen*/
+    return(MS_FAILURE);
+#endif
+}
+
+int msGetRasterTextBBox(imageObj *img, int size, char *string, rectObj *rect) {
+#ifdef USE_AGG
+    if(img!=NULL && MS_RENDERER_AGG(img->format)) {
+        return msGetRasterTextBBoxAGG(img,size,string,rect); 
+    } else
+#endif
+    {
+        gdFontPtr fontPtr;
+        char **token=NULL;
+        int t, num_tokens, max_token_length=0;
+        if((fontPtr = msGetBitmapFont(size)) == NULL)
+            return(-1);
+
+        if((token = msStringSplit(string, '\n', &(num_tokens))) == NULL)
+            return(0);
+
+        for(t=0; t<num_tokens; t++) /* what's the longest token */
+            max_token_length = MS_MAX(max_token_length, (int) strlen(token[t]));
+
+        rect->minx = 0;
+        rect->miny = -(fontPtr->h * num_tokens);
+        rect->maxx = fontPtr->w * max_token_length;
+        rect->maxy = 0;
+
+        msFreeCharArray(token, num_tokens);
+        return MS_SUCCESS;
+    }
+}
+
 /*
 ** Note: All these routines assume a reference point at the LL corner of the text. GD's
 ** bitmapped fonts use UL and this is compensated for. Note the rect is relative to the
@@ -391,16 +755,14 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
 */
 
 /* assumes an angle of 0 regardless of what's in the label object */
-int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fontset, double scalefactor, int adjustBaseline)
+int msGetLabelSize(imageObj *img, char *string, labelObj *label, rectObj *rect, fontSetObj *fontset, double scalefactor, int adjustBaseline, double **advances)
 {
-  int size;
-
+  double size;
   if(label->type == MS_TRUETYPE) {
 #ifdef USE_GD_FT
-    int bbox[8];
-    char *error=NULL, *font=NULL;
+    char *font=NULL;
 
-    size = MS_NINT(label->size*scalefactor);
+    size = label->size*scalefactor;
     size = MS_MAX(size, label->minsize);
     size = MS_MIN(size, label->maxsize);
 
@@ -413,125 +775,36 @@ int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fon
       return(-1);
     }
 
-
-    error = gdImageStringFT(NULL, bbox, 0, font, size, 0, 0, 0, string);
-    if(error) {
-      msSetError(MS_TTFERR, error, "msGetLabelSize()");
-      return(-1);
-    }
-
-    rect->minx = bbox[0];
-    rect->miny = bbox[5];
-    rect->maxx = bbox[2];
-    rect->maxy = bbox[1];
+    if(msGetTruetypeTextBBox(img,font,size,string,rect,advances)!=MS_SUCCESS)
+        return -1;
 
     /* bug 1449 fix (adjust baseline) */
     if(adjustBaseline) {
-      label->offsety += MS_NINT(((bbox[5] + bbox[1]) + size) / 2);
-      label->offsetx += MS_NINT(bbox[0] / 2);
+      int nNewlines = msCountChars(string,'\n');
+      if(!nNewlines) {
+        label->offsety += MS_NINT(((rect->miny + rect->maxy) + size) / 2);
+        label->offsetx += MS_NINT(rect->minx / 2);
+      }
+      else {
+        rectObj rect2; /*bbox of first line only*/
+        char* firstLine = msGetFirstLine(string);
+        msGetTruetypeTextBBox(img,font,size,firstLine,&rect2,NULL);
+        label->offsety += MS_NINT(((rect2.miny+rect2.maxy) + size) / 2);
+        label->offsetx += MS_NINT(rect2.minx / 2);
+        free(firstLine);
+      }
     }
+    return 0;
 #else
     msSetError(MS_TTFERR, "TrueType font support is not available.", "msGetLabelSize()");
     return(-1);
 #endif
   } else { /* MS_BITMAP font */
-    gdFontPtr fontPtr;
-    char **token=NULL;
-    int t, num_tokens, max_token_length=0;
-
-    if((fontPtr = msGetBitmapFont(label->size)) == NULL)
-      return(-1);
-
-    if(label->wrap != '\0') {
-      if((token = msStringSplit(string, label->wrap, &(num_tokens))) == NULL)
-	return(0);
-
-      for(t=0; t<num_tokens; t++) /* what's the longest token */
-	max_token_length = MS_MAX(max_token_length, (int) strlen(token[t]));
-
-      rect->minx = 0;
-      rect->miny = -(fontPtr->h * num_tokens);
-      rect->maxx = fontPtr->w * max_token_length;
-      rect->maxy = 0;
-
-      msFreeCharArray(token, num_tokens);
-    } else {
-      rect->minx = 0;
-      rect->miny = -fontPtr->h;
-      rect->maxx = fontPtr->w * strlen(string);
-      rect->maxy = 0;
-    }
-  }
+    msGetRasterTextBBox(img,MS_NINT(label->size),string,rect);
+ }
   return(0);
 }
 
-/*
- * Return the size of the label, plus an array of individual character
- * offsets, as calculated by gdImageStringFTEx().  The callee is
- * responsible for freeing *offsets.
- */
-int msGetLabelSizeEx(char *string, labelObj *label, rectObj *rect, fontSetObj *fontset, double scalefactor, int adjustBaseline, double **offsets) 
-{
-#if defined (USE_GD_FT) && defined (GD_HAS_FTEX_XSHOW)
-    int size;
-    char *s;
-    int k;
-    int bbox[8];
-    char *error=NULL, *font=NULL;
-    gdFTStringExtra strex;
-    
-    size = MS_NINT(label->size*scalefactor);
-    size = MS_MAX(size, label->minsize);
-    size = MS_MIN(size, label->maxsize);
-
-    font = msLookupHashTable(&(fontset->fonts), label->font);
-    if(!font) {
-      if(label->font) 
-        msSetError(MS_TTFERR, "Requested font (%s) not found.", "msGetLabelSizeEx()", label->font);
-      else
-        msSetError(MS_TTFERR, "Requested font (NULL) not found.", "msGetLabelSizeEx()");
-      return(-1);
-    }
-
-    strex.flags = gdFTEX_XSHOW;
-    error = gdImageStringFTEx(NULL, bbox, 0, font, size, 0, 0, 0, string, &strex);
-    if(error) {
-      msSetError(MS_TTFERR, error, "msGetLabelSizeEx()");
-      return(-1);
-    }
-
-    *offsets = (double*)malloc( strlen(string) * sizeof(double) );
-    
-    s = strex.xshow;
-    k = 0;
-    while ( *s && k < strlen(string) ) {
-      (*offsets)[k++] = atof(s);      
-      while ( *s && *s != ' ')
-        s++;
-      if ( *s == ' ' )
-        s++;
-    }
-
-    gdFree(strex.xshow); /* done with character advances */
-
-    rect->minx = bbox[0];
-    rect->miny = bbox[5];
-    rect->maxx = bbox[2];
-    rect->maxy = bbox[1];
-
-    /* bug 1449 fix (adjust baseline) */
-    if(adjustBaseline) {
-      label->offsety += MS_NINT(((bbox[5] + bbox[1]) + size) / 2);
-      label->offsetx += MS_NINT(bbox[0] / 2);
-    }
-
-    return MS_SUCCESS;
-#else
-    msSetError(MS_TTFERR, "TrueType font support is not available or is not current enough (requires 2.0.29 or higher).", "msGetLabelSizeEx()");
-    return(-1);
-#endif
-}
-
 gdFontPtr msGetBitmapFont(int size)
 {
   switch(size) { /* set the font to use */
@@ -578,12 +851,14 @@ gdFontPtr msGetBitmapFont(int size)
 
 #define MARKER_SLOP 2
 
-/* static pointObj get_metrics(pointObj *p, int position, rectObj rect, int ox, int oy, double angle, int buffer, shapeObj *poly) */
-pointObj get_metrics(pointObj *p, int position, rectObj rect, int ox, int oy, double angle, int buffer, shapeObj *poly)
+/*pointObj get_metrics_line(pointObj *p, int position, rectObj rect, int ox, int oy, double angle, int buffer, lineObj *poly)
+ * called by get_metrics and drawLabelCache 
+ * the poly lineObj MUST have at least 5 points allocated in poly->point
+ * it should also probably have poly->numpoints set to 5 */
+pointObj get_metrics_line(pointObj *p, int position, rectObj rect, int ox, int oy, double angle, int buffer, lineObj *poly)
 {
   pointObj q;
   double x1=0, y1=0, x2=0, y2=0;
-  lineObj line={0,NULL};
   double sin_a,cos_a;
   double w, h, x, y;
 
@@ -638,40 +913,54 @@ pointObj get_metrics(pointObj *p, int position, rectObj rect, int ox, int oy, do
   q.y = p->y - MS_NINT(x * sin_a + (y) * cos_a);
 
   if(poly) {
-    line.point = (pointObj *)malloc(sizeof(pointObj)*5);
-    line.numpoints = 5;
+      /*
+       * here we should/could have a test asserting that the poly lineObj 
+       * has at least 5 points available.
+       */
 
     x2 = x1 - buffer; /* ll */
     y2 = y1 + buffer;
-    line.point[0].x = p->x + MS_NINT(x2 * cos_a - (-y2) * sin_a);
-    line.point[0].y = p->y - MS_NINT(x2 * sin_a + (-y2) * cos_a);
+    poly->point[0].x = p->x + MS_NINT(x2 * cos_a - (-y2) * sin_a);
+    poly->point[0].y = p->y - MS_NINT(x2 * sin_a + (-y2) * cos_a);
 
     x2 = x1 - buffer; /* ul */
     y2 = y1 - h - buffer;
-    line.point[1].x = p->x + MS_NINT(x2 * cos_a - (-y2) * sin_a);
-    line.point[1].y = p->y - MS_NINT(x2 * sin_a + (-y2) * cos_a);
+    poly->point[1].x = p->x + MS_NINT(x2 * cos_a - (-y2) * sin_a);
+    poly->point[1].y = p->y - MS_NINT(x2 * sin_a + (-y2) * cos_a);
 
     x2 = x1 + w + buffer; /* ur */
     y2 = y1 - h - buffer;
-    line.point[2].x = p->x + MS_NINT(x2 * cos_a - (-y2) * sin_a);
-    line.point[2].y = p->y - MS_NINT(x2 * sin_a + (-y2) * cos_a);
+    poly->point[2].x = p->x + MS_NINT(x2 * cos_a - (-y2) * sin_a);
+    poly->point[2].y = p->y - MS_NINT(x2 * sin_a + (-y2) * cos_a);
 
     x2 = x1 + w + buffer; /* lr */
     y2 = y1 + buffer;
-    line.point[3].x = p->x + MS_NINT(x2 * cos_a - (-y2) * sin_a);
-    line.point[3].y = p->y - MS_NINT(x2 * sin_a + (-y2) * cos_a);
+    poly->point[3].x = p->x + MS_NINT(x2 * cos_a - (-y2) * sin_a);
+    poly->point[3].y = p->y - MS_NINT(x2 * sin_a + (-y2) * cos_a);
 
-    line.point[4].x = line.point[0].x;
-    line.point[4].y = line.point[0].y;
-
-    msAddLine(poly, &line);
-    msComputeBounds(poly); /* make sure the polygon has a bounding box for fast collision testing */
-    free(line.point);
+    poly->point[4].x = poly->point[0].x;
+    poly->point[4].y = poly->point[0].y;
   }
 
   return(q);
 }
 
+
+/* static pointObj get_metrics(pointObj *p, int position, rectObj rect, int ox, int oy, double angle, int buffer, shapeObj *poly) */
+pointObj get_metrics(pointObj *p, int position, rectObj rect, int ox, int oy, double angle, int buffer, shapeObj *poly) {
+    lineObj newline;
+    pointObj newpoints[5];
+    pointObj rp;
+    newline.numpoints=5;
+    newline.point=newpoints;
+    rp = get_metrics_line(p, position, rect, ox,oy, angle, buffer, &newline);
+    if(poly) {
+        msAddLine(poly,&newline);
+        msComputeBounds(poly);
+    }
+    return rp;
+}
+
 /*
 ** is a label completely in the image (excluding label buffer)
 */
@@ -774,7 +1063,7 @@ int msImageTruetypePolyline(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p
   if(size*scalefactor > style->maxsize) scalefactor = (float)style->maxsize/(float)size;
   if(size*scalefactor < style->minsize) scalefactor = (float)style->minsize/(float)size;
   gap = MS_ABS(symbol->gap)* (int) scalefactor;
-  label.size = (int) (size * scalefactor);
+  label.size = (size * scalefactor);
   /* label.minsize = style->minsize; */
   /* label.maxsize = style->maxsize; */
 
@@ -782,7 +1071,7 @@ int msImageTruetypePolyline(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p
   label.outlinecolor = style->outlinecolor;
   label.antialias = symbol->antialias;
   
-  if(msGetLabelSize(symbol->character, &label, &label_rect, symbolset->fontset, scalefactor, MS_FALSE) == -1)
+  if(msGetLabelSize(NULL,symbol->character, &label, &label_rect, symbolset->fontset, scalefactor, MS_FALSE,NULL) == -1)
     return(-1);
 
   label_width = (int) label_rect.maxx - (int) label_rect.minx;
