@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: maplayer.c 8901 2009-04-12 16:49:02Z tamas $
  *
  * Project:  MapServer
  * Purpose:  Implementation of most layerObj functions.
@@ -31,7 +32,7 @@
 #include "mapogcfilter.h"
 
 #include <assert.h>
-MS_CVSID("$Id: maplayer.c 6571 2007-08-13 18:19:58Z hobu $")
+MS_CVSID("$Id: maplayer.c 8901 2009-04-12 16:49:02Z tamas $")
 
 
 static int populateVirtualTable(layerVTableObj *vtable);
@@ -76,7 +77,7 @@ int msLayerOpen(layerObj *layer)
   if(layer->tileindex && layer->connectiontype == MS_SHAPEFILE)
     layer->connectiontype = MS_TILED_SHAPEFILE;
 
-  if(layer->type == MS_LAYER_RASTER )
+  if(layer->type == MS_LAYER_RASTER && layer->connectiontype != MS_WMS)
     layer->connectiontype = MS_RASTER;
 
   if ( ! layer->vtable) {
@@ -170,6 +171,8 @@ int msLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record)
 */
 void msLayerClose(layerObj *layer) 
 {
+  int i;
+
   /* no need for items once the layer is closed */
   msLayerFreeItemInfo(layer);
   if(layer->items) {
@@ -178,6 +181,15 @@ void msLayerClose(layerObj *layer)
     layer->numitems = 0;
   }
 
+  /* clear out items used as part of expressions (bug #2702) */
+  for(i=0; i<layer->numclasses; i++) {    
+    msFreeCharArray(layer->class[i]->expression.items, layer->class[i]->expression.numitems);
+    msFree(layer->class[i]->expression.indexes);
+    layer->class[i]->expression.items = NULL;
+    layer->class[i]->expression.indexes = NULL;
+    layer->class[i]->expression.numitems = 0;        
+  }
+
   if (layer->vtable) {
     layer->vtable->LayerClose(layer);
   }
@@ -190,6 +202,7 @@ void msLayerClose(layerObj *layer)
 */
 int msLayerGetItems(layerObj *layer) 
 {
+  const char *itemNames;
   /* clean up any previously allocated instances */
   msLayerFreeItemInfo(layer);
   if(layer->items) {
@@ -208,7 +221,14 @@ int msLayerGetItems(layerObj *layer)
    * was following TODO ITEM:
    */
   /* TO DO! Need to add any joined itemd on to the core layer items, one long list!  */
-  return layer->vtable->LayerGetItems(layer);
+  itemNames = msLayerGetProcessingKey( layer, "ITEMS" );
+  if (itemNames)
+  {
+    layer->items = msStringSplit(itemNames, ',', &layer->numitems);
+    return MS_SUCCESS;
+  }
+  else
+    return layer->vtable->LayerGetItems(layer);
 }
 
 /*
@@ -471,9 +491,20 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
     }
   }
 
-  rv = layer->vtable->LayerCreateItems(layer, nt);
-  if (rv != MS_SUCCESS) {
-    return rv;
+  /* Always retrieve all items in some cases */
+  if (layer->connectiontype == MS_INLINE)
+  {
+     msLayerGetItems(layer);
+     /* need to realloc the array to accept the possible new items*/
+     if (nt > 0)
+        layer->items = (char **)realloc(layer->items, sizeof(char *)*(layer->numitems + nt));
+  }
+  else
+  {
+    rv = layer->vtable->LayerCreateItems(layer, nt);
+    if (rv != MS_SUCCESS) {
+      return rv;
+    }
   }
 
   if(nt > 0) {
@@ -1173,7 +1204,7 @@ msInitializeVirtualTable(layerObj *layer)
     if(layer->tileindex && layer->connectiontype == MS_SHAPEFILE)
       layer->connectiontype = MS_TILED_SHAPEFILE;
 
-    if(layer->type == MS_LAYER_RASTER )
+    if(layer->type == MS_LAYER_RASTER && layer->connectiontype != MS_WMS)
       layer->connectiontype = MS_RASTER;
     
 
@@ -1194,11 +1225,11 @@ msInitializeVirtualTable(layerObj *layer)
             return(msOGRLayerInitializeVirtualTable(layer));
             break;
         case(MS_POSTGIS):
-            return(msPOSTGISLayerInitializeVirtualTable(layer));
+            return(msPostGISLayerInitializeVirtualTable(layer));
             break;
         case(MS_WMS):
-              /* WMS isn't a public layer type, it isn't used anywhere */
-              return MS_FAILURE;
+            /* WMS should be treated as a raster layer */
+              return(msRASTERLayerInitializeVirtualTable(layer));
               break;
         case(MS_ORACLESPATIAL):
             return(msOracleSpatialLayerInitializeVirtualTable(layer));
@@ -1270,6 +1301,12 @@ int msINLINELayerGetShape(layerObj *layer, shapeObj *shape, int tile, long shape
         msSetError(MS_SHPERR, "Cannot retrieve inline shape. There some problem with the shape", "msLayerGetShape()");
         return MS_FAILURE;
     }
+    /* check for the expected size of the values array */
+    if (layer->numitems > shape->numvalues) {
+        shape->values = (char **)realloc(shape->values, sizeof(char *)*(layer->numitems));
+        for (i = shape->numvalues; i < layer->numitems; i++)
+            shape->values[i] = strdup("");
+    }
     return MS_SUCCESS;
 }
 
@@ -1284,6 +1321,14 @@ int msINLINELayerNextShape(layerObj *layer, shapeObj *shape)
 
     layer->currentfeature = layer->currentfeature->next;
 
+    /* check for the expected size of the values array */
+    if (layer->numitems > shape->numvalues) {
+        int i;
+        shape->values = (char **)realloc(shape->values, sizeof(char *)*(layer->numitems));
+        for (i = shape->numvalues; i < layer->numitems; i++)
+            shape->values[i] = strdup("");
+    }
+
     return(MS_SUCCESS);
 }
 
