@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapobject.c 8015 2008-11-11 18:14:14Z pramsey $
  *
  * Project:  MapServer
  * Purpose:  Functions for operating on a mapObj that don't belong in a
@@ -34,7 +35,7 @@
 #  include "cpl_conv.h"
 #endif
 
-MS_CVSID("$Id: mapobject.c 7249 2008-01-07 15:43:27Z tamas $")
+MS_CVSID("$Id: mapobject.c 8015 2008-11-11 18:14:14Z pramsey $")
 
 void freeWeb(webObj *web);
 void freeScalebar(scalebarObj *scalebar);
@@ -225,6 +226,33 @@ void msApplyMapConfigOptions( mapObj *map )
     }
 }
 
+/************************************************************************/
+/*                         msMapIgnoreMissingData()                               */
+/************************************************************************/
+
+int msMapIgnoreMissingData( mapObj *map )
+{
+    const char *result = msGetConfigOption( map, "ON_MISSING_DATA" );
+    const int default_result = 
+#ifndef IGNORE_MISSING_DATA
+        MS_MISSING_DATA_FAIL;
+#else
+        MS_MISSING_DATA_LOG;
+#endif
+        
+    if( result == NULL )
+        return default_result;
+        
+    if( strcasecmp(result,"FAIL") == 0 ) 
+        return MS_MISSING_DATA_FAIL;
+    else if( strcasecmp(result,"LOG") == 0 )
+        return MS_MISSING_DATA_LOG;
+    else if( strcasecmp(result,"IGNORE") == 0 )
+        return MS_MISSING_DATA_IGNORE;
+
+    return default_result;    
+}
+
 /************************************************************************/
 /*                           msMapSetExtent()                           */
 /************************************************************************/
@@ -255,6 +283,65 @@ int msMapSetExtent( mapObj *map,
     return msMapComputeGeotransform( map );
 }
 
+/************************************************************************/
+/*                           msMapOffsetExtent()                        */
+/************************************************************************/
+
+int msMapOffsetExtent( mapObj *map, double x, double y) 
+{ 
+	return msMapSetExtent( map, 
+                        map->extent.minx + x, map->extent.miny + y, 
+		                map->extent.maxx + x, map->extent.maxy + y);
+}
+
+/************************************************************************/
+/*                           msMapScaleExtent()                         */
+/************************************************************************/
+
+int msMapScaleExtent( mapObj *map, double zoomfactor, 
+					 double minscaledenom, double maxscaledenom) 
+{ 
+	double geo_width, geo_height, center_x, center_y, md;
+
+	if (zoomfactor <= 0.0) {
+		msSetError(MS_MISCERR, "The given zoomfactor is invalid", "msMapScaleExtent()"); 
+	}
+
+	geo_width = map->extent.maxx - map->extent.minx;
+    geo_height = map->extent.maxy - map->extent.miny;
+
+    center_x = map->extent.minx + geo_width * 0.5;
+    center_y = map->extent.miny + geo_height * 0.5;
+
+	geo_width *= zoomfactor;
+
+	if (minscaledenom > 0 || maxscaledenom > 0) {
+		/* ensure we are within the valid scale domain */
+		md = (map->width-1)/(map->resolution * msInchesPerUnit(map->units, center_y));
+		if (minscaledenom > 0 && geo_width < minscaledenom * md)
+			geo_width = minscaledenom * md;
+		if (maxscaledenom > 0 && geo_width > maxscaledenom * md)
+			geo_width = maxscaledenom * md;
+	}
+
+	geo_width *= 0.5;
+	geo_height = geo_width * map->height / map->width;
+	
+	return msMapSetExtent( map, 
+                        center_x - geo_width, center_y - geo_height, 
+		                center_x + geo_width, center_y + geo_height);
+}
+
+/************************************************************************/
+/*                           msMapSetCenter()                           */
+/************************************************************************/
+
+int msMapSetCenter( mapObj *map, pointObj *center) 
+{ 
+	return msMapOffsetExtent(map, center->x - (map->extent.minx + map->extent.maxx) * 0.5,
+		                          center->y - (map->extent.miny + map->extent.maxy) * 0.5);
+}
+
 /************************************************************************/
 /*                           msMapSetRotation()                         */
 /************************************************************************/
@@ -438,10 +525,20 @@ int msInsertLayer(mapObj *map, layerObj *layer, int nIndex)
     }
 
     /* Ensure there is room for a new layer */
-    if (msGrowMapLayers(map) == NULL)
-        return -1;
-    /* Catch attempt to insert past end of layers array */
-    else if (nIndex >= map->numlayers) {
+    if (map->numlayers == map->maxlayers)
+              {
+                              if (msGrowMapLayers(map) == NULL)
+                                              return -1;
+              }
+
+              /* msGrowMapLayers allocates the new layer which we don't need to do since we have 1 that we are inserting 
+                              not sure if it is possible for this to be non null otherwise, but better to check since this function
+                              replaces the value */
+              if (map->layers[map->numlayers]!=NULL)
+                              free(map->layers[map->numlayers]);
+
+              /* Catch attempt to insert past end of layers array */
+              if (nIndex >= map->numlayers) {
         msSetError(MS_CHILDERR, "Cannot insert layer beyond index %d",
                    "msInsertLayer()", map->numlayers-1);
         return -1;
@@ -506,18 +603,18 @@ layerObj *msRemoveLayer(mapObj *map, int nIndex)
     }
     else {
     	layer=GET_LAYER(map, nIndex);
-        //msCopyLayer(layer, (GET_LAYER(map, nIndex)));
+        /* msCopyLayer(layer, (GET_LAYER(map, nIndex))); */
         
         /* Iteratively copy the higher index layers down one index */
         for (i=nIndex; i<map->numlayers-1; i++) {
-            //freeLayer((GET_LAYER(map, i)));
-            //initLayer((GET_LAYER(map, i)), map);
-            //msCopyLayer(GET_LAYER(map, i), GET_LAYER(map, i+1));
+            /* freeLayer((GET_LAYER(map, i))); */
+            /* initLayer((GET_LAYER(map, i)), map); */
+            /* msCopyLayer(GET_LAYER(map, i), GET_LAYER(map, i+1)); */
 	    GET_LAYER(map, i)=GET_LAYER(map, i+1);
             GET_LAYER(map, i)->index = i;
         }
         /* Free the extra layer at the end */
-        //freeLayer((GET_LAYER(map, map->numlayers-1)));
+        /* freeLayer((GET_LAYER(map, map->numlayers-1))); */
         GET_LAYER(map, map->numlayers-1)=NULL;
         
         /* Adjust drawing order */
