@@ -1,54 +1,45 @@
-/*****************************************************************************
- *             -- Oracle Spatial (SDO) support for MapServer --              *
- *                                                                           *
- *  Authors: Fernando Simon (fsimon@univali.br)                              *
- *           Rodrigo Becke Cabral (cabral@univali.br)                        *
- *  Collaborator: Adriana Gomes Alves                                        *
- *  MapServer: MapServer 4.8-rc1 (cvs)                                       *
- *  Oracle: Oracle 9.2 Spatial Cartridge 9.2 release 9.0.1                   *
- *                                                                           *
- *****************************************************************************
- = This piece for MapServer was originally developed under the funding of
- = agreement n.45/00 between CTTMAR/UNIVALI (www.cttmar.univali.br)
- = and CEPSUL/IBAMA (www.ibama.gov.br).
- *****************************************************************************
- = Current development is funded by CNPq (www.cnpq.br) under
- = process 401263.03-7
- = and FUNCITEC (www.funcitec.rct-sc.br) under process FCTP1523-031.
- *****************************************************************************
- * $Id: maporaclespatial.c 6435 2007-08-01 18:16:13Z fsimon $
- *****************************************************************************
- * Last five fixed bugs:
- *  #1662, #1736, #1845, #1961, #2056
- * Unsolved bugs yet:
- *  #1846, #2129, #2179
+/******************************************************************************
+ * Project: MapServer
+ * Purpose: Native access to Oracle Spatial (SDO) data.  
+ * Author:  Fernando Simon (fsimon@univali.br)
+ *          Rodrigo Becke Cabral
+ *          Adriana Gomes Alves
  *
- * Using OracleSpatial:
- * - CONNECTIONTYPE oraclespatial
- * - CONNECTION 'username/password@database'
- * - DATA 'geometry_column FROM <table>'
- *   or
- *   DATA 'geometry_column FROM <table> [USING UNIQUE <column>' SRID #srid <function> VERSION <vcode>]'
- *       <function> can be:
- *           'FILTER', 'RELATE', GEOMRELATE' or 'NONE'
- *       <vcode> can be:
- *           '8i', '9i' or '10g'
+ * Notes: Developed under several funding agreements:
  *
- *       <table> can be:
- *            One database table name
- *       or:
- *            (SELECT stmt)
- * - Parts of the CONNECTION string may be encrypted, see MS-RFC-18
+ *   1) n.45/00 between CTTMAR/UNIVALI (www.cttmar.univali.br) 
+ *      and CEPSUL/IBAMA (www.ibama.gov.br)
  *
- *****************************************************************************
- * Notices above shall be included in all copies or portions of the software.
- * This piece is provided "AS IS", without warranties of any kind. Got it?
+ *   2) CNPq (www.cnpq.br) under process 401263.03-7
+ *
+ *   3) FUNCITEC (www.funcitec.rct-sc.br) under process FCTP1523-031
+ *
+ ******************************************************************************
+ * Copyright (c) 1996-2008 Regents of the University of Minnesota.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
 #include "mapserver.h"
 #include <assert.h>
 
-MS_CVSID("$Id: maporaclespatial.c 6435 2007-08-01 18:16:13Z fsimon $")
+MS_CVSID("$Id: maporaclespatial.c 8543 2009-02-12 22:53:42Z sdlime $")
 
 #ifdef USE_ORACLESPATIAL
 
@@ -57,7 +48,7 @@ MS_CVSID("$Id: maporaclespatial.c 6435 2007-08-01 18:16:13Z fsimon $")
 
 #define ARRAY_SIZE                 1024
 #define QUERY_SIZE                 1
-#define TEXT_SIZE                  256
+#define TEXT_SIZE                  4000 /* ticket #2260 */
 #define TYPE_OWNER                 "MDSYS"
 #define SDO_GEOMETRY               TYPE_OWNER".SDO_GEOMETRY"
 #define SDO_GEOMETRY_LEN           strlen( SDO_GEOMETRY )
@@ -290,7 +281,7 @@ static int msSplitData( char *data, char *geometry_column_name, char *table_name
     char *tok_unique = "unique";
     char *tok_srid = "srid";
     char *tok_version = "version";
-    char data_version[3] = "";
+    char data_version[4] = "";
     char tok_function[11] = "";
     int parenthesis, i;
     char *src = data, *tgt;
@@ -845,11 +836,11 @@ static void osFilteritem(layerObj *layer, int function, char *query_str, int mod
         if (layer->filteritem != NULL)
         {
             sprintf (query_str + strlen(query_str), " %s = ", layer->filteritem);
-            //snprintf (query_str + strlen(query_str), " %s = ", layer->filteritem);
+            /* snprintf (query_str + strlen(query_str), " %s = ", layer->filteritem); */
         }
 
         sprintf (query_str + strlen(query_str), " %s ", layer->filter.string);
-        //snprintf(buffer, n, "gfdg %s %s %s", layer->filter.string, (layer->filteritem != NULL ? layer->filteritem : ""), );
+        /* snprintf(buffer, n, "gfdg %s %s %s", layer->filter.string, (layer->filteritem != NULL ? layer->filteritem : ""), ); */
 
         if (function != FUNCTION_NONE)
             sprintf (query_str + strlen(query_str), " AND ");
@@ -1927,6 +1918,7 @@ int msOracleSpatialLayerInitItemInfo( layerObj *layer )
 int msOracleSpatialLayerGetItems( layerObj *layer )
 {
     char *rzt = "";
+    ub2 rzttype = 0;
     char *flk = "";
     int function = 0;
     int version = 0;
@@ -2009,6 +2001,8 @@ int msOracleSpatialLayerGetItems( layerObj *layer )
     for (i = 0; i <= layer->numitems; i++)
     {
         success = TRY( hand, OCIParamGet ((dvoid*) dthand->stmthp, (ub4)OCI_HTYPE_STMT,hand->errhp,(dvoid*)&pard, (ub4)i+1))
+               && TRY( hand, OCIAttrGet ((dvoid *) pard,(ub4) OCI_DTYPE_PARAM,(dvoid*)&rzttype,(ub4 *)0, (ub4) OCI_ATTR_DATA_TYPE, hand->errhp ))
+               && TRY( hand, OCIParamGet ((dvoid*) dthand->stmthp, (ub4)OCI_HTYPE_STMT,hand->errhp,(dvoid*)&pard, (ub4)i+1))
                && TRY( hand, OCIAttrGet ((dvoid *) pard,(ub4) OCI_DTYPE_PARAM,(dvoid*)&rzt,(ub4 *)&flk_len, (ub4) OCI_ATTR_NAME, hand->errhp ));
 
         flk = (char *)malloc(sizeof(char*) * flk_len+1);
@@ -2034,7 +2028,14 @@ int msOracleSpatialLayerGetItems( layerObj *layer )
             }
             else
             {
+              if (rzttype!=OCI_TYPECODE_BLOB) 
+              {
                 strcpy(layer->items[count_item], flk);
+              }
+              else
+              {
+                strcpy(layer->items[count_item], "null");
+              }
             }
             count_item++;
         }
@@ -2632,6 +2633,41 @@ int msOracleSpatialLayerGetShapeVT(layerObj *layer, shapeObj *shape, int tile, l
     return msOracleSpatialLayerGetShape(layer, shape, record);
 }
 
+#ifdef USE_ORACLE_PLUGIN 
+#ifdef USE_ORACLESPATIAL
+MS_DLL_EXPORT  int
+PluginInitializeVirtualTable(layerVTableObj* vtable, layerObj *layer)
+{
+    assert(layer != NULL);
+    assert(vtable != NULL);
+    
+
+    vtable->LayerInitItemInfo = msOracleSpatialLayerInitItemInfo;
+    vtable->LayerFreeItemInfo = msOracleSpatialLayerFreeItemInfo;
+    vtable->LayerOpen = msOracleSpatialLayerOpen;
+    vtable->LayerIsOpen = msOracleSpatialLayerIsOpen;
+    vtable->LayerWhichShapes = msOracleSpatialLayerWhichShapes;
+    vtable->LayerNextShape = msOracleSpatialLayerNextShape;
+    vtable->LayerGetShape = msOracleSpatialLayerGetShapeVT;
+    vtable->LayerClose = msOracleSpatialLayerClose;
+    vtable->LayerGetItems = msOracleSpatialLayerGetItems;
+    vtable->LayerGetExtent = msOracleSpatialLayerGetExtent;
+
+    /* layer->vtable->LayerGetAutoStyle, use default */
+    /* layer->vtable->LayerApplyFilterToLayer, use default */
+
+    vtable->LayerCloseConnection = msOracleSpatialLayerClose;
+    vtable->LayerApplyFilterToLayer = msLayerApplyCondSQLFilterToLayer;
+
+    vtable->LayerSetTimeFilter = msLayerMakePlainTimeFilter;
+
+    /* layer->vtable->LayerGetNumFeatures, use default */
+
+    return MS_SUCCESS;
+}
+#endif
+#endif
+
 int msOracleSpatialLayerInitializeVirtualTable(layerObj *layer)
 {
     assert(layer != NULL);
