@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapoutput.c 8821 2009-03-25 14:26:21Z assefa $
  *
  * Project:  MapServer
  * Purpose:  Various support code related to the outputFormatObj.
@@ -29,7 +30,7 @@
 #include <assert.h>
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapoutput.c 6455 2007-08-03 14:18:54Z warmerdam $")
+MS_CVSID("$Id: mapoutput.c 8821 2009-03-25 14:26:21Z assefa $")
 
 static outputFormatObj *msAllocOutputFormat( mapObj *map, const char *name, 
                                              const char *driver );
@@ -190,7 +191,7 @@ outputFormatObj *msCreateDefaultOutputFormat( mapObj *map,
     if( strcasecmp(driver,"GD/WBMP") == 0 )
     {
         format = msAllocOutputFormat( map, "wbmp", driver );
-        format->mimetype = strdup("image/wbmp");
+        format->mimetype = strdup("image/vnd.wap.wbmp");
         format->imagemode = MS_IMAGEMODE_PC256;
         format->extension = strdup("wbmp");
         format->renderer = MS_RENDER_WITH_GD;
@@ -271,8 +272,17 @@ outputFormatObj *msCreateDefaultOutputFormat( mapObj *map,
         format->extension = strdup("svg");
         format->renderer = MS_RENDER_WITH_SVG;
     }
+
+    if( strcasecmp(driver,"template") == 0 ) {
+      format = msAllocOutputFormat( map, "template", driver );
+      format->mimetype = strdup("text/html");
+      format->extension = strdup("html");
+      format->imagemode = 0;
+      format->renderer = MS_RENDER_WITH_TEMPLATE;
+    }
+
     if( format != NULL )
-        format->inmapfile = MS_FALSE;
+      format->inmapfile = MS_FALSE;
 
     return format;
 }
@@ -813,7 +823,8 @@ void msGetOutputFormatMimeListGD( mapObj *map, char **mime_list, int max_mime )
         }
 
         if( j == mime_count && map->outputformatlist[i]->driver &&
-            strncasecmp(map->outputformatlist[i]->driver, "GD/", 3)==0)
+            (strncasecmp(map->outputformatlist[i]->driver, "GD/", 3)==0 ||
+             strncasecmp(map->outputformatlist[i]->driver, "AGG/", 4)==0))
             mime_list[mime_count++] = map->outputformatlist[i]->mimetype;
     }
 
@@ -847,6 +858,7 @@ void msGetOutputFormatMimeListWMS( mapObj *map, char **mime_list, int max_mime )
 
         if( j == mime_count && map->outputformatlist[i]->driver &&
             (strncasecmp(map->outputformatlist[i]->driver, "GD/", 3)==0 ||
+             strncasecmp(map->outputformatlist[i]->driver, "AGG/", 4)==0 ||
              strncasecmp(map->outputformatlist[i]->driver, "GDAL/", 5)==0 ||
              strcasecmp(map->outputformatlist[i]->driver, "svg")==0)) 
             mime_list[mime_count++] = map->outputformatlist[i]->mimetype;
