@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapows.c 8545 2009-02-13 00:37:18Z assefa $
  *
  * Project:  MapServer
  * Purpose:  OGC Web Services (WMS, WFS) support functions
@@ -27,22 +28,25 @@
  ****************************************************************************/
 
 #include "mapserver.h"
+#include "maptime.h"
 
 #include <ctype.h> /* isalnum() */
 #include <stdarg.h> 
 #include <assert.h>
 
-MS_CVSID("$Id: mapows.c 6734 2007-08-29 14:41:10Z dmorissette $")
+MS_CVSID("$Id: mapows.c 8545 2009-02-13 00:37:18Z assefa $")
 
 /*
 ** msOWSDispatch() is the entry point for any OWS request (WMS, WFS, ...)
 ** - If this is a valid request then it is processed and MS_SUCCESS is returned
 **   on success, or MS_FAILURE on failure.
-** - If this does not appear to be a valid OWS request then MS_DONE
-**   is returned and MapServer is expected to process this as a regular
-**   MapServer request.
+** - If force_ows_mode is true then an exception will be produced if the
+**   request is not recognized as a valid request.
+** - If force_ows_mode is false and this does not appear to be a valid OWS 
+**   request then MS_DONE is returned and MapServer is expected to process 
+**   this as a regular MapServer (traditional CGI) request.
 */
-int msOWSDispatch(mapObj *map, cgiRequestObj *request)
+int msOWSDispatch(mapObj *map, cgiRequestObj *request, int force_ows_mode)
 {
     int i, status = MS_DONE;
     const char *service=NULL;
@@ -52,11 +56,14 @@ int msOWSDispatch(mapObj *map, cgiRequestObj *request)
 
     for( i=0; i<request->NumParams; i++ ) 
     {
-        if(strcasecmp(request->ParamNames[i], "VERSION") == 0)
+        if(strcasecmp(request->ParamNames[i], "SERVICE") == 0)
             service = request->ParamValues[i];
     }
 
 #ifdef USE_WMS_SVR
+    /* Note: SERVICE param did not exist in WMS 1.0.0, it was added only in WMS 1.1.0,
+     * so we need to let msWMSDispatch check for known REQUESTs even if SERVICE is not set.
+     */
     if ((status = msWMSDispatch(map, request)) != MS_DONE )
         return status;
 #else
@@ -67,6 +74,9 @@ int msOWSDispatch(mapObj *map, cgiRequestObj *request)
 #endif
 
 #ifdef USE_WFS_SVR
+    /* Note: WFS supports POST requests, so the SERVICE param may only be in the post data
+     * and not be present in the GET URL
+     */
     if ((status = msWFSDispatch(map, request)) != MS_DONE )
         return status;
 #else
@@ -96,7 +106,32 @@ int msOWSDispatch(mapObj *map, cgiRequestObj *request)
                     "msOWSDispatch()" );
 #endif
 
-    return MS_DONE;  /* Not a WMS/WFS request... let MapServer handle it */
+    if (force_ows_mode) {
+        if (service == NULL)
+            /* Here we should return real OWS Common exceptions... once 
+             * we have a proper exception handler in mapowscommon.c 
+             */
+            msSetError( MS_MISCERR,
+                        "OWS Common exception: exceptionCode=MissingParameterValue, locator=SERVICE, ExceptionText=SERVICE parameter missing.", 
+                        "msOWSDispatch()");
+        else
+            /* Here we should return real OWS Common exceptions... once 
+             * we have a proper exception handler in mapowscommon.c 
+             */
+            msSetError( MS_MISCERR,
+                        "OWS Common exception: exceptionCode=InvalidParameterValue, locator=SERVICE, ExceptionText=SERVICE parameter value invalid.", 
+                        "msOWSDispatch()");
+        
+        /* Force mapserv to report the error by returning MS_FAILURE. 
+         * The day we have a proper exception handler here then we should 
+         * return MS_SUCCESS since the exception will have been processed 
+         * the OWS way, which is a success as far as mapserv is concerned 
+         */
+        return MS_FAILURE; 
+    }
+
+    return MS_DONE;  /* Not a WMS/WFS request... let MapServer handle it 
+                      * since we're not in force_ows_mode*/
 }
 
 #if defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR) || defined(USE_SOS_SVR) || defined(USE_WMS_LYR) || defined(USE_WFS_LYR)
@@ -139,10 +174,7 @@ int msOWSMakeAllLayersUnique(mapObj *map)
       {
           if (GET_LAYER(map, i)->name == NULL || GET_LAYER(map, j)->name == NULL)
           {
-              msSetError(MS_MISCERR, 
-                         "At least one layer is missing a name in map file.", 
-                         "msOWSMakeAllLayersUnique()");
-              return MS_FAILURE;
+              continue;
           }
           if (strcasecmp(GET_LAYER(map, i)->name, GET_LAYER(map, j)->name) == 0 &&
               msRenameLayer((GET_LAYER(map, j)), ++count) != MS_SUCCESS)
@@ -160,6 +192,37 @@ int msOWSMakeAllLayersUnique(mapObj *map)
   return MS_SUCCESS;
 }
 
+/*
+** msOWSNegotiateVersion()
+**
+** returns the most suitable version an OWS is to support given a client
+** version parameter.
+**
+** supported_versions must be ordered from highest to lowest
+**
+** Returns a version integer of the supported version
+**
+*/
+
+int msOWSNegotiateVersion(int requested_version, int supported_versions[], int num_supported_versions) {
+  int i;
+
+  /* if version is not set return highest version */
+  if (! requested_version)
+    return supported_versions[0];
+
+  /* if the requested version is lower than the lowest version return the lowest version  */
+  if (requested_version < supported_versions[num_supported_versions-1])
+    return supported_versions[num_supported_versions-1];
+
+  /* return the first entry that's lower than or equal to the requested version */
+  for (i = 0; i < num_supported_versions; i++) {
+    if (supported_versions[i] <= requested_version)
+      return supported_versions[i];
+  }
+
+  return requested_version;
+}
 
 /*
 ** msOWSTerminateOnlineResource()
@@ -329,7 +392,7 @@ const char *msOWSGetLanguage(mapObj *map, const char *context)
     /* if this is an exception, MapServer always returns Exception
        messages in en-US
     */
-    if (context == "exception") {
+    if (strcmp(context,"exception") == 0) {
       language = MS_ERROR_LANGUAGE;
     }
     /* if not, fetch language from mapfile metadata */
@@ -348,7 +411,8 @@ const char *msOWSGetLanguage(mapObj *map, const char *context)
 ** Parse a version string in the format "a.b.c" or "a.b" and return an
 ** integer in the format 0x0a0b0c suitable for regular integer comparisons.
 **
-** Returns -1 if version could not be parsed.
+** Returns one of OWS_VERSION_NOTSET or OWS_VERSION_BADFORMAT if version 
+** could not be parsed.
 */
 int msOWSParseVersionString(const char *pszVersion)
 {
@@ -361,13 +425,13 @@ int msOWSParseVersionString(const char *pszVersion)
         digits = msStringSplit(pszVersion, '.', &numDigits);
         if (digits == NULL || numDigits < 2 || numDigits > 3)
         {
-            msSetError(MS_WMSERR, 
-                       "Invalid version (%s). OWS version must be in the "
+            msSetError(MS_OWSERR, 
+                       "Invalid version (%s). Version must be in the "
                        "format 'x.y' or 'x.y.z'",
                        "msOWSParseVersionString()", pszVersion);
             if (digits)
                 msFreeCharArray(digits, numDigits);
-            return -1;
+            return OWS_VERSION_BADFORMAT;
         }
 
         nVersion = atoi(digits[0])*0x010000;
@@ -380,7 +444,7 @@ int msOWSParseVersionString(const char *pszVersion)
         return nVersion;
     }
 
-    return -1;
+    return OWS_VERSION_NOTSET;
 }
 
 /* msOWSGetVersionString()
@@ -480,6 +544,35 @@ const char *msOWSLookupMetadata(hashTableObj *metadata,
     return value;
 }
 
+/*
+** msOWSLookupMetadata2()
+**
+** Attempts to lookup a given metadata name in multiple hashTables, and
+** in multiple OWS namespaces within each. First searches the primary
+** table and if no result is found, attempts the search using the 
+** secondary (fallback) table.
+**
+** 'namespaces' is a string with a letter for each namespace to lookup 
+** in the order they should be looked up. e.g. "MO" to lookup wms_ and ows_
+** If namespaces is NULL then this function just does a regular metadata
+** lookup.
+*/
+const char *msOWSLookupMetadata2(hashTableObj *pri,
+                                        hashTableObj *sec,
+                                        const char *namespaces,
+                                        const char *name)
+{
+    const char *result;
+    
+    if ((result = msOWSLookupMetadata(pri, namespaces, name)) == NULL)
+    {
+        /* Try the secondary table */
+        result = msOWSLookupMetadata(sec, namespaces, name);
+    }
+
+    return result;
+}
+
 /*
 ** msOWSPrintMetadata()
 **
@@ -1067,6 +1160,39 @@ int msOWSPrintEncodeParamList(FILE *stream, const char *name,
     return status;
 }
 
+
+/*
+** msOWSPrintEX_GeographicBoundingBox()
+**
+** Print a EX_GeographicBoundingBox tag for WMS1.3.0
+**
+*/
+void msOWSPrintEX_GeographicBoundingBox(FILE *stream, const char *tabspace, 
+                                        rectObj *extent, projectionObj *srcproj)
+
+{
+  const char *pszTag = "EX_GeographicBoundingBox";  /* The default for WMS */
+  rectObj ext;
+
+  ext = *extent;
+
+  /* always project to lat long */
+  if (srcproj->numargs > 0 && !pj_is_latlong(srcproj->proj)) {
+      msProjectRect(srcproj, NULL, &ext);
+  }
+  
+
+  msIO_fprintf(stream, "%s<%s>\n", tabspace, pszTag);
+  msIO_fprintf(stream, "%s    <westBoundLongitude>%g</westBoundLongitude>\n", tabspace, ext.minx);
+  msIO_fprintf(stream, "%s    <eastBoundLongitude>%g</eastBoundLongitude>\n", tabspace, ext.maxx);
+  msIO_fprintf(stream, "%s    <southBoundLatitude>%g</southBoundLatitude>\n", tabspace, ext.miny);
+  msIO_fprintf(stream, "%s    <northBoundLatitude>%g</northBoundLatitude>\n", tabspace, ext.maxy);
+  msIO_fprintf(stream, "%s</%s>\n", tabspace, pszTag);
+
+  //msIO_fprintf(stream, "%s<%s minx=\"%g\" miny=\"%g\" maxx=\"%g\" maxy=\"%g\" />\n", 
+  //      tabspace, pszTag, ext.minx, ext.miny, ext.maxx, ext.maxy);
+}
+
 /*
 ** msOWSPrintLatLonBoundingBox()
 **
@@ -1076,22 +1202,29 @@ int msOWSPrintEncodeParamList(FILE *stream, const char *name,
 */
 void msOWSPrintLatLonBoundingBox(FILE *stream, const char *tabspace, 
                                  rectObj *extent, projectionObj *srcproj,
-                                 int nService)
+                                 projectionObj *wfsproj, int nService)
 {
   const char *pszTag = "LatLonBoundingBox";  /* The default for WMS */
-  rectObj ll_ext;
+  rectObj ext;
 
-  ll_ext = *extent;
+  ext = *extent;
 
-  if (srcproj->numargs > 0 && !pj_is_latlong(srcproj->proj)) {
-    msProjectRect(srcproj, NULL, &ll_ext);
+  if (nService == OWS_WMS) { /* always project to lat long */
+    if (srcproj->numargs > 0 && !pj_is_latlong(srcproj->proj)) {
+      msProjectRect(srcproj, NULL, &ext);
+    }
   }
 
-  if (nService == OWS_WFS)
+  if (nService == OWS_WFS) {
       pszTag = "LatLongBoundingBox";
+      if (wfsproj) {
+          if (msProjectionsDiffer(srcproj, wfsproj) == MS_TRUE)
+              msProjectRect(srcproj, wfsproj, &ext);
+      }
+  }
 
   msIO_fprintf(stream, "%s<%s minx=\"%g\" miny=\"%g\" maxx=\"%g\" maxy=\"%g\" />\n", 
-         tabspace, pszTag, ll_ext.minx, ll_ext.miny, ll_ext.maxx, ll_ext.maxy);
+         tabspace, pszTag, ext.minx, ext.miny, ext.maxx, ext.maxy);
 }
 
 /*
@@ -1101,24 +1234,48 @@ void msOWSPrintBoundingBox(FILE *stream, const char *tabspace,
                            rectObj *extent, 
                            projectionObj *srcproj,
                            hashTableObj *metadata,
-                           const char *namespaces) 
+                           const char *namespaces,
+                           int wms_version) 
 {
     const char	*value, *resx, *resy;
     char *encoded, *encoded_resx, *encoded_resy;
+    projectionObj proj;
 
     /* Look for EPSG code in PROJECTION block only.  "wms_srs" metadata cannot be
      * used to establish the native projection of a layer for BoundingBox purposes.
      */
     value = msOWSGetEPSGProj(srcproj, NULL, namespaces, MS_TRUE);
     
+    /*for wms 1.3.0 we need to make sure that we present the BBOX with  
+      a revered axes for some espg codes*/
+    if (wms_version >= OWS_1_3_0 && value && strncasecmp(value, "EPSG:", 5) == 0)
+    {
+        msInitProjection(&proj);
+        if (msLoadProjectionStringEPSG(&proj, (char *)value) == 0)
+        {
+            msAxisNormalizePoints( &proj, 1, &extent->minx, &extent->miny );
+            msAxisNormalizePoints( &proj, 1, &extent->maxx, &extent->maxy );
+        }
+        msFreeProjection( &proj );
+    }
+    
+
     if( value != NULL )
     {
         encoded = msEncodeHTMLEntities(value);
-        msIO_fprintf(stream, "%s<BoundingBox SRS=\"%s\"\n"
+        if (wms_version >= OWS_1_3_0)
+          msIO_fprintf(stream, "%s<BoundingBox CRS=\"%s\"\n"
                "%s            minx=\"%g\" miny=\"%g\" maxx=\"%g\" maxy=\"%g\"",
                tabspace, encoded, 
                tabspace, extent->minx, extent->miny, 
                extent->maxx, extent->maxy);
+        else
+          msIO_fprintf(stream, "%s<BoundingBox SRS=\"%s\"\n"
+               "%s            minx=\"%g\" miny=\"%g\" maxx=\"%g\" maxy=\"%g\"",
+               tabspace, encoded, 
+               tabspace, extent->minx, extent->miny, 
+               extent->maxx, extent->maxy);
+
         msFree(encoded);
 
         if( (resx = msOWSLookupMetadata( metadata, "MFO", "resx" )) != NULL &&
@@ -1459,13 +1616,76 @@ const char *msOWSGetEPSGProj(projectionObj *proj, hashTableObj *metadata, const
   } else if (proj && proj->numargs > 0 && (value = strstr(proj->args[0], "init=epsg:")) != NULL && strlen(value) < 20) {
     sprintf(epsgCode, "EPSG:%s", value+10);
     return epsgCode;
-  } else if (proj && proj->numargs > 0 && strncasecmp(proj->args[0], "AUTO:", 5) == 0 ) {
+  } else if (proj && proj->numargs > 0 && (strncasecmp(proj->args[0], "AUTO:", 5) == 0 ||
+                                           strncasecmp(proj->args[0], "AUTO2:", 6) == 0)) {
     return proj->args[0];
   }
 
   return NULL;
 }
 
+/*
+** msOWSGetProjURN()
+**
+** Fetch an OGC URN for this layer or map.  Similar to msOWSGetEPSGProj()
+** but returns the result in the form "urn:ogc:def:crs:EPSG::27700".
+** The returned buffer is dynamically allocated, and must be freed by the
+** caller.
+*/
+char *msOWSGetProjURN(projectionObj *proj, hashTableObj *metadata, const char *namespaces, int bReturnOnlyFirstOne)
+{
+    char *result;
+    char **tokens;
+    int numtokens, i;
+
+    const char *oldStyle = msOWSGetEPSGProj( proj, metadata, namespaces, 
+                                             bReturnOnlyFirstOne );
+
+    if( strncmp(oldStyle,"EPSG:",5) != 0 )
+        return NULL;
+
+    result = strdup("");
+
+    tokens = msStringSplit(oldStyle, ' ', &numtokens);
+    for(i=0; tokens != NULL && i<numtokens; i++)
+    {
+        char urn[100];
+
+        if( strncmp(tokens[i],"EPSG:",5) == 0 )
+            sprintf( urn, "urn:ogc:def:crs:EPSG::%s", tokens[i]+5 );
+        else if( strcasecmp(tokens[i],"imageCRS") == 0 )
+            sprintf( urn, "urn:ogc:def:crs:OGC::imageCRS" );
+        else if( strncmp(tokens[i],"urn:ogc:def:crs:",16) == 0 )
+            sprintf( urn, tokens[i] );
+        else
+            strcpy( urn, "" );
+
+        if( strlen(urn) > 0 )
+        {
+            result = (char *) realloc(result,strlen(result)+strlen(urn)+2);
+            
+            if( strlen(result) > 0 )
+                strcat( result, " " );
+            strcat( result, urn );
+        }
+        else
+        {
+            msDebug( "msOWSGetProjURN(): Failed to process SRS '%s', ignored.", 
+                     tokens[i] );
+        }
+    }
+
+    msFreeCharArray(tokens, numtokens);
+
+    if( strlen(result) == 0 )
+    {
+        msFree( result );
+        return NULL;
+    }
+    else
+        return result;
+}
+
 
 /*
 ** msOWSGetDimensionInfo()
@@ -1558,6 +1778,82 @@ void msOWSGetDimensionInfo(layerObj *layer, const char *pszDimension,
     return;
 }
 
+/**
+ * msOWSNegotiateUpdateSequence()
+ *
+ * returns the updateSequence value for an OWS
+ *
+ * @param requested_updatesequence the updatesequence passed by the client
+ * @param updatesequence the updatesequence set by the server
+ *
+ * @return result of comparison (-1, 0, 1)
+ * -1: lower / higher OR values not set by client or server
+ *  1: higher / lower
+ *  0: equal
+ */
+
+int msOWSNegotiateUpdateSequence(const char *requested_updatesequence, const char *updatesequence) {
+  int i;
+  int valtype1 = 1; /* default datatype for updatesequence passed by client */
+  int valtype2 = 1; /* default datatype for updatesequence set by server */
+  struct tm tm_requested_updatesequence, tm_updatesequence;
+
+  /* if not specified by client, or set by server,
+     server responds with latest Capabilities XML */
+  if (! requested_updatesequence || ! updatesequence)
+    return -1; 
+
+  /* test to see if server value is an integer (1), string (2) or timestamp (3) */
+  if (msStringIsInteger(updatesequence) == MS_FAILURE)
+    valtype1 = 2;
+
+  if (valtype1 == 2) { /* test if timestamp */
+    msTimeInit(&tm_updatesequence);
+    if (msParseTime(updatesequence, &tm_updatesequence) == MS_TRUE)
+      valtype1 = 3;
+    msResetErrorList();
+  }
+
+  /* test to see if client value is an integer (1), string (2) or timestamp (3) */
+  if (msStringIsInteger(requested_updatesequence) == MS_FAILURE)
+    valtype2 = 2;
+
+  if (valtype2 == 2) { /* test if timestamp */
+    msTimeInit(&tm_requested_updatesequence);
+    if (msParseTime(requested_updatesequence, &tm_requested_updatesequence) == MS_TRUE)
+      valtype2 = 3;
+    msResetErrorList();
+  }
+
+  /* if the datatypes do not match, do not compare, */
+  if (valtype1 != valtype2)
+    return -1;
+
+  if (valtype1 == 1) { /* integer */
+    if (atoi(requested_updatesequence) < atoi(updatesequence))
+      return -1;
+
+    if (atoi(requested_updatesequence) > atoi(updatesequence))
+      return 1;
+
+    if (atoi(requested_updatesequence) == atoi(updatesequence))
+      return 0;
+  }
+
+  if (valtype1 == 2) /* string */
+    return strcasecmp(requested_updatesequence, updatesequence);
+
+  if (valtype1 == 3) { /* timestamp */
+    /* compare timestamps */
+    i = msDateCompare(&tm_requested_updatesequence, &tm_updatesequence) +
+        msTimeCompare(&tm_requested_updatesequence, &tm_updatesequence);
+    return i;
+  }
+
+  /* return default -1 */
+  return -1;
+}
+
 #endif /* USE_WMS_SVR || USE_WFS_SVR  || USE_WCS_SVR */
 
 
