@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mappdf.c 8482 2009-01-29 23:34:29Z tamas $
  *
  * Project:  MapServer
  * Purpose:  An api for PDF format output using the pdflib library
@@ -35,7 +36,7 @@
 #endif
 #include "mapserver.h"
 
-MS_CVSID("$Id: mappdf.c 7299 2008-01-23 15:14:10Z Assefa $")
+MS_CVSID("$Id: mappdf.c 8482 2009-01-29 23:34:29Z tamas $")
 
 /* -------------------------------------------------------------------- */
 /*      prototypes.                                                     */
@@ -242,7 +243,6 @@ void drawPolylinePDF(PDF *pdf, shapeObj *p, colorObj  *c, double width)
     if (width)
     {
         PDF_setlinejoin(pdf,1);
-         PDF_setlinecap(pdf,1); 
         PDF_setlinewidth(pdf,(float)width);
     }
 
@@ -451,7 +451,7 @@ int msDrawLabelPDF(imageObj *image, pointObj labelPnt, char *string,
         pointObj p;
         rectObj r;
 
-        if(msGetLabelSize(string, label, &r, fontset, scalefactor, MS_FALSE) == -1) return(-1);
+        if(msGetLabelSize(image,string, label, &r, fontset, scalefactor, MS_FALSE,NULL) == -1) return(-1);
         p = get_metrics(&labelPnt, label->position, r,
                                    label->offsetx,
                                    label->offsety,
@@ -503,9 +503,13 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
     if (strcasecmp(msGetOutputFormatOption(image->format,"OUTPUT_TYPE",""), 
                    "RASTER") == 0)
     {
+        int orig_renderer;
         imagetmp = (imageObj *)image->img.pdf->imagetmp;
         msImageInitGD( imagetmp, &map->imagecolor);
-        msDrawLabelCacheGD(imagetmp->img.gd, map);
+        orig_renderer=image->format->renderer;
+        image->format->renderer = MS_RENDER_WITH_GD;
+        msDrawLabelCache(imagetmp, map);
+        image->format->renderer = orig_renderer;
         return 0;
     }
 
@@ -530,7 +534,7 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
         if(strlen(cachePtr->text) == 0)
             continue; /* not an error, just don't want to do anything */
 
-        if(msGetLabelSize(cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor, MS_TRUE) == -1)
+        if(msGetLabelSize(image,cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor, MS_TRUE,NULL) == -1)
             return(-1);
 
         if(labelPtr->autominfeaturesize && ((r.maxx-r.minx) > cachePtr->featuresize))
@@ -1202,7 +1206,7 @@ int msDrawTextPDF(imageObj *image, pointObj labelPnt, char *string,
 {
     int x, y, x1, y1;
     int font = 0;
-    int size;
+    double size;
     float phi = label->angle;
     colorObj  sColor;
     char *wrappedString;
@@ -1304,8 +1308,8 @@ int msDrawTextPDF(imageObj *image, pointObj labelPnt, char *string,
 
 /*set up font handling*/
     /*set an alias for the font outline file*/
-    nFontOutlineLen = strlen(fontPath + 3);
-    fontOutlineAlias = (char *)malloc(sizeof(char)*(nFontOutlineLen));
+    nFontOutlineLen = strlen(fontPath);
+    fontOutlineAlias = (char *)malloc(sizeof(char)*(nFontOutlineLen+4));
     sprintf(fontOutlineAlias, "f1=%s",fontPath);
     PDF_set_parameter(pdf, "FontOutline", fontOutlineAlias);
     free(fontOutlineAlias);
