@@ -1,11 +1,12 @@
 /******************************************************************************
+ * $Id: mapprimitive.c 9131 2009-07-01 04:20:32Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Implementations for rectObj, pointObj, lineObj, shapeObj, etc.
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2005 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2008 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,7 +32,7 @@
 #include <assert.h>
 #include <locale.h>
 
-MS_CVSID("$Id: mapprimitive.c 6777 2007-09-03 03:54:47Z sdlime $")
+MS_CVSID("$Id: mapprimitive.c 9131 2009-07-01 04:20:32Z sdlime $")
 
 typedef enum {CLIP_LEFT, CLIP_MIDDLE, CLIP_RIGHT} CLIP_STATE;
 
@@ -41,9 +42,9 @@ typedef enum {CLIP_LEFT, CLIP_MIDDLE, CLIP_RIGHT} CLIP_STATE;
 #define EDGE_CHECK( x0, x, x1) ((x) < MS_MIN( (x0), (x1)) ? CLIP_LEFT : ((x) > MS_MAX( (x0), (x1)) ? CLIP_RIGHT : CLIP_MIDDLE ))
 
 #ifndef INFINITY
-#define INFINITY	(1.0e+30)
+#define INFINITY (1.0e+30)
 #endif
-#define NEARZERO	(1.0e-30)	/* 1/INFINITY */
+#define NEARZERO (1.0e-30) /* 1/INFINITY */
 
 void msPrintShape(shapeObj *p) 
 {
@@ -167,22 +168,19 @@ void msFreeLabelPathObj(labelPathObj *path)
 }
 
 void msShapeDeleteLine( shapeObj *shape, int line )
-
 {
-    if( line < 0 || line >= shape->numlines )
-    {
-        assert( 0 );
-        return;
-    }
+  if( line < 0 || line >= shape->numlines ) {
+    assert( 0 );
+    return;
+  }
 
-    free( shape->line[line].point );
-    if( line < shape->numlines - 1 )
-    {
-        memmove( shape->line + line, 
-                 shape->line + line + 1, 
-                 sizeof(lineObj) * (shape->numlines - line - 1) );
-    }
-    shape->numlines--;
+  free( shape->line[line].point );
+  if( line < shape->numlines - 1 ) {
+    memmove( shape->line + line, 
+             shape->line + line + 1, 
+             sizeof(lineObj) * (shape->numlines - line - 1) );
+  }
+  shape->numlines--;
 }
 
 void msComputeBounds(shapeObj *shape)
@@ -209,13 +207,27 @@ void msComputeBounds(shapeObj *shape)
 static int isOuterRing(shapeObj *shape, int r) 
 {
   int i, status=MS_TRUE; 
+  int result1, result2;
 
   if(shape->numlines == 1) return(MS_TRUE);
 
   for(i=0; i<shape->numlines; i++) {
     if(i == r) continue;
-    if(msPointInPolygon(&(shape->line[r].point[0]), &(shape->line[i])) == MS_TRUE)
+
+    /*
+    ** We have to test 2, or perhaps 3 points on the shape against the ring because
+    ** it is possible that at most one point could touch the ring and the function
+    ** msPointInPolygon() is indeterminite in that case. (bug #2434)
+    */
+    result1 = msPointInPolygon(&(shape->line[r].point[0]), &(shape->line[i]));
+    result2 = msPointInPolygon(&(shape->line[r].point[1]), &(shape->line[i]));
+    if(result1 == result2) { /* same result twice, neither point was on the edge */
+      if(result1 == MS_TRUE) status = !status;
+    } else { /* one of the first 2 points were on the edge of the ring, the next one isn't */
+      if(msPointInPolygon(&(shape->line[r].point[2]), &(shape->line[i])) == MS_TRUE)
       status = !status;
+    }
+
   }
 
   return(status);
@@ -274,32 +286,28 @@ int *msGetInnerList(shapeObj *shape, int r, int *outerlist)
 
 int msAddPointToLine(lineObj *line, pointObj *point )
 {
-    line->numpoints += 1;
+  line->numpoints += 1;
     
-    line->point = (pointObj *) 
-        realloc(line->point, sizeof(pointObj) * line->numpoints);
+  line->point = (pointObj *) realloc(line->point, sizeof(pointObj) * line->numpoints);
+  line->point[line->numpoints-1] = *point;
 
-    line->point[line->numpoints-1] = *point;
-
-    return MS_SUCCESS;
+  return MS_SUCCESS;
 }
 
 int msAddLine(shapeObj *p, lineObj *new_line)
 {
-    lineObj lineCopy;
+  lineObj lineCopy;
 
-    lineCopy.numpoints = new_line->numpoints;
-    lineCopy.point = (pointObj *) malloc(new_line->numpoints*sizeof(pointObj));
-    if( lineCopy.point == NULL )
-    {
-        msSetError(MS_MEMERR, NULL, "msAddLine()");
-        return(MS_FAILURE);
-    }
+  lineCopy.numpoints = new_line->numpoints;
+  lineCopy.point = (pointObj *) malloc(new_line->numpoints*sizeof(pointObj));
+  if( lineCopy.point == NULL ) {
+    msSetError(MS_MEMERR, NULL, "msAddLine()");
+    return(MS_FAILURE);
+  }
     
-    memcpy( lineCopy.point, new_line->point, 
-            sizeof(pointObj) * new_line->numpoints );
+  memcpy( lineCopy.point, new_line->point, sizeof(pointObj) * new_line->numpoints );
 
-    return msAddLineDirectly( p, &lineCopy );
+  return msAddLineDirectly( p, &lineCopy );
 }
 
 /*
@@ -311,10 +319,15 @@ int msAddLineDirectly(shapeObj *p, lineObj *new_line)
   int c;
 
   if( p->numlines == 0 )
-      p->line = (lineObj *) malloc(sizeof(lineObj));
+    p->line = (lineObj *) malloc(sizeof(lineObj));
   else
-      p->line = (lineObj *) realloc(p->line, (p->numlines+1)*sizeof(lineObj));
-  
+    p->line = (lineObj *) realloc(p->line, (p->numlines+1)*sizeof(lineObj));
+
+  if(!p->line) {
+    msSetError(MS_MEMERR, NULL, "msAddLineDirectly()");
+    return(MS_FAILURE);
+  }
+
   /* Copy the new line onto the end of the extended line array */
   c= p->numlines;
   p->line[c].numpoints = new_line->numpoints;  
@@ -367,8 +380,8 @@ void msRectToPolygon(rectObj rect, shapeObj *poly)
 }
 
 /*
-** Private implementation of the Sutherland-Cohen algorithm. Taken in part
-** from "Getting Graphic: Programming Fundamentals in C and C++" by Mark Finlay
+** Private implementation of the Sutherland-Cohen algorithm. Inspired by
+** "Getting Graphic: Programming Fundamentals in C and C++" by Mark Finlay
 ** and John Petritis. (pages 179-182)
 */
 static int clipLine(double *x1, double *y1, double *x2, double *y2, rectObj rect)
@@ -483,22 +496,22 @@ void msClipPolylineRect(shapeObj *shape, rectObj rect)
       y2 = shape->line[i].point[j].y;
 
       if(clipLine(&x1,&y1,&x2,&y2,rect) == MS_TRUE) {
-	if(line.numpoints == 0) { /* first segment, add both points */
-	  line.point[0].x = x1;
-	  line.point[0].y = y1;
-	  line.point[1].x = x2;
-	  line.point[1].y = y2;
-	  line.numpoints = 2;
-	} else { /* add just the last point */
-	  line.point[line.numpoints].x = x2;
-	  line.point[line.numpoints].y = y2;
-	  line.numpoints++;	  
-	}
+        if(line.numpoints == 0) { /* first segment, add both points */
+          line.point[0].x = x1;
+          line.point[0].y = y1;
+          line.point[1].x = x2;
+          line.point[1].y = y2;
+          line.numpoints = 2;
+        } else { /* add just the last point */
+          line.point[line.numpoints].x = x2;
+          line.point[line.numpoints].y = y2;
+          line.numpoints++;      
+        }
 
-	if((x2 != shape->line[i].point[j].x) || (y2 != shape->line[i].point[j].y)) {
-	  msAddLine(&tmp, &line);
-	  line.numpoints = 0; /* new line */
-	}
+        if((x2 != shape->line[i].point[j].x) || (y2 != shape->line[i].point[j].y)) {
+          msAddLine(&tmp, &line);
+          line.numpoints = 0; /* new line */
+        }
       }
 
       x1 = shape->line[i].point[j].x;
@@ -565,94 +578,92 @@ void msClipPolygonRect(shapeObj *shape, rectObj rect)
       
       deltax = x2-x1;
       if (deltax == 0) { /* bump off of the vertical */
-	deltax = (x1 > rect.minx) ? -NEARZERO : NEARZERO ;
+        deltax = (x1 > rect.minx) ? -NEARZERO : NEARZERO ;
       }
       deltay = y2-y1;
       if (deltay == 0) { /* bump off of the horizontal */
-	deltay = (y1 > rect.miny) ? -NEARZERO : NEARZERO ;
+        deltay = (y1 > rect.miny) ? -NEARZERO : NEARZERO ;
       }
       
-      if (deltax > 0) {		/*  points to right */
-	xin = rect.minx;
-	xout = rect.maxx;
+      if (deltax > 0) { /*  points to right */
+        xin = rect.minx;
+        xout = rect.maxx;
+      } else {
+        xin = rect.maxx;
+        xout = rect.minx;
       }
-      else {
-	xin = rect.maxx;
-	xout = rect.minx;
-      }
-      if (deltay > 0) {		/*  points up */
-	yin = rect.miny;
-	yout = rect.maxy;
-      }
-      else {
-	yin = rect.maxy;
-	yout = rect.miny;
+      if (deltay > 0) { /*  points up */
+        yin = rect.miny;
+        yout = rect.maxy;
+      } else {
+        yin = rect.maxy;
+        yout = rect.miny;
       }
       
       tinx = (xin - x1)/deltax;
       tiny = (yin - y1)/deltay;
       
-      if (tinx < tiny) {	/* hits x first */
-	tin1 = tinx;
-	tin2 = tiny;
-      } else {			/* hits y first */
-	tin1 = tiny;
-	tin2 = tinx;
+      if (tinx < tiny) { /* hits x first */
+        tin1 = tinx;
+        tin2 = tiny;
+      } else {            /* hits y first */
+        tin1 = tiny;
+        tin2 = tinx;
       }
       
       if (1 >= tin1) {
-	if (0 < tin1) {
-	  line.point[line.numpoints].x = xin;
-	  line.point[line.numpoints].y = yin;
-	  line.numpoints++;
-	}
-	if (1 >= tin2) {
-	  toutx = (xout - x1)/deltax;
-	  touty = (yout - y1)/deltay;
-	  
-	  tout = (toutx < touty) ? toutx : touty ;
-	  
-	  if (0 < tin2 || 0 < tout) {
-	    if (tin2 <= tout) {
-	      if (0 < tin2) {
-		if (tinx > tiny) {
-		  line.point[line.numpoints].x = xin;
-		  line.point[line.numpoints].y = y1 + tinx*deltay;
-		  line.numpoints++;
-		} else {
-		  line.point[line.numpoints].x = x1 + tiny*deltax;
-		  line.point[line.numpoints].y = yin;
-		  line.numpoints++;
-		}
-	      }
-	      if (1 > tout) {
-		if (toutx < touty) {
-		  line.point[line.numpoints].x = xout;
-		  line.point[line.numpoints].y = y1 + toutx*deltay;
-		  line.numpoints++;
-		} else {
-		  line.point[line.numpoints].x = x1 + touty*deltax;
-		  line.point[line.numpoints].y = yout;
-		  line.numpoints++;
-		}
-	      }	else {
-		line.point[line.numpoints].x = x2;
-		line.point[line.numpoints].y = y2;
-		line.numpoints++;
-	      }
-	    } else {
-	      if (tinx > tiny) {
-		line.point[line.numpoints].x = xin;
-		line.point[line.numpoints].y = yout;
-		line.numpoints++;
-	      } else {
-		line.point[line.numpoints].x = xout;
-		line.point[line.numpoints].y = yin;
-		line.numpoints++;
-	      }
-	    }
-	  }
-	}
+        if (0 < tin1) {
+          line.point[line.numpoints].x = xin;
+          line.point[line.numpoints].y = yin;
+          line.numpoints++;
+        }
+        if (1 >= tin2) {
+          toutx = (xout - x1)/deltax;
+          touty = (yout - y1)/deltay;
+      
+          tout = (toutx < touty) ? toutx : touty ;
+      
+          if (0 < tin2 || 0 < tout) {
+            if (tin2 <= tout) {
+              if (0 < tin2) {
+                if (tinx > tiny) {
+                  line.point[line.numpoints].x = xin;
+                  line.point[line.numpoints].y = y1 + tinx*deltay;
+                  line.numpoints++;
+                } else {
+                  line.point[line.numpoints].x = x1 + tiny*deltax;
+                  line.point[line.numpoints].y = yin;
+                  line.numpoints++;
+                }
+              }
+              if (1 > tout) {
+                if (toutx < touty) {
+                  line.point[line.numpoints].x = xout;
+                  line.point[line.numpoints].y = y1 + toutx*deltay;
+                  line.numpoints++;
+                } else {
+                  line.point[line.numpoints].x = x1 + touty*deltax;
+                  line.point[line.numpoints].y = yout;
+                  line.numpoints++;
+                }
+              } else {
+                line.point[line.numpoints].x = x2;
+                line.point[line.numpoints].y = y2;
+                line.numpoints++;
+              }
+            } else {
+              if (tinx > tiny) {
+                line.point[line.numpoints].x = xin;
+                line.point[line.numpoints].y = yout;
+                line.numpoints++;
+              } else {
+                line.point[line.numpoints].x = xout;
+                line.point[line.numpoints].y = yin;
+                line.numpoints++;
+              }
+            }
+          }
+        }
       }
     }
 
@@ -682,7 +693,7 @@ void msOffsetPointRelativeTo(pointObj *point, layerObj *layer)
 {
   double x=0, y=0;
   if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
-	return;
+    return;
   
 
   if(layer->transform == MS_TRUE) return; /* nothing to do */
@@ -745,7 +756,7 @@ void msOffsetShapeRelativeTo(shapeObj *shape, layerObj *layer)
 
   if(layer->transform == MS_TRUE) return; /* nothing to do */
   if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
-	return;
+    return;
   
 
   if(layer->units == MS_PERCENTAGES) {
@@ -814,76 +825,56 @@ void msTransformShapeToPixel(shapeObj *shape, rectObj extent, double cellsize)
 
   if(shape->numlines == 0) return; /* nothing to transform */
 
-  if(shape->type == MS_SHAPE_LINE || shape->type == MS_SHAPE_POLYGON) { /* remove co-linear vertices */
-  
+  if(shape->type == MS_SHAPE_LINE || shape->type == MS_SHAPE_POLYGON) { /* remove duplicate vertices */
     for(i=0; i<shape->numlines; i++) { /* for each part */
-      
       shape->line[i].point[0].x = MS_MAP2IMAGE_X_IC(shape->line[i].point[0].x, extent.minx, inv_cs);
       shape->line[i].point[0].y = MS_MAP2IMAGE_Y_IC(shape->line[i].point[0].y, extent.maxy, inv_cs);
-      
       for(j=1, k=1; j < shape->line[i].numpoints; j++ ) {
-	
-	shape->line[i].point[k].x = MS_MAP2IMAGE_X_IC(shape->line[i].point[j].x, extent.minx, inv_cs);
-	shape->line[i].point[k].y = MS_MAP2IMAGE_Y_IC(shape->line[i].point[j].y, extent.maxy, inv_cs);
-
-	if(k == 1) {
-	  if((shape->line[i].point[0].x != shape->line[i].point[1].x) || (shape->line[i].point[0].y != shape->line[i].point[1].y))
-	    k++;
-	} else {
-	  if((shape->line[i].point[k-1].x != shape->line[i].point[k].x) || (shape->line[i].point[k-1].y != shape->line[i].point[k].y)) {
-	    if(((shape->line[i].point[k-2].y - shape->line[i].point[k-1].y)*(shape->line[i].point[k-1].x - shape->line[i].point[k].x)) == ((shape->line[i].point[k-2].x - shape->line[i].point[k-1].x)*(shape->line[i].point[k-1].y - shape->line[i].point[k].y))) {	    
-	      shape->line[i].point[k-1].x = shape->line[i].point[k].x;
-	      shape->line[i].point[k-1].y = shape->line[i].point[k].y;	
-	    } else {
-	      k++;
-	    }
-	  }
-	}
+        shape->line[i].point[k].x = MS_MAP2IMAGE_X_IC(shape->line[i].point[j].x, extent.minx, inv_cs);
+        shape->line[i].point[k].y = MS_MAP2IMAGE_Y_IC(shape->line[i].point[j].y, extent.maxy, inv_cs);
+        if(shape->line[i].point[k].x!=shape->line[i].point[k-1].x || shape->line[i].point[k].y!=shape->line[i].point[k-1].y)
+          k++;
       }
       shape->line[i].numpoints = k; /* save actual number kept */
     }
   } else { /* points or untyped shapes */
     for(i=0; i<shape->numlines; i++) { /* for each part */
       for(j=1; j < shape->line[i].numpoints; j++ ) {
-	shape->line[i].point[j].x = MS_MAP2IMAGE_X_IC(shape->line[i].point[j].x, extent.minx, inv_cs);
-	shape->line[i].point[j].y = MS_MAP2IMAGE_Y_IC(shape->line[i].point[j].y, extent.maxy, inv_cs);
-	  }
-	}
+        shape->line[i].point[j].x = MS_MAP2IMAGE_X_IC(shape->line[i].point[j].x, extent.minx, inv_cs);
+        shape->line[i].point[j].y = MS_MAP2IMAGE_Y_IC(shape->line[i].point[j].y, extent.maxy, inv_cs);
+      }
+    }
   }
 }
+
 /*
 ** Converts from map coordinates to image coordinates
 */
 void msTransformPixelToShape(shapeObj *shape, rectObj extent, double cellsize)
 {
-	int i,j; /* loop counters */
+  int i,j; /* loop counters */
 
-	if(shape->numlines == 0) return; /* nothing to transform */
+  if(shape->numlines == 0) return; /* nothing to transform */
 
-	if(shape->type == MS_SHAPE_LINE || shape->type == MS_SHAPE_POLYGON)  /* remove co-linear vertices */
-	{
-		for(i=0; i<shape->numlines; i++)  /* for each part */
-		{ 
-			for(j=0; j < shape->line[i].numpoints; j++ ) 
-			{
-				shape->line[i].point[j].x = MS_IMAGE2MAP_X(shape->line[i].point[j].x, extent.minx, cellsize);
-				shape->line[i].point[j].y = MS_IMAGE2MAP_Y(shape->line[i].point[j].y, extent.maxy, cellsize);
-			}
-		}
-	}
-	else  /* points or untyped shapes */
-	{
-		for(i=0; i<shape->numlines; i++)  /* for each part */
-		{
-			for(j=1; j < shape->line[i].numpoints; j++ ) 
-			{
-				shape->line[i].point[j].x = MS_IMAGE2MAP_X(shape->line[i].point[j].x, extent.minx, cellsize);
-				shape->line[i].point[j].y = MS_IMAGE2MAP_Y(shape->line[i].point[j].y, extent.maxy, cellsize);
-			}
-		}
-	}
+  if(shape->type == MS_SHAPE_LINE || shape->type == MS_SHAPE_POLYGON) { /* remove co-linear vertices */
 
-	return;
+    for(i=0; i<shape->numlines; i++) { /* for each part */         
+      for(j=0; j < shape->line[i].numpoints; j++ ) {
+        shape->line[i].point[j].x = MS_IMAGE2MAP_X(shape->line[i].point[j].x, extent.minx, cellsize);
+        shape->line[i].point[j].y = MS_IMAGE2MAP_Y(shape->line[i].point[j].y, extent.maxy, cellsize);
+      }
+    }
+  } else { /* points or untyped shapes */
+
+    for(i=0; i<shape->numlines; i++) { /* for each part */
+      for(j=1; j < shape->line[i].numpoints; j++ ) {
+        shape->line[i].point[j].x = MS_IMAGE2MAP_X(shape->line[i].point[j].x, extent.minx, cellsize);
+        shape->line[i].point[j].y = MS_IMAGE2MAP_Y(shape->line[i].point[j].y, extent.maxy, cellsize);
+      }
+    }
+  }
+
+  return;
 }
 
 /*
@@ -947,11 +938,11 @@ void bufferPolyline(shapeObj *p, shapeObj *op, int w)
 
       angle = asin(MS_ABS(p->line[i].point[j].x - p->line[i].point[j-1].x)/sqrt((pow((p->line[i].point[j].x - p->line[i].point[j-1].x),2.0) + pow((p->line[i].point[j].y - p->line[i].point[j-1].y),2.0))));
       if(p->line[i].point[j-1].x < p->line[i].point[j].x)
-	dy = sin(angle) * (w/2);
+        dy = sin(angle) * (w/2);
       else
-	dy = -sin(angle) * (w/2);
+        dy = -sin(angle) * (w/2);
       if(p->line[i].point[j-1].y < p->line[i].point[j].y)
-	dx = -cos(angle) * (w/2);
+        dx = -cos(angle) * (w/2);
       else
         dx = cos(angle) * (w/2);
 
@@ -980,9 +971,68 @@ void bufferPolyline(shapeObj *p, shapeObj *op, int w)
   return;
 }
 
-/* Currently unused. */
-#ifdef notdef
-static int get_centroid(shapeObj *p, pointObj *lp, double *miny, double *maxy)
+static double getRingArea(lineObj *ring)
+{
+  int i;
+  double s=0;
+
+  for(i=0; i<ring->numpoints-1; i++)
+    s += (ring->point[i].x*ring->point[i+1].y - ring->point[i+1].x*ring->point[i].y);
+
+  return (MS_ABS(s/2));
+}
+
+double msGetPolygonArea(shapeObj *p) 
+{
+  int i;
+  double area=0;
+
+  for(i=0; i<p->numlines; i++) {
+    if(isOuterRing(p, i))
+      area += getRingArea(&(p->line[i]));
+    else
+      area -= getRingArea(&(p->line[i])); /* hole */
+  }
+
+  return area;
+}
+
+/*
+** Computes the center of gravity for a polygon based on it's largest outer ring only.
+*/
+static int getPolygonCenterOfGravity(shapeObj *p, pointObj *lp) 
+{
+  int i, j;
+  double area=0;
+  double sx=0, sy=0, tsx, tsy, s; /* sums */
+  double a;
+
+  double largestArea=0;
+
+  for(i=0; i<p->numlines; i++) {
+    tsx = tsy = s = 0; /* reset the ring sums */
+    for(j=0; j<p->line[i].numpoints-1; j++) {
+      a = p->line[i].point[j].x*p->line[i].point[j+1].y - p->line[i].point[j+1].x*p->line[i].point[j].y;
+      s += a;
+      tsx += (p->line[i].point[j].x + p->line[i].point[j+1].x)*a;
+      tsy += (p->line[i].point[j].y + p->line[i].point[j+1].y)*a;
+    }
+    area = MS_ABS(s/2);
+
+    if(area > largestArea) {
+      largestArea = area;
+      sx = s>0?tsx:-tsx;
+      sy = s>0?tsy:-tsy;
+    }  
+  }
+
+  lp->x = sx/(6*largestArea);
+  lp->y = sy/(6*largestArea);
+
+  return MS_SUCCESS;
+}
+
+int msGetPolygonCentroid(shapeObj *p, pointObj *lp, double *miny, double *maxy)
 {
   int i,j;
   double cent_weight_x=0.0, cent_weight_y=0.0;
@@ -993,7 +1043,7 @@ static int get_centroid(shapeObj *p, pointObj *lp, double *miny, double *maxy)
     for(j=1; j<p->line[i].numpoints; j++) {
       *miny = MS_MIN(*miny, p->line[i].point[j].y);
       *maxy = MS_MAX(*maxy, p->line[i].point[j].y);
-      len = length(p->line[i].point[j-1], p->line[i].point[j]);
+      len = msDistancePointToPoint(&(p->line[i].point[j-1]), &(p->line[i].point[j]));
       cent_weight_x += len * ((p->line[i].point[j-1].x + p->line[i].point[j].x)/2);
       cent_weight_y += len * ((p->line[i].point[j-1].y + p->line[i].point[j].y)/2);
       total_len += len;
@@ -1001,30 +1051,31 @@ static int get_centroid(shapeObj *p, pointObj *lp, double *miny, double *maxy)
   }
 
   if(total_len == 0)
-    return(-1);
+    return(MS_FAILURE);
 
   lp->x = cent_weight_x / total_len;
   lp->y = cent_weight_y / total_len;
   
-  return(0);
+  return(MS_SUCCESS);
 }
-#endif
 
 #define NUM_SCANLINES 5
 
 /*
 ** Find a label point in a polygon.
 */
-int msPolygonLabelPoint(shapeObj *p, pointObj *lp, int min_dimension)
+int msPolygonLabelPoint(shapeObj *p, pointObj *lp, double min_dimension)
 {
   double slope;
-  pointObj *point1=NULL, *point2=NULL;
-  int i, j, k, nfound;
-  double x, y, *xintersect, temp;
-  double hi_y, lo_y;
+  pointObj *point1=NULL, *point2=NULL, cp;
+  int i, j, nfound;
+  double x, y, *intersect, temp;
+  double min, max;
   int wrong_order, n;
   double len, max_len=0;
-  double skip, minx, maxx, maxy, miny;
+  double minx, maxx, maxy, miny;
+
+  int method = 2;
 
   msComputeBounds(p);
   minx = p->bounds.minx;
@@ -1035,101 +1086,208 @@ int msPolygonLabelPoint(shapeObj *p, pointObj *lp, int min_dimension)
   if(min_dimension != -1)
     if(MS_MIN(maxx-minx,maxy-miny) < min_dimension) return(MS_FAILURE);
 
-  /* if(get_centroid(p, lp, &miny, &maxy) == -1) return(MS_FAILURE); */
-  lp->x = (maxx+minx)/2.0;
-  lp->y = (maxy+miny)/2.0;
+  cp.x = (maxx+minx)/2.0;
+  cp.y = (maxy+miny)/2.0;
 
-  if(msIntersectPointPolygon(lp, p) == MS_TRUE) return(MS_SUCCESS);
+  switch (method) {
+  case 0: /* MBR */
+    lp->x = cp.x;
+    lp->y = cp.y;
+    break;
+  case 1: /* centroid */
+    if(msGetPolygonCentroid(p, lp, &miny, &maxy) != MS_SUCCESS) return(MS_FAILURE);  
+    break;
+  case 2: /* center of gravity */
+    if(getPolygonCenterOfGravity(p, lp) != MS_SUCCESS) return(MS_FAILURE);
+    break;
+  }
 
-  /* do it the hard way - scanline */
+  if(msIntersectPointPolygon(lp, p) == MS_TRUE) {
+    double dist, min_dist=-1;
 
-  skip = (maxy - miny)/NUM_SCANLINES;
+    /* compute a distance to the polygon */
+    for(j=0;j<p->numlines;j++) {
+      for(i=1; i<p->line[j].numpoints; i++) {
+        dist = msDistancePointToSegment(lp, &(p->line[j].point[i-1]), &(p->line[j].point[i]));
+        if((dist < min_dist) || (min_dist < 0)) min_dist = dist;
+      }
+    }
+
+    if(min_dist > .1*MS_MAX(maxx-minx, maxy-miny))
+      return(MS_SUCCESS); /* point is not too close to the edge */
+  }
+
+  // printf("label: %s\n", p->text);
+  // printf("    bbox: %g %g %g %g\n",minx, miny, maxx, maxy);
+  // printf("    center: %g %g\n", cp.x, cp.y);
+  // printf("    center of gravity: %g %g\n", lp->x, lp->y);
+  // printf("    dx: %g, dy: %g\n", lp->x-cp.x, lp->y-cp.y);
+  // printf("    distance to parent shape: %g\n", min_dist);
+  // return MS_SUCCESS;
 
   n=0;
   for(j=0; j<p->numlines; j++) /* count total number of points */
     n += p->line[j].numpoints;
-  xintersect = (double *)calloc(n, sizeof(double));
+  if(!(intersect = (double *) calloc(n, sizeof(double)))) {
+    msSetError(MS_MEMERR, "Allocation of intersection array failed.", "msPolygonLabelPoint()");
+    return MS_FAILURE;
+  }
 
-  for(k=1; k<=NUM_SCANLINES; k++) { /* sample the shape in the y direction */
-    
-    y = maxy - k*skip; 
+  if(MS_ABS((int)lp->x - (int)cp.x) > MS_ABS((int)lp->y - (int)cp.y)) { /* center horizontally, fix y */
+
+    y = lp->y;
 
     /* need to find a y that won't intersect any vertices exactly */  
-    hi_y = y - 1; /* first initializing lo_y, hi_y to be any 2 pnts on either side of lp->y */
-    lo_y = y + 1;
+    max = y - 1; /* first initializing min, max to be any 2 pnts on either side of y */
+    min = y + 1;
     for(j=0; j<p->numlines; j++) {
-      if((lo_y < y) && (hi_y >= y)) 
-	break; /* already initialized */
+      if((min < y) && (max >= y))  break;
       for(i=0; i < p->line[j].numpoints; i++) {
-	if((lo_y < y) && (hi_y >= y)) 
-	  break; /* already initialized */
-	if(p->line[j].point[i].y < y)
-	  lo_y = p->line[j].point[i].y;
-	if(p->line[j].point[i].y >= y)
-	  hi_y = p->line[j].point[i].y;
+        if((min < y) && (max >= y))  break;
+        if(p->line[j].point[i].y < y) 
+          min = p->line[j].point[i].y;
+        if(p->line[j].point[i].y >= y) 
+          max = p->line[j].point[i].y;
       }
     }
 
     n=0;
     for(j=0; j<p->numlines; j++) {
       for(i=0; i < p->line[j].numpoints; i++) {
-	if((p->line[j].point[i].y < y) && ((y - p->line[j].point[i].y) < (y - lo_y)))
-	  lo_y = p->line[j].point[i].y;
-	if((p->line[j].point[i].y >= y) && ((p->line[j].point[i].y - y) < (hi_y - y)))
-	  hi_y = p->line[j].point[i].y;
-      }      
+        if((p->line[j].point[i].y < y) && ((y - p->line[j].point[i].y) < (y - min)))
+          min = p->line[j].point[i].y;
+        if((p->line[j].point[i].y >= y) && ((p->line[j].point[i].y - y) < (max - y)))
+          max = p->line[j].point[i].y;
+      }
     }
 
-    if(lo_y == hi_y) 
+    if(min == max) 
       return (MS_FAILURE);
     else  
-      y = (hi_y + lo_y)/2.0;    
+      y = (max + min)/2.0;    
     
     nfound = 0;
     for(j=0; j<p->numlines; j++) { /* for each line */
       
       point1 = &( p->line[j].point[p->line[j].numpoints-1] );
       for(i=0; i < p->line[j].numpoints; i++) {
-	point2 = &( p->line[j].point[i] );
-	
-	if(EDGE_CHECK(point1->y, y, point2->y) == CLIP_MIDDLE) {
-	  
-	  if(point1->y == point2->y)
-	    continue; /* ignore horizontal edges */
-	  else	  
-	    slope = (point2->x - point1->x) / (point2->y - point1->y);
-	  
-	  x = point1->x + (y - point1->y)*slope;
-	  xintersect[nfound++] = x;
-	} /* End of checking this edge */
-	
-	point1 = point2;  /* Go on to next edge */
-      }
-    } /* Finished the scanline */
+        point2 = &( p->line[j].point[i] );
     
-    /* First, sort the intersections */
+        if(EDGE_CHECK(point1->y, y, point2->y) == CLIP_MIDDLE) {
+      
+          if(point1->y == point2->y)
+            continue; /* ignore horizontal edges */
+          else      
+            slope = (point2->x - point1->x) / (point2->y - point1->y);
+      
+          x = point1->x + (y - point1->y)*slope;
+          intersect[nfound++] = x;
+        } /* end checking this edge */
+    
+        point1 = point2; /* next edge */
+      }
+    } /* finished line */
+    
+    /* sort the intersections */
     do {
       wrong_order = 0;
       for(i=0; i < nfound-1; i++) {
-	if(xintersect[i] > xintersect[i+1]) {
-	  wrong_order = 1;
-	  SWAP(xintersect[i], xintersect[i+1], temp);
-	}
+        if(intersect[i] > intersect[i+1]) {
+          wrong_order = 1;
+          SWAP(intersect[i], intersect[i+1], temp);
+        }
       }
     } while(wrong_order);
     
-    /* Great, now find longest span */
+    /* find longest span */
     for(i=0; i < nfound; i += 2) {
-      len = fabs(xintersect[i] - xintersect[i+1]);
+      len = fabs(intersect[i] - intersect[i+1]);
       if(len > max_len) {
-	max_len = len;
-	lp->x = (xintersect[i] + xintersect[i+1])/2;
-	lp->y = y;
+        max_len = len;
+        lp->x = (intersect[i] + intersect[i+1])/2;
+        // lp->y = y;
+      }
+    }
+  } else { /* center vertically, fix x */
+    x = lp->x;
+
+    /* need to find a x that won't intersect any vertices exactly */  
+    max = x - 1; /* first initializing min, max to be any 2 pnts on either side of x */
+    min = x + 1;
+    for(j=0; j<p->numlines; j++) {
+      if((min < x) && (max >= x))  break;
+      for(i=0; i < p->line[j].numpoints; i++) {
+        if((min < x) && (max >= x))  break;
+        if(p->line[j].point[i].x < x) 
+          min = p->line[j].point[i].x;
+        if(p->line[j].point[i].x >= x) 
+          max = p->line[j].point[i].x;
+      }
+    }
+
+    n=0;
+    for(j=0; j<p->numlines; j++) {
+      for(i=0; i < p->line[j].numpoints; i++) {
+        if((p->line[j].point[i].x < x) && ((x - p->line[j].point[i].x) < (x - min)))
+          min = p->line[j].point[i].x;
+        if((p->line[j].point[i].x >= x) && ((p->line[j].point[i].x - x) < (max - x)))
+          max = p->line[j].point[i].x;
+      }
+    }
+
+    if(min == max) 
+      return (MS_FAILURE);
+    else  
+      x = (max + min)/2.0;
+    
+    nfound = 0;
+    for(j=0; j<p->numlines; j++) { /* for each line */
+      
+      point1 = &( p->line[j].point[p->line[j].numpoints-1] );
+      for(i=0; i < p->line[j].numpoints; i++) {
+        point2 = &( p->line[j].point[i] );
+    
+        if(EDGE_CHECK(point1->x, x, point2->x) == CLIP_MIDDLE) {
+      
+          if(point1->x == point2->x)
+            continue; /* ignore vertical edges */
+          else if(point1->y == point2->y)
+            y = point1->y; /* for a horizontal edge we know y */
+          else {
+            slope = (point2->x - point1->x) / (point2->y - point1->y);
+            y = (x - point1->x)/slope + point1->y;
+          }
+
+          intersect[nfound++] = y;
+        } /* end checking this edge */
+    
+        point1 = point2; /* next edge */
+      }
+    } /* finished line */
+    
+    /* sort the intersections */
+    do {
+      wrong_order = 0;
+      for(i=0; i < nfound-1; i++) {
+        if(intersect[i] > intersect[i+1]) {
+          wrong_order = 1;
+          SWAP(intersect[i], intersect[i+1], temp);
+        }
+      }
+    } while(wrong_order);
+    
+    /* find longest span */
+    for(i=0; i < nfound; i += 2) {
+      len = fabs(intersect[i] - intersect[i+1]);
+      if(len > max_len) {        
+        max_len = len;
+        lp->y = (intersect[i] + intersect[i+1])/2;
+        // lp->x = x;
       }
     }
   }
 
-  free(xintersect);
+  free(intersect);
 
   if(max_len > 0)
     return(MS_SUCCESS);
@@ -1137,6 +1295,7 @@ int msPolygonLabelPoint(shapeObj *p, pointObj *lp, int min_dimension)
     return(MS_FAILURE);
 }
 
+
 /*
 ** Find center of longest segment in polyline p. The polyline must have been converted
 ** to image coordinates before calling this function.
@@ -1160,8 +1319,8 @@ int msPolylineLabelPoint(shapeObj *p, pointObj *lp, int min_length, double *angl
       segment_length = sqrt((pow((p->line[i].point[j].x-p->line[i].point[j-1].x),2.0) + pow((p->line[i].point[j].y-p->line[i].point[j-1].y),2.0)));
       line_length += segment_length;
       if(segment_length > max_segment_length) {
-	max_segment_length = segment_length;
-	temp_segment_index = j;
+        max_segment_length = segment_length;
+        temp_segment_index = j;
       }
     }
 
@@ -1213,7 +1372,7 @@ int msPolylineLabelPoint(shapeObj *p, pointObj *lp, int min_length, double *angl
  * polyline must be converted to image coordinates before calling this
  * function.
  */
-labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fontset, char *string, labelObj *label, double scalefactor, int *status)
+labelPathObj* msPolylineLabelPath(imageObj *img,shapeObj *p, int min_length, fontSetObj *fontset, char *string, labelObj *label, double scalefactor, int *status)
 {
   double line_length, max_line_length, segment_length, total_length, distance_along_segment;
   double fwd_line_length, rev_line_length, text_length, text_start_length;
@@ -1252,14 +1411,8 @@ labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fonts
   goto FAILURE; /* we don't have a current enough version of GD, fall back to ANGLE AUTO */
 #else
 
-  /* If the labelObj has an encodiing set then we expect UTF-8 as input, otherwise
-   * we treat the string as a regular array of chars 
-   */
-  if (label->encoding)
-      numchars = msGetNumUTF8Chars(string);
-  else
-      numchars = strlen(string);
-
+  numchars = msGetNumGlyphs(string);
+  
   /* skip the label and use the normal algorithm if it has fewer than 2 characters */
   if(numchars < 2)
     goto FAILURE;
@@ -1267,7 +1420,7 @@ labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fonts
   segment_index = line_index = 0;
   total_length = max_line_length = 0.0;
   
-  /* determine longest line */
+  /* determine longest line segment */
   segment_lengths = (double **) malloc(sizeof(double *) * p->numlines);
   for(i=0; i<p->numlines; i++) {
     
@@ -1305,7 +1458,7 @@ labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fonts
     goto FAILURE;
   
   /* determine the total length of the text */
-  if (msGetLabelSizeEx(string, label, &bbox, fontset, scalefactor, MS_FALSE, &offsets) == MS_FAILURE) {
+  if (msGetLabelSize(img,string, label, &bbox, fontset, scalefactor, MS_FALSE, &offsets) == -1) {
     *status = MS_FAILURE;
     goto FAILURE;
   }
@@ -1607,20 +1760,16 @@ FAILURE:
    Pretty printing of primitive objects
    ======================================================================== */
 
-void msRectToFormattedString(rectObj *rect, char *format, char *buffer, 
-                             int buffer_length) 
+void msRectToFormattedString(rectObj *rect, char *format, char *buffer, int buffer_length) 
 {
-    snprintf(buffer, buffer_length, format,
-             rect->minx, rect->miny, rect->maxx, rect->maxy);
+  snprintf(buffer, buffer_length, format, rect->minx, rect->miny, rect->maxx, rect->maxy);
 }
 
-void msPointToFormattedString(pointObj *point, const char *format, 
-                              char *buffer, int buffer_length) 
+void msPointToFormattedString(pointObj *point, const char *format, char *buffer, int buffer_length) 
 {
 #ifdef USE_POINT_Z_M
-    snprintf(buffer, buffer_length, format, point->x, point->y, point->z, point->m);
+  snprintf(buffer, buffer_length, format, point->x, point->y, point->z, point->m);
 #else
-    snprintf(buffer, buffer_length, format, point->x, point->y);
+  snprintf(buffer, buffer_length, format, point->x, point->y);
 #endif
 }
-
