@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapproject.h 7504 2008-04-08 21:13:45Z nsavard $
  *
  * Project:  MapServer
  * Purpose:  Declarations of the projectionObj and related functions.
@@ -74,6 +75,10 @@ MS_DLL_EXPORT int msInitProjection(projectionObj *p);
 MS_DLL_EXPORT int msProcessProjection(projectionObj *p);
 MS_DLL_EXPORT int msLoadProjectionString(projectionObj *p, char *value);
 MS_DLL_EXPORT char *msGetProjectionString(projectionObj *proj);
+MS_DLL_EXPORT void msAxisNormalizePoints( projectionObj *proj, int count,
+                                          double *x, double *y );
+MS_DLL_EXPORT void msAxisDenormalizePoints( projectionObj *proj, int count,
+                                            double *x, double *y );
 
 MS_DLL_EXPORT void msSetPROJ_LIB( const char * );
 
