@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapquery.c 9000 2009-05-02 13:49:08Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  layer query support.
@@ -28,7 +29,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapquery.c 7559 2008-05-05 14:46:11Z pramsey $")
+MS_CVSID("$Id: mapquery.c 9000 2009-05-02 13:49:08Z sdlime $")
 
 /*
 ** msIsLayerQueryable()  returns MS_TRUE/MS_FALSE
@@ -120,6 +121,11 @@ int msLoadQuery(mapObj *map, char *filename) {
     return(MS_FAILURE);
   }
 
+  /* 
+  ** Make sure the file at least has the right extension. 
+  */ 
+  if(msEvalRegex("\\.qy$", filename) != MS_TRUE) return MS_FAILURE; 
+
   stream = fopen(filename, "rb");
   if(!stream) {
     msSetError(MS_IOERR, "(%s)", "msLoadQuery()", filename);
@@ -207,7 +213,7 @@ int _msQueryByIndex(mapObj *map, int qlayer, int tileindex, int shapeindex,
     return(MS_FAILURE);
   }
 
-  shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+  shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom, NULL, 0);
   if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
     msSetError(MS_NOTFOUND, "Shape %d not valid against layer classification.", "msQueryByIndex()", shapeindex);
     msFreeShape(&shape);
@@ -247,6 +253,25 @@ int msQueryByIndex(mapObj *map, int qlayer, int tileindex, int shapeindex)
     return _msQueryByIndex(map, qlayer, tileindex, shapeindex, 0);
 }
 
+void msRestoreOldFilter(layerObj *lp, int old_filtertype, char *old_filteritem, char *old_filterstring)
+{
+  freeExpression(&(lp->filter));
+  if(lp->filteritem) {
+    free(lp->filteritem);
+    lp->filteritem = NULL;
+    lp->filteritemindex = -1;
+  }
+  
+  /* restore any previously defined filter */
+  if(old_filterstring) {
+    lp->filter.type = old_filtertype;
+    lp->filter.string = old_filterstring;
+    if(old_filteritem) { 
+      lp->filteritem = old_filteritem;
+    }
+  }
+}
+
 int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int mode)
 {
   layerObj *lp;
@@ -258,6 +283,9 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
   rectObj searchrect;
 
   shapeObj shape;
+  
+  int nclasses = 0;
+  int *classgroup = NULL;
 
   if(qlayer < 0 || qlayer >= map->numlayers) {
     msSetError(MS_MISCERR, "No query layer defined.", "msQueryByAttributes()"); 
@@ -306,11 +334,19 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
 
   /* open this layer */
   status = msLayerOpen(lp);
-  if(status != MS_SUCCESS) return(MS_FAILURE);
+  if(status != MS_SUCCESS) {
+      /* Manually reset the filter */
+      msRestoreOldFilter(lp,old_filtertype,old_filteritem,old_filterstring);
+      return(MS_FAILURE);
+  }
   
   /* build item list (no annotation) */
   status = msLayerWhichItems(lp, MS_TRUE, MS_FALSE, NULL);
-  if(status != MS_SUCCESS) return(MS_FAILURE);
+  if(status != MS_SUCCESS) {
+      /* Manually reset the filter */
+      msRestoreOldFilter(lp,old_filtertype,old_filteritem,old_filterstring);
+      return(MS_FAILURE);
+  }
 
   /* identify target shapes */
   searchrect = map->extent;
@@ -320,12 +356,17 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
   else
     lp->project = MS_FALSE;
 #endif
+
   status = msLayerWhichShapes(lp, searchrect);
   if(status == MS_DONE) { /* no overlap */
+    /* Manually reset the filter */
+    msRestoreOldFilter(lp,old_filtertype,old_filteritem,old_filterstring);
     msLayerClose(lp);
     msSetError(MS_NOTFOUND, "No matching record(s) found, layer and area of interest do not overlap.", "msQueryByAttributes()");
     return(MS_FAILURE);
   } else if(status != MS_SUCCESS) {
+    /* Manually reset the filter */
+    msRestoreOldFilter(lp,old_filtertype,old_filteritem,old_filterstring);
     msLayerClose(lp);
     return(MS_FAILURE);
   }
@@ -335,14 +376,19 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
   lp->resultcache->numresults = lp->resultcache->cachesize = 0;
   lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
   
+  nclasses = 0;
+  classgroup = NULL;
+  if (lp->classgroup && lp->numclasses > 0)
+      classgroup = msAllocateValidClassGroups(lp, &nclasses);
+
   while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
-    shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);    
+    shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom, classgroup, nclasses );    
     if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
       msFreeShape(&shape);
       continue;
     }
-    
+
     if(!(lp->template) && !(lp->class[shape.classindex]->template)) { /* no valid template */
       msFreeShape(&shape);
       continue;
@@ -370,27 +416,18 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
     }
   }
 
-  if(status != MS_DONE) return(MS_FAILURE);
+  if (classgroup)
+    msFree(classgroup);
 
-  /* the FILTER set was just temporary, clean up here */
-  freeExpression(&(lp->filter));
-  if(lp->filteritem) {
-    free(lp->filteritem);
-    lp->filteritem = NULL;
-    lp->filteritemindex = -1;
-  }
-  
-  /* restore any previously defined filter */
-  if(old_filterstring) {
-    lp->filter.type = old_filtertype;
-    lp->filter.string = strdup(old_filterstring);
-    free(old_filterstring);
-    if(old_filteritem) { 
-      lp->filteritem = strdup(old_filteritem);
-      free(old_filteritem);
-    }
+  if(status != MS_DONE){
+     /* Manually reset the filter */
+     msRestoreOldFilter(lp,old_filtertype,old_filteritem,old_filterstring);
+     return(MS_FAILURE);
   }
 
+  /* Manually reset the filter */
+  msRestoreOldFilter(lp,old_filtertype,old_filteritem,old_filterstring);
+
   msLayerClose(lp);
 
   /* was anything found?  */
@@ -411,6 +448,9 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
   char status;
   shapeObj shape, searchshape;
   rectObj searchrect;
+  
+  int nclasses = 0;
+  int *classgroup = NULL;
 
   msInitShape(&shape);
   msInitShape(&searchshape);
@@ -442,7 +482,12 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
     if(map->scaledenom > 0) {
       if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom)) continue;
       if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
-    }    
+    }
+
+    if (lp->maxscaledenom <= 0 && lp->minscaledenom <= 0) {
+      if((lp->maxgeowidth > 0) && ((map->extent.maxx - map->extent.minx) > lp->maxgeowidth)) continue;
+      if((lp->mingeowidth > 0) && ((map->extent.maxx - map->extent.minx) < lp->mingeowidth)) continue;
+    }
 
     /* Raster layers are handled specially. */
     if( lp->type == MS_LAYER_RASTER ) {
@@ -482,9 +527,14 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
     lp->resultcache->numresults = lp->resultcache->cachesize = 0;
     lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
 
+    nclasses = 0;
+    classgroup = NULL;
+    if (lp->classgroup && lp->numclasses > 0)
+      classgroup = msAllocateValidClassGroups(lp, &nclasses);
+
     while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
-      shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+      shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom, classgroup, nclasses);
       if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
 	      msFreeShape(&shape);
 	      continue;
@@ -532,6 +582,9 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
       msFreeShape(&shape);
     } /* next shape */
       
+    if (classgroup)
+      msFree(classgroup);
+
     if(status != MS_DONE) return(MS_FAILURE);
 
     msLayerClose(lp);
@@ -569,6 +622,8 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
 
   rectObj searchrect;
   shapeObj shape, selectshape;
+  int nclasses = 0;
+  int *classgroup = NULL;
 
   msInitShape(&shape);
   msInitShape(&selectshape);
@@ -623,15 +678,20 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
       if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }
 
+    if (lp->maxscaledenom <= 0 && lp->minscaledenom <= 0) {
+      if((lp->maxgeowidth > 0) && ((map->extent.maxx - map->extent.minx) > lp->maxgeowidth)) continue;
+      if((lp->mingeowidth > 0) && ((map->extent.maxx - map->extent.minx) < lp->mingeowidth)) continue;
+    }
+
     /* Get the layer tolerance
        default is 3 for point and line layers, 0 for others */
-    if(lp->tolerance == -1)
-        if(lp->status == MS_LAYER_POINT || lp->status == MS_LAYER_LINE)
-            layer_tolerance = 3;
-        else
-            layer_tolerance = 0;
-    else
-        layer_tolerance = lp->tolerance;
+    if(lp->tolerance == -1) {
+      if(lp->type == MS_LAYER_POINT || lp->type == MS_LAYER_LINE)
+        layer_tolerance = 3;
+       else
+        layer_tolerance = 0;
+    } else
+      layer_tolerance = lp->tolerance;
   
     if(lp->toleranceunits == MS_PIXELS)
       tolerance = layer_tolerance * msAdjustExtent(&(map->extent), map->width, map->height);
@@ -656,10 +716,10 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
 	return(MS_FAILURE);
       }
 
-      if(selectshape.type != MS_SHAPE_POLYGON) {
+      if(selectshape.type != MS_SHAPE_POLYGON && selectshape.type != MS_SHAPE_LINE) {
 	msLayerClose(lp);
 	msLayerClose(slp);
-	msSetError(MS_QUERYERR, "Selection features MUST be polygons.", "msQueryByFeatures()");
+	msSetError(MS_QUERYERR, "Selection features MUST be polygons or lines.", "msQueryByFeatures()");
 	return(MS_FAILURE);
       }
       
@@ -703,12 +763,17 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
 	lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;    
       }      
 
+      nclasses = 0;
+      classgroup = NULL;
+      if (lp->classgroup && lp->numclasses > 0)
+        classgroup = msAllocateValidClassGroups(lp, &nclasses);
+
       while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
 	/* check for dups when there are multiple selection shapes */
 	if(i > 0 && is_duplicate(lp->resultcache, shape.index, shape.tileindex)) continue;
 
-	shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+	shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom, classgroup, nclasses);
 	if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
 	  msFreeShape(&shape);
 	  continue;
@@ -726,9 +791,8 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
 	  lp->project = MS_FALSE;
 #endif
  
-	switch(selectshape.type) { /* may eventually support types other than polygon */
-	case MS_SHAPE_POLYGON:
-	  
+	switch(selectshape.type) { /* may eventually support types other than polygon on line */
+	case MS_SHAPE_POLYGON:	  
 	  switch(shape.type) { /* make sure shape actually intersects the selectshape */
 	  case MS_SHAPE_POINT:
 	    if(tolerance == 0) /* just test for intersection */
@@ -755,26 +819,61 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
             }
 	    break;
 	  default:
+            status = MS_FALSE;
 	    break;
 	  }
-	  
-	  if(status == MS_TRUE) {
-	    addResult(lp->resultcache, shape.classindex, shape.index, shape.tileindex);
-	    
-	    if(lp->resultcache->numresults == 1)
-	      lp->resultcache->bounds = shape.bounds;
-	    else
-	      msMergeRect(&(lp->resultcache->bounds), &shape.bounds);
-	  }
-
 	  break;
+	case MS_SHAPE_LINE:
+          switch(shape.type) { /* make sure shape actually intersects the selectshape */
+          case MS_SHAPE_POINT:
+            if(tolerance == 0) { /* just test for intersection */
+              distance = msDistanceShapeToShape(&selectshape, &shape);
+              if(distance == 0) status = MS_TRUE;
+            } else {
+	      distance = msDistanceShapeToShape(&selectshape, &shape);
+              if(distance < tolerance) status = MS_TRUE;
+            }
+            break;
+          case MS_SHAPE_LINE:
+            if(tolerance == 0) { /* just test for intersection */
+              status = msIntersectPolylines(&shape, &selectshape);
+            } else { /* check distance, distance=0 means they intersect */
+              distance = msDistanceShapeToShape(&selectshape, &shape);
+              if(distance < tolerance) status = MS_TRUE;
+            }
+            break;
+          case MS_SHAPE_POLYGON:
+            if(tolerance == 0) /* just test for intersection */
+              status = msIntersectPolylinePolygon(&selectshape, &shape);
+            else { /* check distance, distance=0 means they intersect */
+              distance = msDistanceShapeToShape(&selectshape, &shape);
+              if(distance < tolerance) status = MS_TRUE;
+            }
+            break;
+          default:
+            status = MS_FALSE;
+            break;
+          }
+          break;
 	default:
-	  break;
+	  break; /* should never get here as we test for selection shape type explicitly earlier */
+	}
+
+	if(status == MS_TRUE) {
+	  addResult(lp->resultcache, shape.classindex, shape.index, shape.tileindex);
+
+	  if(lp->resultcache->numresults == 1)
+	    lp->resultcache->bounds = shape.bounds;
+	  else
+	    msMergeRect(&(lp->resultcache->bounds), &shape.bounds);
 	}
 
 	msFreeShape(&shape);
       } /* next shape */
 
+      if (classgroup)
+        msFree(classgroup);
+
       if(status != MS_DONE) return(MS_FAILURE);
 
       msFreeShape(&selectshape);
@@ -797,7 +896,21 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
   return(MS_FAILURE);
 }
 
-int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
+/* msQueryByPoint()
+ *
+ * With mode=MS_SINGLE:
+ *   Pass maxresults = 0 to have a single result across all layers (the closest
+ *   shape from the first layer that finds a match).
+ *   Pass maxresults = 1 to have up to one result per layer (the closest shape
+ *   from each layer).
+ *
+ * With mode=MS_MULTIPLE:
+ *   Pass maxresults = 0 to have an unlimited number of results.
+ *   Pass maxresults > 0 to limit the number of results per layer (the shapes
+ *   returned are the first ones found in each layer and are not necessarily
+ *   the closest ones).
+ */
+int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer, int maxresults)
 {
   int l;
   int start, stop=0;
@@ -810,6 +923,8 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
   char status;
   rectObj rect, searchrect;
   shapeObj shape;
+  int nclasses = 0;
+  int *classgroup = NULL;
 
   msInitShape(&shape);
 
@@ -840,6 +955,11 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
       if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }
 
+    if (lp->maxscaledenom <= 0 && lp->minscaledenom <= 0) {
+      if((lp->maxgeowidth > 0) && ((map->extent.maxx - map->extent.minx) > lp->maxgeowidth)) continue;
+      if((lp->mingeowidth > 0) && ((map->extent.maxx - map->extent.minx) < lp->mingeowidth)) continue;
+    }
+
     /* Raster layers are handled specially.  */
     if( lp->type == MS_LAYER_RASTER ) {
         if( msRasterQueryByPoint( map, lp, mode, p, buffer )
@@ -851,7 +971,7 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
     /* Get the layer tolerance
        default is 3 for point and line layers, 0 for others */
     if(lp->tolerance == -1)
-        if(lp->status == MS_LAYER_POINT || lp->status == MS_LAYER_LINE)
+        if(lp->type == MS_LAYER_POINT || lp->type == MS_LAYER_LINE)
             layer_tolerance = 3;
         else
             layer_tolerance = 0;
@@ -901,9 +1021,14 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
     lp->resultcache->numresults = lp->resultcache->cachesize = 0;
     lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
 
+    nclasses = 0;
+    classgroup = NULL;
+    if (lp->classgroup && lp->numclasses > 0)
+      classgroup = msAllocateValidClassGroups(lp, &nclasses);
+
     while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
-      shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+      shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom, classgroup, nclasses);
       if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
 	msFreeShape(&shape);
 	continue;
@@ -937,15 +1062,24 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
 	}
       }
  
-      msFreeShape(&shape);	
+      msFreeShape(&shape);
+
+      if (mode == MS_MULTIPLE && maxresults > 0 && lp->resultcache->numresults == maxresults) {
+          status = MS_DONE;   /* got enough results for this layer */
+          break;
+      }
+
     } /* next shape */
 
+    if (classgroup)
+      msFree(classgroup);
+
     if(status != MS_DONE) return(MS_FAILURE);
 
     msLayerClose(lp);
 
-    if((lp->resultcache->numresults > 0) && (mode == MS_SINGLE)) /* no need to search any further */
-      break;
+    if((lp->resultcache->numresults > 0) && (mode == MS_SINGLE) && (maxresults == 0)) 
+      break;   /* no need to search any further */
   } /* next layer */
 
   /* was anything found? */
@@ -967,6 +1101,9 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
   double distance, tolerance, layer_tolerance;
   rectObj searchrect;
 
+  int nclasses = 0;
+  int *classgroup = NULL;
+
    msInitShape(&shape);
 
   /* FIX: do some checking on selectshape here... */
@@ -1004,6 +1141,11 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
       if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }
 
+    if (lp->maxscaledenom <= 0 && lp->minscaledenom <= 0) {
+      if((lp->maxgeowidth > 0) && ((map->extent.maxx - map->extent.minx) > lp->maxgeowidth)) continue;
+      if((lp->mingeowidth > 0) && ((map->extent.maxx - map->extent.minx) < lp->mingeowidth)) continue;
+    }
+
     /* Raster layers are handled specially. */
     if( lp->type == MS_LAYER_RASTER )
     {
@@ -1017,7 +1159,7 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
     /* Get the layer tolerance
        default is 3 for point and line layers, 0 for others */
     if(lp->tolerance == -1)
-        if(lp->status == MS_LAYER_POINT || lp->status == MS_LAYER_LINE)
+        if(lp->type == MS_LAYER_POINT || lp->type == MS_LAYER_LINE)
             layer_tolerance = 3;
         else
             layer_tolerance = 0;
@@ -1065,9 +1207,14 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
     lp->resultcache->numresults = lp->resultcache->cachesize = 0;
     lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
 
+    nclasses = 0;
+    classgroup = NULL;
+    if (lp->classgroup && lp->numclasses > 0)
+      classgroup = msAllocateValidClassGroups(lp, &nclasses);
+
     while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
-      shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+      shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom, classgroup, nclasses);
       if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
 	msFreeShape(&shape);
 	continue;
@@ -1145,7 +1292,7 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
 /************************************************************************/
 /*                            msQueryByOperator                         */
 /*                                                                      */
-/*      uary using a shape and a valid operator : all queries are       */
+/*      query using a shape and a valid operator : all queries are       */
 /*      done using geos.                                                */
 /************************************************************************/
 int msQueryByOperator(mapObj *map, int qlayer, shapeObj *selectshape,
@@ -1155,14 +1302,15 @@ int msQueryByOperator(mapObj *map, int qlayer, shapeObj *selectshape,
     int start, stop=0, l;
     shapeObj shape;
     layerObj *lp;
-    char status;
+    int status;
     rectObj searchrect;
+     
+    int nclasses = 0;
+    int *classgroup = NULL;
+    double dfValue;
 
     msInitShape(&shape);
 
-    /*todo call should be moved somewhere else*/
-    msGEOSSetup();
-
     /* this should not be a necessary test for uries using geos*/
     /*
       if(selectshape->type != MS_SHAPE_POLYGON) {
@@ -1200,6 +1348,11 @@ int msQueryByOperator(mapObj *map, int qlayer, shapeObj *selectshape,
         if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
       }
 
+      if (lp->maxscaledenom <= 0 && lp->minscaledenom <= 0) {
+        if((lp->maxgeowidth > 0) && ((map->extent.maxx - map->extent.minx) > lp->maxgeowidth)) continue;
+        if((lp->mingeowidth > 0) && ((map->extent.maxx - map->extent.minx) < lp->mingeowidth)) continue;
+      }
+
       /* Raster layers are handled specially. */
       if( lp->type == MS_LAYER_RASTER )
       {
@@ -1245,9 +1398,13 @@ int msQueryByOperator(mapObj *map, int qlayer, shapeObj *selectshape,
       lp->resultcache->numresults = lp->resultcache->cachesize = 0;
       lp->resultcache->bounds.minx = lp->resultcache->bounds.miny = lp->resultcache->bounds.maxx = lp->resultcache->bounds.maxy = -1;
 
+       classgroup = NULL;
+       if (lp->classgroup && lp->numclasses > 0)
+         classgroup = msAllocateValidClassGroups(lp, &nclasses);
+
       while((status = msLayerNextShape(lp, &shape)) == MS_SUCCESS) { /* step through the shapes */
 
-        shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
+        shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom, classgroup, nclasses);
         if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
           msFreeShape(&shape);
           continue;
@@ -1310,19 +1467,27 @@ int msQueryByOperator(mapObj *map, int qlayer, shapeObj *selectshape,
             break;
           
           case MS_GEOS_CONTAINS:
-            status = msGEOSContains(&shape, selectshape);
+            status = msGEOSContains(selectshape, &shape);
             if (status != MS_TRUE && status != MS_FALSE)
               status = MS_FALSE;
             break;
 
-            /*beyond is opposite of dwith use in filter encoding
-              see ticket 2105*/
+            /*beyond is opposite of dwithin use in filter encoding
+              see ticket 2105, 2564*/
           case MS_GEOS_BEYOND:
-            status = msGEOSWithin(&shape, selectshape);
-            if (status != MS_TRUE && status != MS_FALSE)
-              status = MS_FALSE;
-            else
-              status = !status;
+            status = MS_FALSE;
+            dfValue = msGEOSDistance(&shape, selectshape);
+            if (dfValue > 0.0)
+               status = MS_TRUE;
+          break;
+
+          /*dwithin is used with filter encoding (#2564)*/
+          case MS_GEOS_DWITHIN:
+            status = MS_FALSE;
+            dfValue = msGEOSDistance(&shape, selectshape);
+            if (dfValue == 0.0)
+              status = MS_TRUE;
+
           break;
 
           default:
@@ -1347,9 +1512,6 @@ int msQueryByOperator(mapObj *map, int qlayer, shapeObj *selectshape,
       msLayerClose(lp);
     } /* next layer */
 
-    /*todo call should be moved somewhere else*/
-    msGEOSCleanup();
-
     /* was anything found? */
     for(l=start; l>=stop; l--) {    
       if(GET_LAYER(map, l)->resultcache && GET_LAYER(map, l)->resultcache->numresults > 0)
