@@ -1,10 +1,12 @@
 /******************************************************************************
+ * $Id: mapraster.c 9055 2009-05-29 03:29:47Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  msDrawRasterLayer(): generic raster layer drawing, including
  *           implementation of non-GDAL raster layer renderers.
- * Author:   Steve Lime 
- *           Frank Warmerdam, warmerdam@pobox.com
+ * Author:   Frank Warmerdam, warmerdam@pobox.com
+ *           Pete Olson (LMIC)            
+ *           Steve Lime
  *
  ******************************************************************************
  * Copyright (c) 1996-2005 Regents of the University of Minnesota.
@@ -34,7 +36,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapraster.c 6481 2007-08-06 18:16:54Z warmerdam $")
+MS_CVSID("$Id: mapraster.c 9055 2009-05-29 03:29:47Z sdlime $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -330,6 +332,40 @@ int msAddColorAGG(mapObj *map, gdImagePtr img, int cmt, int r, int g, int b)
 }
 #endif
 
+/************************************************************************/
+/*                        resetExtension()                              */
+/*                                                                      */
+/*      Adapted from GDAL's CPLResetExtension().                        */
+/************************************************************************/
+static char *resetExtension( const char *pszFilename, const char *pszNewExt )
+{
+  char *pszResult = (char*) malloc(strlen(pszFilename) + 1 + strlen(pszNewExt) + 1);
+  size_t  i;
+
+  strcpy(pszResult, pszFilename);
+
+  /* -------------------------------------------------------------------- */
+  /*      First, try and strip off any existing extension.                */
+  /* -------------------------------------------------------------------- */
+  for( i = strlen(pszResult) - 1; i > 0; i-- ) {
+    if( pszResult[i] == '.' ) {
+      pszResult[i] = '\0';
+      break;
+    }
+
+    if( pszResult[i] == '/' || pszResult[i] == '\\' || pszResult[i] == ':' )
+      break;
+  }
+
+  /* -------------------------------------------------------------------- */
+  /*      Append the new extension.                                       */
+  /* -------------------------------------------------------------------- */
+  strcat(pszResult, ".");
+  strcat(pszResult, pszNewExt);
+
+  return pszResult;
+}
+
 /************************************************************************/
 /*                           readWorldFile()                            */
 /*                                                                      */
@@ -337,28 +373,29 @@ int msAddColorAGG(mapObj *map, gdImagePtr img, int cmt, int r, int g, int b)
 /*      from an ESRI world file.                                        */
 /************************************************************************/
 
-static int readWorldFile(char *filename, double *ulx, double *uly, double *cx, double *cy) {
+static int readWorldFile(const char *filename, double *ulx, double *uly, double *cx, double *cy) {
   FILE *stream;
   char *wld_filename;
   int i=0;
   char buffer[BUFLEN];
 
-  wld_filename = strdup(filename);
-
-  strcpy(strrchr(wld_filename, '.'), ".wld");
+  wld_filename = resetExtension(filename, "wld");
   stream = fopen(wld_filename, "r");
+  free(wld_filename);
   if(!stream) {
-    strcpy(strrchr(wld_filename, '.'), ".tfw");
+    wld_filename = resetExtension(filename, "tfw");
     stream = fopen(wld_filename, "r");
+    free(wld_filename);
     if(!stream) {
-      strcpy(strrchr(wld_filename, '.'), ".jgw");
+      wld_filename = resetExtension(filename, "jgw");
       stream = fopen(wld_filename, "r");
+      free(wld_filename);
       if(!stream) {
-        strcpy(strrchr(wld_filename, '.'), ".gfw");
+        wld_filename = resetExtension(filename, "gfw");
         stream = fopen(wld_filename, "r");
+        free(wld_filename);
         if(!stream) {	
 	  msSetError(MS_IOERR, "Unable to open world file for reading.", "readWorldFile()");
-	  free(wld_filename);
 	  return(-1);
         }
       }
@@ -387,7 +424,6 @@ static int readWorldFile(char *filename, double *ulx, double *uly, double *cx, d
   }
     
   fclose(stream);
-  free(wld_filename);
 
   return(0);
 }
@@ -1203,14 +1239,14 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
   shapeObj tshp;
 
   int force_gdal;
-  char szPath[MS_MAXPATHLEN], cwd[MS_MAXPATHLEN];
+  char szPath[MS_MAXPATHLEN];
   int final_status = MS_SUCCESS;
 
   rectObj searchrect;
   gdImagePtr img;
   char *pszTmp = NULL;
-
-  cwd[0] = '\0';
+  char tiAbsFilePath[MS_MAXPATHLEN];
+  char *tiAbsDirPath = NULL;
 
   if(layer->debug > 0 || map->debug > 1)
     msDebug( "msDrawRasterLayerLow(%s): entering.\n", layer->name );
@@ -1236,6 +1272,19 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
     }
   }
 
+  if(layer->maxscaledenom <= 0 && layer->minscaledenom <= 0) {
+    if((layer->maxgeowidth > 0) && ((map->extent.maxx - map->extent.minx) > layer->maxgeowidth)) {
+      if(layer->debug == MS_TRUE) msDebug( "msDrawRasterLayerLow(%s): skipping, map width %.2g > MAXSCALEDENOM=%g\n", layer->name, 
+          (map->extent.maxx - map->extent.minx), layer->maxgeowidth );
+      return(0);
+    }
+    if((layer->mingeowidth > 0) && ((map->extent.maxx - map->extent.minx) < layer->mingeowidth)) {
+      if(layer->debug == MS_TRUE) msDebug( "msDrawRasterLayerLow(%s): skipping, map width %.2g < MINSCALEDENOM=%g\n", layer->name, 
+          (map->extent.maxx - map->extent.minx), layer->mingeowidth );
+      return(0);
+    }
+  }
+
   force_gdal = MS_FALSE;
   if(MS_RENDERER_GD(image->format))
     img = image->img.gd;
@@ -1383,10 +1432,12 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
 
       if(status == MS_DONE) break; /* no more tiles/images */
        
-      if(layer->data == NULL) /* assume whole filename is in attribute field */
-          strcpy( tilename, tshp.values[tileitemindex] );
-      else
-          sprintf(tilename, "%s/%s", tshp.values[tileitemindex], layer->data);
+      if(layer->data == NULL || strlen(layer->data) == 0 ) { /* assume whole filename is in attribute field */
+	(void) strncpy(tilename, tshp.values[tileitemindex], sizeof(tilename)-1);
+	tilename[sizeof(tilename)-1] = '\0';
+      } else {
+	snprintf(tilename, sizeof(tilename), "%s/%s", tshp.values[tileitemindex], layer->data);
+      }
       filename = tilename;
       
       msFreeShape(&tshp); /* done with the shape */
@@ -1409,7 +1460,17 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
 #endif 
 */
 
-    msBuildPath3(szPath, map->mappath, map->shapepath, filename);
+    /*
+    ** If using a tileindex then build the path relative to that file if SHAPEPATH is not set.
+    */
+    if(layer->tileindex && !map->shapepath) { 
+      msBuildPath(tiAbsFilePath, map->mappath, layer->tileindex); /* absolute path to tileindex file */
+      tiAbsDirPath = msGetPath(tiAbsFilePath); /* tileindex file's directory */
+      msBuildPath(szPath, tiAbsDirPath, filename); 
+      free(tiAbsDirPath);
+    } else {
+      msBuildPath3(szPath, map->mappath, map->shapepath, filename);
+    }
 
     /*
     ** Try to open the file, and read the first 8 bytes as a signature. 
@@ -1563,20 +1624,27 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
     ** Generate an error.
     */
     if( !f ) {
-      msSetError(MS_IOERR, "(%s)", "msDrawRaster()", filename);
+      int ignore_missing = msMapIgnoreMissingData( map );
+      if( ignore_missing != MS_MISSING_DATA_IGNORE ) {
+        msSetError(MS_IOERR, "%s using full path %s", "msDrawRaster()", filename, szPath);
+      }
 
-#ifndef IGNORE_MISSING_DATA
-      if( layer->debug || map->debug )
+      if( ignore_missing == MS_MISSING_DATA_FAIL ) {
+        if( layer->debug || map->debug )  
           msDebug( "Unable to open file %s for layer %s ... fatal error.\n", filename, layer->name );
+        final_status = MS_FAILURE;
+        break;
+      }
 
-      final_status = MS_FAILURE;
-      break;
-#else
-      if( layer->debug || map->debug )
+      if( ignore_missing == MS_MISSING_DATA_LOG ) {
+        if( layer->debug || map->debug )
           msDebug( "Unable to open file %s for layer %s ... ignoring this missing data.\n", filename, layer->name );
+        continue;
+      }
 
-      continue; /* skip it, next tile */
-#endif
+      if( ignore_missing == MS_MISSING_DATA_IGNORE ) {
+        continue; /* skip it, next tile */
+      }
     }
 
     /* put others which may require checks here */  
