@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: maprasterquery.c 8901 2009-04-12 16:49:02Z tamas $
  *
  * Project:  MapServer
  * Purpose:  Implementation of query operations on rasters. 
@@ -31,7 +32,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: maprasterquery.c 6932 2007-10-08 16:13:37Z warmerdam $")
+MS_CVSID("$Id: maprasterquery.c 8901 2009-04-12 16:49:02Z tamas $")
 
 int msRASTERLayerGetShape(layerObj *layer, shapeObj *shape, int tile, 
                           long record);
@@ -189,7 +190,8 @@ static void msRasterLayerInfoInitialize( layerObj *layer )
     /* We need to do this or the layer->layerinfo will be interpreted */
     /* as shapefile access info because the default connectiontype is */
     /* MS_SHAPEFILE. */
-    layer->connectiontype = MS_RASTER;
+    if (layer->connectiontype != MS_WMS)
+        layer->connectiontype = MS_RASTER;
 
     rlinfo->query_result_hard_max = 1000000;
 
@@ -666,6 +668,8 @@ int msRasterQueryByRect(mapObj *map, layerObj *layer, rectObj queryRect)
     char szPath[MS_MAXPATHLEN];
     rectObj searchrect;
     rasterLayerInfo *rlinfo = NULL;
+    char tiAbsFilePath[MS_MAXPATHLEN];
+    char *tiAbsDirPath = NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Get the layer info.                                             */
@@ -716,10 +720,10 @@ int msRasterQueryByRect(mapObj *map, layerObj *layer, rectObj queryRect)
 /*      Open tile index if we have one.                                 */
 /* -------------------------------------------------------------------- */
     if(layer->tileindex) { /* we have in index file */
-        if(msSHPOpenFile(&tilefile, "rb", 
+        if(msShapefileOpen(&tilefile, "rb", 
                          msBuildPath3(szPath, map->mappath, map->shapepath, 
-                                      layer->tileindex)) == -1) 
-            if(msSHPOpenFile(&tilefile, "rb", msBuildPath(szPath, map->mappath, layer->tileindex)) == -1) 
+                                      layer->tileindex), MS_TRUE) == -1) 
+            if(msShapefileOpen(&tilefile, "rb", msBuildPath(szPath, map->mappath, layer->tileindex), MS_TRUE) == -1) 
                 return(MS_FAILURE);    
 
         tileitemindex = msDBFGetItemIndex(tilefile.hDBF, layer->tileitem);
@@ -732,7 +736,7 @@ int msRasterQueryByRect(mapObj *map, layerObj *layer, rectObj queryRect)
         if((map->projection.numargs > 0) && (layer->projection.numargs > 0))
             msProjectRect(&map->projection, &layer->projection, &searchrect); /* project the searchrect to source coords */
 #endif
-        status = msSHPWhichShapes(&tilefile, searchrect, layer->debug);
+        status = msShapefileWhichShapes(&tilefile, searchrect, layer->debug);
         if(status != MS_SUCCESS) 
             numtiles = 0; /* could be MS_DONE or MS_FAILURE */
         else
@@ -770,29 +774,39 @@ int msRasterQueryByRect(mapObj *map, layerObj *layer, rectObj queryRect)
 /* -------------------------------------------------------------------- */
         msGDALInitialize();
 
+        /*
+        ** If using a tileindex then build the path relative to that file if SHAPEPATH is not set.
+        */
+        if(layer->tileindex && !map->shapepath) { 
+            msBuildPath(tiAbsFilePath, map->mappath, layer->tileindex); /* absolute path to tileindex file */
+            tiAbsDirPath = msGetPath(tiAbsFilePath); /* tileindex file's directory */
+            msBuildPath(szPath, tiAbsDirPath, filename); 
+            free(tiAbsDirPath);
+        } else {
+            msBuildPath3(szPath, map->mappath, map->shapepath, filename);
+        }
+
         msAcquireLock( TLOCK_GDAL );
-        hDS = GDALOpen(
-            msBuildPath3( szPath, map->mappath, map->shapepath, filename ), 
-            GA_ReadOnly );
+        hDS = GDALOpen(szPath, GA_ReadOnly );
         
         if( hDS == NULL )
         {
+            int ignore_missing = msMapIgnoreMissingData( map );
             msReleaseLock( TLOCK_GDAL );
 
-#ifndef IGNORE_MISSING_DATA
-            if( layer->debug || map->debug )
+            if ( ignore_missing == MS_MISSING_DATA_FAIL ) {
+              if( layer->debug || map->debug )
                 msSetError( MS_IMGERR, 
                             "Unable to open file %s for layer %s ... fatal error.\n%s", 
                             szPath, layer->name, CPLGetLastErrorMsg(),
                             "msRasterQueryByRect()" );
-            
-            return(MS_FAILURE);
-#else
-            if( layer->debug || map->debug )
+              return(MS_FAILURE);
+            }
+            if( ignore_missing == MS_MISSING_DATA_LOG ) {
+              if( layer->debug || map->debug )
                 msDebug( "Unable to open file %s for layer %s ... ignoring this missing data.\n%s", 
                          filename, layer->name, CPLGetLastErrorMsg() );
-            
-#endif
+            }
             continue;
         }
 
@@ -842,7 +856,14 @@ int msRasterQueryByRect(mapObj *map, layerObj *layer, rectObj queryRect)
     } /* next tile */
 
     if(layer->tileindex) /* tiling clean-up */
-        msSHPCloseFile(&tilefile);    
+        msShapefileClose(&tilefile);    
+
+/* -------------------------------------------------------------------- */
+/*      On failure, or empty result set, cleanup the rlinfo since we    */
+/*      likely won't ever have it accessed or cleaned up later.         */
+/* -------------------------------------------------------------------- */
+    if( status == MS_FAILURE || rlinfo->query_results == 0 )
+        msRasterLayerInfoFree( layer );
 
     return status;
 #endif /* def USE_GDAL */
@@ -872,7 +893,9 @@ int msRasterQueryByShape(mapObj *map, layerObj *layer, shapeObj *selectshape)
     msComputeBounds( selectshape );
     status = msRasterQueryByRect( map, layer, selectshape->bounds );
 
-    rlinfo->searchshape = NULL;
+    rlinfo = (rasterLayerInfo *) layer->layerinfo;
+    if( rlinfo )
+        rlinfo->searchshape = NULL;
 
     return status;
 #endif /* USE_GDAL  */
@@ -997,21 +1020,17 @@ int msRASTERLayerOpen(layerObj *layer)
                 "msRasterQueryByRect()" );
     return MS_FAILURE;
 #else
-    rasterLayerInfo *rlinfo = (rasterLayerInfo *) layer->layerinfo;
+    rasterLayerInfo *rlinfo;
 
-    if( rlinfo != NULL )
-        rlinfo->refcount = rlinfo->refcount + 1;
+    /* If we don't have info, initialize an empty one now */
+    if( layer->layerinfo == NULL )
+        msRasterLayerInfoInitialize( layer );
+        
+    rlinfo = (rasterLayerInfo *) layer->layerinfo;
 
-    if( rlinfo == NULL )
-    {
-        msSetError( MS_MISCERR, 
-                    "Attempt to open a RASTER layer, but this is"
-                    " only supported after a raster query.",
-                    "msRASTERLayerOpen()" );
-        return MS_FAILURE;
-    }
-    else 
-        return MS_SUCCESS;
+    rlinfo->refcount = rlinfo->refcount + 1;
+
+    return MS_SUCCESS;
 #endif /* def USE_GDAL */
 }
 
@@ -1162,6 +1181,7 @@ int msRASTERLayerGetShape(layerObj *layer, shapeObj *shape, int tile,
 #endif
 
         msAddLine( shape, &line );
+        msComputeBounds( shape );
     }
 
 /* -------------------------------------------------------------------- */
