@@ -1,6 +1,7 @@
 /******************************************************************************
+ * $Id: mapresample.c 7765 2008-06-30 18:01:38Z tbonfort $
  *
- * Project:  CFS OGC MapServer
+ * Project:  MapServer
  * Purpose:  Assorted code related to resampling rasters.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
@@ -28,8 +29,9 @@
 
 #include <assert.h>
 #include "mapresample.h"
+#include "mapthread.h"
 
-MS_CVSID("$Id: mapresample.c 7131 2007-11-30 22:13:28Z warmerdam $")
+MS_CVSID("$Id: mapresample.c 7765 2008-06-30 18:01:38Z tbonfort $")
 
 #ifndef MAX
 #  define MIN(a,b)      ((a<b) ? a : b)
@@ -170,41 +172,6 @@ msNearestRasterResampler( imageObj *psSrcImage, colorObj offsite,
                     }
                 }
             }
-#ifdef USE_AGG
-            else if( MS_RENDERER_AGG(psSrcImage->format) )
-            {
-                if( !gdImageTrueColor(psSrcImage->img.gd) )
-                {
-                    nValue = panCMap[srcImg->pixels[nSrcY][nSrcX]];
-
-                    if( nValue == -1 )
-                        continue;
-
-                    nSetPoints++;
-                    dstImg->pixels[nDstY][nDstX] = nValue; 
-                }
-                else
-                {
-                    int nValue = srcImg->tpixels[nSrcY][nSrcX];
-                    int gd_alpha = gdTrueColorGetAlpha(nValue);
-
-                    if( gd_alpha == 0 )
-                    {
-                        nSetPoints++;
-                        dstImg->tpixels[nDstY][nDstX] = nValue;
-                    }
-                    else if( gd_alpha == 127 )
-                        /* overlay is transparent, do nothing */;
-                    else
-                    {
-                        nSetPoints++;
-                        dstImg->tpixels[nDstY][nDstX] = 
-                            gdAlphaBlend( dstImg->tpixels[nDstY][nDstX], 
-                                          nValue );
-                    }
-                }
-            }
-#endif
             else if( MS_RENDERER_RAWDATA(psSrcImage->format) )
             {
                 int band;
@@ -324,31 +291,6 @@ static void msSourceSample( imageObj *psSrcImage, int iSrcX, int iSrcY,
             }
         }
     }
-#ifdef USE_AGG
-    else if( MS_RENDERER_AGG(psSrcImage->format) )
-    {
-        if( !gdImageTrueColor(psSrcImage->img.gd) )
-        {
-            padfPixelSum[0] += 
-                (dfWeight * psSrcImage->img.gd->pixels[iSrcY][iSrcX]);
-            *pdfWeightSum += dfWeight;
-        }
-        else
-        {
-            int nValue = psSrcImage->img.gd->tpixels[iSrcY][iSrcX];
-            int gd_alpha = gdTrueColorGetAlpha(nValue);
-
-            if( gd_alpha != 127 )
-            {
-                padfPixelSum[0] += dfWeight * gdTrueColorGetRed(nValue);
-                padfPixelSum[1] += dfWeight * gdTrueColorGetGreen(nValue);
-                padfPixelSum[2] += dfWeight * gdTrueColorGetBlue(nValue);
-                
-                *pdfWeightSum += dfWeight;
-            }
-        }
-    }
-#endif
     else
     {
         int band;
@@ -547,45 +489,6 @@ msBilinearRasterResampler( imageObj *psSrcImage, colorObj offsite,
                     }
                 }
             }
-#ifdef USE_AGG
-            else if( MS_RENDERER_AGG(psSrcImage->format) )
-            {
-                if( !gdImageTrueColor(psSrcImage->img.gd) )
-                {
-                    int nResult = panCMap[(int) padfPixelSum[0]];
-                    if( nResult != -1 )
-                    {                        
-                        nSetPoints++;
-                        dstImg->pixels[nDstY][nDstX] = nResult;
-                    }
-                }
-                else
-                {
-                    nSetPoints++;
-                    if( dfWeightSum > 0.99 )
-                        dstImg->tpixels[nDstY][nDstX] = 
-                            gdTrueColor( (int) padfPixelSum[0], 
-                                         (int) padfPixelSum[1], 
-                                         (int) padfPixelSum[2] );
-                    else
-                    {
-                        int gd_color;
-                        int gd_alpha = 127 - 127.9 * dfWeightSum;
-
-                        gd_alpha = MAX(0,MIN(127,gd_alpha));
-                        gd_color = gdTrueColorAlpha(
-                            (int) padfPixelSum[0], 
-                            (int) padfPixelSum[1], 
-                            (int) padfPixelSum[2], 
-                            gd_alpha );
-                        
-                        dstImg->tpixels[nDstY][nDstX] = 
-                            msAlphaBlend( dstImg->tpixels[nDstY][nDstX],
-                                          gd_color );
-                    }
-                }
-            }
-#endif
             else if( MS_RENDERER_RAWDATA(psSrcImage->format) )
             {
                 int band;
@@ -812,46 +715,6 @@ msAverageRasterResampler( imageObj *psSrcImage, colorObj offsite,
                     }
                 }
             }
-#ifdef USE_AGG
-            else if( MS_RENDERER_AGG(psSrcImage->format) )
-            {
-                if( !gdImageTrueColor(psSrcImage->img.gd) )
-                {
-                    int nResult = panCMap[(int) padfPixelSum[0]];
-                    if( nResult != -1 )
-                    {                        
-                        nSetPoints++;
-                        dstImg->pixels[nDstY][nDstX] = nResult;
-                    }
-                }
-                else
-                {
-                    nSetPoints++;
-                    if( dfAlpha01 > 0.99 )
-                        dstImg->tpixels[nDstY][nDstX] = 
-                            gdTrueColor( (int) padfPixelSum[0], 
-                                         (int) padfPixelSum[1], 
-                                         (int) padfPixelSum[2] );
-                    else
-                    {
-                        int gd_color;
-                        int gd_alpha = (int) (127 - 127.9 * dfAlpha01);
-
-                        gd_alpha = MAX(0,MIN(127,gd_alpha));
-                        gd_color = gdTrueColorAlpha(
-                            (int) padfPixelSum[0], 
-                            (int) padfPixelSum[1], 
-                            (int) padfPixelSum[2], 
-                            gd_alpha );
-                        
-                        dstImg->tpixels[nDstY][nDstX] = 
-                            msAlphaBlend( dstImg->tpixels[nDstY][nDstX],
-                                          gd_color );
-                        
-                    }
-                }
-            }
-#endif
             else if( MS_RENDERER_RAWDATA(psSrcImage->format) )
             {
                 int band;
@@ -1037,10 +900,16 @@ int msProjTransformer( void *pCBData, int nPoints,
     if( psPTInfo->bUseProj )
     {
         double *z;
+        int tr_result;
         
         z = (double *) calloc(sizeof(double),nPoints);
-        if( pj_transform( psPTInfo->psDstProj, psPTInfo->psSrcProj, 
-                          nPoints, 1, x, y,  z) != 0 )
+
+        msAcquireLock( TLOCK_PROJ );
+        tr_result = pj_transform( psPTInfo->psDstProj, psPTInfo->psSrcProj, 
+                                  nPoints, 1, x, y,  z);
+        msReleaseLock( TLOCK_PROJ );
+
+        if( tr_result != 0 )
         {
             free( z );
             for( i = 0; i < nPoints; i++ )
@@ -1321,6 +1190,8 @@ static int msTransformMapToSource( int nDstXSize, int nDstYSize,
 /* -------------------------------------------------------------------- */
     if( psDstProj->proj && psSrcProj->proj )
     {
+        int tr_result;
+
         if( pj_is_latlong(psDstProj->proj) )
         {
             for( i = 0; i < nSamples; i++ )
@@ -1330,11 +1201,13 @@ static int msTransformMapToSource( int nDstXSize, int nDstYSize,
             }
         }
         
-        if( pj_transform( psDstProj->proj, psSrcProj->proj,
-                          nSamples, 1, x, y, z ) != 0 )
-        {
+        msAcquireLock( TLOCK_PROJ );
+        tr_result = pj_transform( psDstProj->proj, psSrcProj->proj,
+                                  nSamples, 1, x, y, z );
+        msReleaseLock( TLOCK_PROJ );
+
+        if( tr_result != 0 )
             return MS_FALSE;
-        }
         
         if( pj_is_latlong(psSrcProj->proj) )
         {
@@ -1634,6 +1507,21 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
 /*      for the temporary image to include transparentency support.     */
 /* -------------------------------------------------------------------- */
     sDummyMap.outputformat = msCloneOutputFormat( image->format );
+  #ifdef USE_AGG
+    if( MS_RENDERER_AGG(sDummyMap.outputformat))
+    {
+/*
+Fix for bug 2540 :
+
+The resampling methods used later assume that the pixel format of the
+dummy image is in gd format (alpha between 127 and 0). If we keep an AGG
+outpuformat, the image is initialised with its alpha=0, which is interpreted
+as fully opaque instead of fully transparent. The fix is to completely forget
+about AGG here, and create a GD imageObj
+*/
+        sDummyMap.outputformat->renderer=MS_RENDER_WITH_GD;
+    }
+#endif  
     sDummyMap.width = nLoadImgXSize;
     sDummyMap.height = nLoadImgYSize;
     
@@ -1652,16 +1540,6 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
         sDummyMap.imagecolor.green = 17;
         sDummyMap.imagecolor.blue = 191;
     }
-#ifdef USE_AGG
-    else if( MS_RENDERER_AGG(sDummyMap.outputformat) 
-        && !gdImageTrueColor( image->img.gd ) )
-    {
-        sDummyMap.outputformat->transparent = MS_TRUE;
-        sDummyMap.imagecolor.red = 117;
-        sDummyMap.imagecolor.green = 17;
-        sDummyMap.imagecolor.blue = 191;
-    }
-#endif
 /* -------------------------------------------------------------------- */
 /*      If we are working in RGB mode ensure we produce an RGBA         */
 /*      image so the transparency can be preserved.                     */
@@ -1679,21 +1557,6 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
         sDummyMap.imagecolor.green = map->imagecolor.green;
         sDummyMap.imagecolor.blue = map->imagecolor.blue;
     }
-#ifdef USE_AGG
-    else if( MS_RENDERER_AGG(sDummyMap.outputformat) 
-             && gdImageTrueColor( image->img.gd ) )
-    {
-        assert( sDummyMap.outputformat->imagemode == MS_IMAGEMODE_RGB
-                || sDummyMap.outputformat->imagemode == MS_IMAGEMODE_RGBA );
-
-        sDummyMap.outputformat->transparent = MS_TRUE;
-        sDummyMap.outputformat->imagemode = MS_IMAGEMODE_RGBA;
-
-        sDummyMap.imagecolor.red = map->imagecolor.red;
-        sDummyMap.imagecolor.green = map->imagecolor.green;
-        sDummyMap.imagecolor.blue = map->imagecolor.blue;
-    }
-#endif
 /* -------------------------------------------------------------------- */
 /*      Setup a dummy map object we can use to read from the source     */
 /*      raster, with the newly established extents, and resolution.     */
@@ -1749,27 +1612,6 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
         for( iColor = nColorCount; iColor < 256; iColor++ )
             anCMap[iColor] = -1;
     }
-#ifdef USE_AGG
-    else if( MS_RENDERER_AGG(srcImage->format)
-        && !gdImageTrueColor( srcImage->img.gd ) )
-    {
-        int  iColor, nColorCount;
-
-        anCMap[0] = -1; /* color zero is always transparent */
-
-        nColorCount = gdImageColorsTotal( srcImage->img.gd );
-        for( iColor = 1; iColor < nColorCount; iColor++ )
-        {
-            anCMap[iColor] = 
-                msAddColorAGG( map, image->img.gd, 0, 
-                              gdImageRed( srcImage->img.gd, iColor ),
-                              gdImageGreen( srcImage->img.gd, iColor ),
-                              gdImageBlue( srcImage->img.gd, iColor ) );
-        }
-        for( iColor = nColorCount; iColor < 256; iColor++ )
-            anCMap[iColor] = -1;
-    }
-#endif
 /* -------------------------------------------------------------------- */
 /*      Setup transformations between our source image, and the         */
 /*      target map image.                                               */
