@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapscale.c 8633 2009-03-05 06:32:31Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Scale object rendering. 
@@ -28,7 +29,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapscale.c 6567 2007-08-13 08:20:26Z tbonfort $")
+MS_CVSID("$Id: mapscale.c 8633 2009-03-05 06:32:31Z sdlime $")
 
 #define VMARGIN 3 /* buffer around the scalebar */
 #define HMARGIN 3
@@ -62,6 +63,10 @@ static double roundInterval(double d)
     return(MS_NINT(d/1000) * 1000);
   if(d<1000000)
     return(MS_NINT(d/10000) * 10000);
+  if(d<10000000)
+    return(MS_NINT(d/100000) * 100000);
+  if(d<100000000)
+    return(MS_NINT(d/1000000) * 1000000);
 
   return(-1);
 }
@@ -219,7 +224,7 @@ imageObj *msDrawScalebar(mapObj *map)
 #endif
   }
   else
-    fontPtr = msGetBitmapFont(map->scalebar.label.size);
+    fontPtr = msGetBitmapFont(MS_NINT(map->scalebar.label.size));
 
   if(!fontPtr) return(NULL);
 
@@ -275,7 +280,16 @@ imageObj *msDrawScalebar(mapObj *map)
 
   if (map->outputformat->imagemode == MS_IMAGEMODE_RGB || map->outputformat->imagemode == MS_IMAGEMODE_RGBA) gdImageAlphaBlending(image->img.gd, 1);
 
-  ox = MS_NINT((map->scalebar.width - sx)/2.0 + fontPtr->w/2.0); /* center the computed scalebar */
+  switch(map->scalebar.align) {
+  case(MS_ALIGN_LEFT):
+    ox = HMARGIN;
+    break;
+  case(MS_ALIGN_RIGHT):
+    ox = MS_NINT((map->scalebar.width - sx) + fontPtr->w);
+    break;
+  default:
+    ox = MS_NINT((map->scalebar.width - sx)/2.0 + fontPtr->w/2.0); /* center the computed scalebar */
+  }
   oy = VMARGIN;
 
   /* turn RGB colors into indexed values */
@@ -300,8 +314,8 @@ imageObj *msDrawScalebar(mapObj *map)
       map->scalebar.label.position = MS_CC;
       p.x = ox + j*isx; /* + MS_NINT(fontPtr->w/2); */
       p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontPtr->h);
-      if(msGetLabelSize(label,&(map->scalebar.label), &r, 
-              &(map->fontset), 1, MS_FALSE) == -1) return(NULL);
+      if(msGetLabelSize(NULL,label,&(map->scalebar.label), &r, 
+              &(map->fontset), 1, MS_FALSE,NULL) == -1) return(NULL);
       p2 = get_metrics(&p, MS_CC, r, 0,0, 0, 0, NULL);
       msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
@@ -313,8 +327,8 @@ imageObj *msDrawScalebar(mapObj *map)
     map->scalebar.label.position = MS_CR;
     p.x = ox; /* + MS_NINT(fontPtr->w/2); */
     p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontPtr->h);
-    if(msGetLabelSize(label,&(map->scalebar.label), &r, 
-            &(map->fontset), 1, MS_FALSE) == -1) return(NULL);
+    if(msGetLabelSize(NULL,label,&(map->scalebar.label), &r, 
+            &(map->fontset), 1, MS_FALSE,NULL) == -1) return(NULL);
     p2 = get_metrics(&p, MS_CR, r, 0,0, 0, 0, NULL);
     msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
@@ -333,8 +347,8 @@ imageObj *msDrawScalebar(mapObj *map)
       map->scalebar.label.position = MS_CC;
       p.x = ox + j*isx; /* + MS_NINT(fontPtr->w/2); */
       p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontPtr->h);
-      if(msGetLabelSize(label,&(map->scalebar.label), &r, 
-              &(map->fontset), 1, MS_FALSE) == -1) return(NULL);
+      if(msGetLabelSize(NULL,label,&(map->scalebar.label), &r, 
+              &(map->fontset), 1, MS_FALSE,NULL) == -1) return(NULL);
       p2 = get_metrics(&p, MS_CC, r, 0,0, 0, 0, NULL);
       msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
@@ -349,8 +363,8 @@ imageObj *msDrawScalebar(mapObj *map)
     map->scalebar.label.position = MS_CR;
     p.x = ox; /* + MS_NINT(fontPtr->w/2); */
     p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontPtr->h);
-    if(msGetLabelSize(label,&(map->scalebar.label), &r, 
-            &(map->fontset), 1, MS_FALSE) == -1) return(NULL);
+    if(msGetLabelSize(NULL,label,&(map->scalebar.label), &r, 
+            &(map->fontset), 1, MS_FALSE,NULL) == -1) return(NULL);
     p2 = get_metrics(&p, MS_CR, r, 0,0, 0, 0, NULL);
     msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
@@ -370,23 +384,21 @@ imageObj *msDrawScalebar(mapObj *map)
 
 }
 
-int msEmbedScalebar(mapObj *map, gdImagePtr img)
+int msEmbedScalebar(mapObj *map, imageObj *img)
 {
   int s,l;
   pointObj point;
   imageObj *image = NULL;
 
   s = msGetSymbolIndex(&(map->symbolset), "scalebar", MS_FALSE);
-  if(s == -1) {
-    if (msGrowSymbolSet(&map->symbolset) == NULL)
-        return -1;
-    s = map->symbolset.numsymbols;
-    map->symbolset.numsymbols++;
-    initSymbol(map->symbolset.symbol[s]);
-  } else {
-    if(map->symbolset.symbol[s]->img) 
-      gdImageDestroy(map->symbolset.symbol[s]->img);
-  }
+  if(s != -1) 
+    msRemoveSymbol(&(map->symbolset), s); /* solves some caching issues in AGG with long-running processes */
+
+  if(msGrowSymbolSet(&map->symbolset) == NULL)
+    return -1;
+  s = map->symbolset.numsymbols;
+  map->symbolset.numsymbols++;
+  initSymbol(map->symbolset.symbol[s]);
   
   image = msDrawScalebar(map);
   map->symbolset.symbol[s]->img =  image->img.gd; /* TODO  */
@@ -399,8 +411,7 @@ int msEmbedScalebar(mapObj *map, gdImagePtr img)
 
   /* in 8 bit, mark 0 as being transparent.  In 24bit hopefully already
      have transparency enabled ... */
-  if(map->scalebar.transparent == MS_ON 
-     && !gdImageTrueColor(image->img.gd ) )
+  if(map->scalebar.transparent == MS_ON && !gdImageTrueColor(image->img.gd ) )
     gdImageColorTransparent(map->symbolset.symbol[s]->img, 0);
 
   switch(map->scalebar.position) {
@@ -450,12 +461,10 @@ int msEmbedScalebar(mapObj *map, gdImagePtr img)
     map->layerorder[l] = l;
   }
 
-  /* to resolve bug 490 */
-  GET_LAYER(map, l)->opacity = MS_GD_ALPHA;
-  
+  GET_LAYER(map, l)->opacity = MS_GD_ALPHA; /* to resolve bug 490 */
   GET_LAYER(map, l)->status = MS_ON;
 
-/* TODO: Change this when we get rid of MS_MAXSTYLES */
+  /* TODO: Change this when we get rid of MS_MAXSTYLES */
   if (msMaybeAllocateStyle(GET_LAYER(map, l)->class[0], 0)==MS_FAILURE) return MS_FAILURE;
   GET_LAYER(map, l)->class[0]->styles[0]->symbol = s;
   GET_LAYER(map, l)->class[0]->styles[0]->color.pen = -1;
@@ -465,13 +474,21 @@ int msEmbedScalebar(mapObj *map, gdImagePtr img)
 
   if(map->scalebar.postlabelcache) /* add it directly to the image //TODO */
   {
-    msDrawMarkerSymbolGD(&map->symbolset, img, &point, GET_LAYER(map, l)->class[0]->styles[0], 1.0);
+#ifdef USE_AGG
+      /* the next function will call the AGG renderer so we must make
+       * sure the alpha channel of the image is in a coherent state.
+       * the alpha values aren't actually switched unless the last 
+       * layer of the map was a raster layer
+       */
+      if(MS_RENDERER_AGG(map->outputformat))
+        msAlphaGD2AGG(img);
+#endif
+      msDrawMarkerSymbol(&map->symbolset, img, &point, GET_LAYER(map, l)->class[0]->styles[0], 1.0);
+  } else {
+    msAddLabel(map, l, 0, NULL, &point, NULL, " ", 1.0, NULL);
   }
-  else
-    msAddLabel(map, l, 0, -1, -1, &point, NULL, " ", 1.0, NULL);
 
-  /* Mark layer as deleted so that it doesn't interfere with html legends or */
-  /* with saving maps */
+  /* Mark layer as deleted so that it doesn't interfere with html legends or with saving maps */
   GET_LAYER(map, l)->status = MS_DELETE;
 
   /* Free image (but keep the GD image which is in the symbol cache now) */
