@@ -1,13 +1,14 @@
-/*
- * mapscript_i.c: interface file for MapServer PHP scripting extension 
- *                called MapScript.
+/******************************************************************************
+ * $Id: mapscript_i.c 9062 2009-06-02 12:38:19Z dmorissette $
  *
- * This file was originally based on the SWIG interface file mapscript.i
- *
- * $Id: mapscript_i.c 6601 2007-08-16 15:34:18Z dmorissette $
+ * Project:  MapServer
+ * Purpose:  Interface file for MapServer PHP scripting extension 
+ *           called MapScript. This file was originally based on 
+ *           the SWIG interface file mapscript.i
+ * Author:   Daniel Morissette, DM Solutions Group (dmorissette@dmsolutions.ca)
  *
  **********************************************************************
- * Copyright (c) 2000, 2007, Daniel Morissette, DM Solutions Group
+ * Copyright (c) 2000, 2007, Daniel Morissette, DM Solutions Group Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -38,6 +39,7 @@
 #include "../../mapsymbol.h"
 #include "../../mapshape.h"
 #include "../../mapproject.h"
+#include "../../maphash.h"
 
 /**********************************************************************
  * class extensions for mapObj
@@ -50,6 +52,15 @@ mapObj *mapObj_new(char *filename, char *new_path) {
     } 
 }
 
+mapObj *mapObj_newFromString(char *map_text, char *new_path) {
+
+      if(map_text && strlen(map_text))
+        return msLoadMapFromString(map_text, new_path);
+      else { /* create an empty map, no layers etc... */
+        return msNewMapObj();
+      }
+}
+
 void  mapObj_destroy(mapObj* self) {
     msFreeMap(self);
   }
@@ -131,12 +142,12 @@ imageObj *mapObj_drawReferenceMap(mapObj* self) {
 
 //TODO
 int mapObj_embedScalebar(mapObj* self, imageObj *img) {	
-    return msEmbedScalebar(self, img->img.gd);
+    return msEmbedScalebar(self, img);
   }
 
 //TODO
 int mapObj_embedLegend(mapObj* self, imageObj *img) {	
-    return msEmbedLegend(self, img->img.gd);
+    return msEmbedLegend(self, img);
   }
 
 int mapObj_drawLabelCache(mapObj* self, imageObj *img) {
@@ -145,7 +156,7 @@ int mapObj_drawLabelCache(mapObj* self, imageObj *img) {
 
 int mapObj_queryByPoint(mapObj* self, pointObj *point, 
                          int mode, double buffer) {
-    return msQueryByPoint(self, -1, mode, *point, buffer);
+    return msQueryByPoint(self, -1, mode, *point, buffer, 0);
   }
 
 int mapObj_queryByRect(mapObj* self, rectObj rect) {
@@ -319,7 +330,7 @@ int mapObj_applySLDURL(mapObj *self, char *sld)
 
 char *mapObj_generateSLD(mapObj *self)
 {
-    return msSLDGenerateSLD(self, -1);
+  return msSLDGenerateSLD(self, -1, NULL);
 }
 
 
@@ -331,7 +342,20 @@ int mapObj_loadOWSParameters(mapObj *self, cgiRequestObj *request,
 
 int mapObj_OWSDispatch(mapObj *self, cgiRequestObj *req )
 {
-    return msOWSDispatch( self, req);
+    return msOWSDispatch( self, req, MS_TRUE);
+}
+
+int mapObj_insertLayer(mapObj *self, layerObj *layer, int index)
+{
+    if (self && layer)
+      return msInsertLayer(self, layer, index);
+
+    return -1;
+}
+
+layerObj *mapObj_removeLayer(mapObj *self, int layerindex)
+{
+    return msRemoveLayer(self, layerindex);
 }
 
 /**********************************************************************
@@ -354,8 +378,22 @@ layerObj *layerObj_new(mapObj *map) {
   }
 
 void layerObj_destroy(layerObj *self) {
-    return; // map deconstructor takes care of it
-  }
+   /* if the layer has a parent_map, let's the map object destroy it */
+   if ((self->map == NULL) && (self->refcount == 1)) {
+      /* if there is no other PHP Object that use this C layer object, delete it */
+         freeLayer(self);
+         free(self);
+         self = NULL;
+   } else 
+   {
+      MS_REFCNT_DECR(self);  
+   }
+   return;
+}
+
+int layerObj_updateFromString(layerObj *self, char *snippet) {
+   return msUpdateLayerFromString(self, snippet, MS_FALSE);
+}
 
 int layerObj_open(layerObj *self) {
     return msLayerOpen(self);
@@ -363,8 +401,16 @@ int layerObj_open(layerObj *self) {
 
 
 int layerObj_whichShapes(layerObj *self, rectObj *poRect) {
-  msLayerGetItems(self);
-  return msLayerWhichShapes(self, *poRect);
+    int oldconnectiontype = self->connectiontype;
+    self->connectiontype = MS_INLINE;
+
+    if(msLayerWhichItems(self, MS_TRUE, MS_FALSE, NULL) != MS_SUCCESS) {
+        self->connectiontype = oldconnectiontype;
+        return MS_FAILURE;
+    }
+    self->connectiontype = oldconnectiontype;
+
+    return msLayerWhichShapes(self, *poRect);
 }
 
 
@@ -437,7 +483,7 @@ int layerObj_queryByPoint(layerObj *self, mapObj *map,
         
     status = self->status;
     self->status = MS_ON;
-    retval = msQueryByPoint(map, self->index, mode, *point, buffer);
+    retval = msQueryByPoint(map, self->index, mode, *point, buffer, 0);
     self->status = status;
 
     return retval;
@@ -510,6 +556,8 @@ int layerObj_setProjection(layerObj *self, char *string) {
   }
 
 int layerObj_addFeature(layerObj *self, shapeObj *shape) {
+    if (self->features == NULL) shape->index = 0;
+    else shape->index = self->features->shape.index + 1;
     if(insertFeatureList(&(self->features), shape) == NULL) 
       return -1;
     else
@@ -554,7 +602,7 @@ int layerObj_applySLDURL(layerObj *self, char *sld, char *stylelayer)
 
 char *layerObj_generateSLD(layerObj *self)
 {
-    return msSLDGenerateSLD(self->map, self->index);
+  return msSLDGenerateSLD(self->map, self->index, NULL);
 }
 
 int layerObj_moveClassUp(layerObj *self, int index)
@@ -572,6 +620,63 @@ classObj *layerObj_removeClass(layerObj *self, int index)
     return msRemoveClass(self, index);
 }
 
+int layerObj_setConnectionType(layerObj *self, int connectiontype, 
+                               const char *library_str) 
+{
+    /* Caller is responsible to close previous layer correctly before calling
+     * msConnectLayer() 
+     */
+    if (msLayerIsOpen(self))
+        msLayerClose(self);
+
+    return msConnectLayer(self, connectiontype, library_str);
+}
+
+/**********************************************************************
+ * class extensions for labelObj
+ **********************************************************************/
+int labelObj_updateFromString(labelObj *self, char *snippet) {
+   return msUpdateLabelFromString(self, snippet);
+}
+
+/**********************************************************************
+ * class extensions for legendObj
+ **********************************************************************/
+int legendObj_updateFromString(legendObj *self, char *snippet) {
+   return msUpdateLegendFromString(self, snippet, MS_FALSE);
+}
+
+/**********************************************************************
+ * class extensions for queryMapObj
+ **********************************************************************/
+int queryMapObj_updateFromString(queryMapObj *self, char *snippet) {
+   return msUpdateQueryMapFromString(self, snippet, MS_FALSE);
+}
+
+/**********************************************************************
+ * class extensions for referenceMapObj
+ **********************************************************************/
+
+int referenceMapObj_updateFromString(referenceMapObj *self, char *snippet) {
+   return msUpdateReferenceMapFromString(self, snippet, MS_FALSE);
+}
+
+/**********************************************************************
+ * class extensions for scaleBarObj
+ **********************************************************************/
+
+int scalebarObj_updateFromString(scalebarObj *self, char *snippet) {
+   return msUpdateScalebarFromString(self, snippet, MS_FALSE);
+}
+
+/**********************************************************************
+ * class extensions for webObj
+ **********************************************************************/
+
+int webObj_updateFromString(webObj *self, char *snippet) {
+   return msUpdateWebFromString(self, snippet, MS_FALSE);
+}
+
 /**********************************************************************
  * class extensions for classObj, always within the context of a layer
  **********************************************************************/
@@ -594,6 +699,10 @@ classObj *classObj_new(layerObj *layer, classObj *class) {
     return (layer->class[layer->numclasses-1]);
   }
 
+int classObj_updateFromString(classObj *self, char *snippet) {
+   return msUpdateClassFromString(self, snippet, MS_FALSE);
+}
+
 void  classObj_destroy(classObj *self) {
     return; /* do nothing, map deconstrutor takes care of it all */
   }
@@ -618,7 +727,7 @@ char *classObj_getTextString(classObj *self) {
     return msGetExpressionString(&(self->text));
 }
 
-int classObj_drawLegendIcon(classObj *self, mapObj *map, layerObj *layer, int width, int height, gdImagePtr dstImg, int dstX, int dstY) {
+int classObj_drawLegendIcon(classObj *self, mapObj *map, layerObj *layer, int width, int height, imageObj *dstImg, int dstX, int dstY) {
   msClearLayerPenValues(layer); // just in case the mapfile has already been processed
     return msDrawLegendIcon(map, layer, self, width, height, dstImg, dstX, dstY);
 }
@@ -673,6 +782,21 @@ int classObj_deleteStyle(classObj *self, int index)
     return msDeleteStyle(self, index);
 }
 
+char *classObj_getMetaData(classObj *self, char *name) {
+    return(msLookupHashTable(&(self->metadata), name));
+  }
+
+
+int classObj_setMetaData(classObj *self, char *name, char *value) {
+    if (msInsertHashTable(&(self->metadata), name, value) == NULL)
+	return MS_FAILURE;
+    return MS_SUCCESS;
+  }
+
+int classObj_removeMetaData(classObj *self, char *name) {
+    return(msRemoveHashTable(&(self->metadata), name));
+  }
+
 /**********************************************************************
  * class extensions for pointObj, useful many places
  **********************************************************************/
@@ -797,7 +921,7 @@ int shapeObj_add(shapeObj *self, lineObj *line) {
 
 int shapeObj_draw(shapeObj *self, mapObj *map, layerObj *layer, 
                   imageObj *img) {
-    return msDrawShape(map, layer, self, img, -1);
+    return msDrawShape(map, layer, self, img, -1, MS_FALSE);
   }
 
 void shapeObj_setBounds(shapeObj *self) {
@@ -1011,7 +1135,7 @@ int rectObj_draw(rectObj *self, mapObj *map, layerObj *layer,
     shape.classindex = classindex;
     shape.text = strdup(text);
 
-    msDrawShape(map, layer, &shape, img, -1);
+    msDrawShape(map, layer, &shape, img, -1, MS_FALSE);
 
     msFreeShape(&shape);
     
@@ -1030,14 +1154,14 @@ shapefileObj *shapefileObj_new(char *filename, int type) {
       return NULL;
 
     if(type == -1)
-      status = msSHPOpenFile(shapefile, "rb", filename);
+      status = msShapefileOpen(shapefile, "rb", filename, MS_TRUE);
     else if (type == -2)
-      status = msSHPOpenFile(shapefile, "rb+", filename);
+      status = msShapefileOpen(shapefile, "rb+", filename, MS_TRUE);
     else
-      status = msSHPCreateFile(shapefile, filename, type);
+      status = msShapefileCreate(shapefile, filename, type);
 
     if(status == -1) {
-      msSHPCloseFile(shapefile);
+      msShapefileClose(shapefile);
       free(shapefile);
       return NULL;
     }
@@ -1046,7 +1170,7 @@ shapefileObj *shapefileObj_new(char *filename, int type) {
   }
 
 void shapefileObj_destroy(shapefileObj *self) {
-    msSHPCloseFile(self);
+    msShapefileClose(self);
     free(self);  
   }
 
@@ -1114,6 +1238,10 @@ projectionObj *projectionObj_new(char *string) {
     return proj;
   }
 
+int projectionObj_getUnits(projectionObj *self)
+{
+   return GetMapserverUnitUsingProj(self);
+}
 
 void projectionObj_destroy(projectionObj *self) {
     msFreeProjection(self);
@@ -1181,6 +1309,9 @@ void  styleObj_destroy(styleObj *self) {
     return; /* do nothing, map deconstrutor takes care of it all */
   }
 
+int styleObj_updateFromString(styleObj *self, char *snippet) {
+   return msUpdateStyleFromString(self, snippet, MS_FALSE);
+}
 
 int styleObj_setSymbolByName(styleObj *self, mapObj *map, char* pszSymbolName) {
     self->symbol = msGetSymbolIndex(&map->symbolset, pszSymbolName, MS_TRUE);
@@ -1275,3 +1406,47 @@ void cgirequestObj_destroy(cgiRequestObj *self)
     msFreeCharArray(self->ParamValues, self->NumParams);
     free(self);
 }
+
+
+/**********************************************************************
+ * class extensions hashTableObj
+ **********************************************************************/
+
+// New instance
+hashTableObj *hashTableObj_new() {
+   return msCreateHashTable();
+}
+
+// Destroy instance
+void hashTableObj_destroy(hashTableObj *self) {
+   msFreeHashTable(self);
+}
+
+// set a hash item given key and value
+int hashTableObj_set(hashTableObj *self, const char *key, const char *value) {
+   if (msInsertHashTable(self, key, value) == NULL) {
+      return MS_FAILURE;
+   }
+   return MS_SUCCESS;
+}
+
+// get value from item by its key
+const char *hashTableObj_get(hashTableObj *self, const char *key) {
+   return (msLookupHashTable(self, key));
+}
+
+// Remove one item from hash table
+int hashTableObj_remove(hashTableObj *self, const char *key) {
+   return (msRemoveHashTable(self, key));
+}
+
+// Clear all items in hash table (to NULL)
+void hashTableObj_clear(hashTableObj *self) {
+   msFreeHashItems(self);
+   initHashTable(self);
+}
+
+// Return the next key or first key if previousKey == NULL
+char *hashTableObj_nextKey(hashTableObj *self, const char *previousKey) {
+   return ((char *)msNextKeyFromHashTable(self, previousKey));
+}
