@@ -1,14 +1,13 @@
 /**********************************************************************
- * $Id: php_mapscript.h 6601 2007-08-16 15:34:18Z dmorissette $
+ * $Id: php_mapscript.h 8426 2009-01-12 16:57:27Z aboudreault $
  *
- * Name:     php_mapscript.c
- * Project:  PHP/MapScript extension for MapServer.
- * Language: ANSI C
- * Purpose:  Header file - prototypes / module definitions
- * Author:   Daniel Morissette, morissette@dmsolutions.ca
+ * Project:  MapServer
+ * Purpose:  PHP/MapScript extension for MapServer. Header file 
+ *           - prototypes / module definitions
+ * Author:   Daniel Morissette, DM Solutions Group (dmorissette@dmsolutions.ca)
  *
  **********************************************************************
- * Copyright (c) 2000-2005, Daniel Morissette, DM Solutions Group
+ * Copyright (c) 2000-2005, Daniel Morissette, DM Solutions Group Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -60,6 +59,7 @@
  *====================================================================*/
 
 mapObj         *mapObj_new(char *filename, char *new_path);
+mapObj         *mapObj_newFromString(char *map_text, char *new_path);
 void            mapObj_destroy(mapObj* self);
 mapObj         *mapObj_clone(mapObj* self);
 int             mapObj_setRotation(mapObj* self, double rotation_angle );
@@ -127,9 +127,12 @@ char            *mapObj_generateSLD(mapObj *self);
 int             mapObj_loadOWSParameters(mapObj *self, cgiRequestObj *request, 
                                           char *wmtver_string);
 int             mapObj_OWSDispatch(mapObj *self, cgiRequestObj *req );
+int             mapObj_insertLayer(mapObj *self, layerObj *layer, int index);
+layerObj        *mapObj_removeLayer(mapObj *self, int layerindex);
 
 layerObj       *layerObj_new(mapObj *map);
 void            layerObj_destroy(layerObj* self);
+int             layerObj_updateFromString(layerObj *self, char *snippet);
 int             layerObj_open(layerObj *self);
 int             layerObj_whichShapes(layerObj *self, rectObj *poRect);
 shapeObj        *layerObj_nextShape(layerObj *self);
@@ -170,9 +173,23 @@ char            *layerObj_generateSLD(layerObj *self);
 int             layerObj_moveClassUp(layerObj *self, int index);
 int             layerObj_moveClassDown(layerObj *self, int index);
 classObj        *layerObj_removeClass(layerObj *self, int index);
+int             layerObj_setConnectionType(layerObj *self, int connectiontype, 
+                                           const char *library_str) ;
 
+int             labelObj_updateFromString(labelObj *self, char *snippet);
+
+int             legendObj_updateFromString(legendObj *self, char *snippet);
+
+int             queryMapObj_updateFromString(queryMapObj *self, char *snippet);
+
+int             referenceMapObj_updateFromString(referenceMapObj *self, char *snippet);
+
+int             scalebarObj_updateFromString(scalebarObj *self, char *snippet);
+
+int             webObj_updateFromString(webObj *self, char *snippet);
 
 classObj       *classObj_new(layerObj *layer, classObj *class);
+int             classObj_updateFromString(classObj *self, char *snippet);
 void            classObj_destroy(classObj* self);
 int             classObj_setExpression(classObj *self, char *string);
 char            *classObj_getExpressionString(classObj *self);
@@ -182,7 +199,7 @@ int             classObj_drawLegendIcon(classObj *self,
                                         mapObj *map,
                                         layerObj *layer, 
                                         int width, int height, 
-                                        gdImagePtr im, 
+                                        imageObj *im, 
                                         int dstX, int dstY);
 imageObj       *classObj_createLegendIcon(classObj *self, 
                                           mapObj *map, 
@@ -198,6 +215,9 @@ classObj        *classObj_clone(classObj *class, layerObj *layer);
 int             classObj_moveStyleUp(classObj *self, int index);
 int             classObj_moveStyleDown(classObj *self, int index);
 int             classObj_deleteStyle(classObj *self, int index);
+char            *classObj_getMetaData(classObj *self, char *name);
+int             classObj_setMetaData(classObj *self, char *name, char *value);
+int             classObj_removeMetaData(classObj *self, char *name);
 
 pointObj       *pointObj_new();
 void            pointObj_destroy(pointObj *self);
@@ -275,6 +295,7 @@ int             shapefileObj_add(shapefileObj *self, shapeObj *shape);
 int             shapefileObj_addPoint(shapefileObj *self, pointObj *point);
 
 projectionObj   *projectionObj_new(char *string);
+int             projectionObj_getUnits(projectionObj *self);
 void            projectionObj_destroy(projectionObj *self);
 
 void            labelCacheObj_freeCache(labelCacheObj *self);
@@ -286,19 +307,17 @@ DBFFieldType    DBFInfo_getFieldType(DBFInfo *self, int iField);
 
 styleObj       *styleObj_new(classObj *class, styleObj *style);
 void            styleObj_destroy(styleObj* self);
+int             styleObj_updateFromString(styleObj *self, char *snippet);
 int             styleObj_setSymbolByName(styleObj *self, mapObj *map, 
                                          char* pszSymbolName);
 styleObj       *styleObj_clone(styleObj *style);
 
-/*=====================================================================
- *                   Internal functions from hashtable_i.c
- *====================================================================*/
 hashTableObj   *hashTableObj_new();
 int             hashTableObj_set(hashTableObj *self, const char *key, 
                                  const char *value);
 const char     *hashTableObj_get(hashTableObj *self, const char *key);
-int            *hashTableObj_remove(hashTableObj *self, const char *key);
-void           *hashTableObj_clear(hashTableObj *self);
+int            hashTableObj_remove(hashTableObj *self, const char *key);
+void           hashTableObj_clear(hashTableObj *self);
 char           *hashTableObj_nextKey(hashTableObj *self, const char *prevkey);
 
 
