@@ -1,14 +1,13 @@
 /**********************************************************************
- * $Id: php_mapscript_util.h 6214 2007-06-15 14:10:05Z Assefa $
+ * $Id: php_mapscript_util.h 9180 2009-07-17 14:23:42Z aboudreault $
  *
- * Name:     php_mapscript_util.h
- * Project:  PHP/MapScript extension for MapServer : Utility functions
- * Language: ANSI C
- * Purpose:  Utility functions
- * Author:   Daniel Morissette, morissette@dmsolutions.ca
+ * Project:  MapServer
+ * Purpose:  PHP/MapScript extension for MapServer : Utility functions
+ *           Header - macros
+ * Author:   Daniel Morissette, DM Solutions Group (dmorissette@dmsolutions.ca)
  *
  **********************************************************************
- * Copyright (c) 2000, 2001, Daniel Morissette, DM Solutions Group
+ * Copyright (c) 2000, 2001, Daniel Morissette, DM Solutions Group Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -45,6 +44,11 @@
 #define TSRMLS_CC
 #endif
 
+/* PHP >=5.3 replaced ZVAL_DELREF by Z_DELREF_P */
+#if ZEND_MODULE_API_NO >= 20090626
+#define ZVAL_DELREF Z_DELREF_P
+#define ZVAL_ADDREF Z_ADDREF_P
+#endif
 
 /**********************************************************************
  *                  macros for setting object properties
