@@ -1,9 +1,16 @@
 /******************************************************************************
+ * $Id: mapsearch.c 7995 2008-10-23 03:19:03Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Various geospatial search operations.
  * Author:   Steve Lime and the MapServer team.
  *
+ * Notes: For information on point in polygon function please see:
+ *
+ *   http://www.ecse.rpi.edu/Homepages/wrf/Research/Short_Notes/pnpoly.html
+ *                                                                                                                  
+ * The appropriate copyright notice accompanies the funtion definition.
+ *
  ******************************************************************************
  * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
@@ -28,7 +35,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapsearch.c 7559 2008-05-05 14:46:11Z pramsey $")
+MS_CVSID("$Id: mapsearch.c 7995 2008-10-23 03:19:03Z sdlime $")
 
 #define LASTVERT(v,n)  ((v) == 0 ? n-2 : v-1)
 #define NEXTVERT(v,n)  ((v) == n-2 ? 0 : v+1)
@@ -106,6 +113,28 @@ int msPolygonDirection(lineObj *c)
       return(0); /* shouldn't happen unless the polygon is self intersecting */
 }
 
+/*                                                                                                                         
+** Copyright (c) 1970-2003, Wm. Randolph Franklin                                                                          
+**                                                                                                                         
+** Permission is hereby granted, free of charge, to any person obtaining a copy of this software and                       
+** associated documentation files (the "Software"), to deal in the Software without restriction, including                 
+** without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell                 
+** copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the                
+** following conditions:                                                                                                   
+**                                                                                                                         
+** 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the               
+**    following disclaimers.                                                                                               
+** 2. Redistributions in binary form must reproduce the above copyright notice in the documentation and/or                 
+**    other materials provided with the distribution.                                                                      
+** 3. The name of W. Randolph Franklin may not be used to endorse or promote products derived from this                    
+**    Software without specific prior written permission.                                                                  
+**                                                                                                                         
+** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT                   
+** LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN                  
+** NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,                    
+** WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE                     
+** SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                                                  
+*/
 int msPointInPolygon(pointObj *p, lineObj *c)
 {
   int i, j, status = MS_FALSE;
