@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapshape.h 8015 2008-11-11 18:14:14Z pramsey $
  *
  * Project:  MapServer
  * Purpose:  Shapefile access API
@@ -36,6 +37,8 @@
 extern "C" {
 #endif
 
+#define SHX_BUFFER_PAGE 1024
+
 #ifndef SWIG
 #define MS_PATH_LENGTH 1024
 
@@ -83,8 +86,11 @@ typedef	struct {
 
     int		nRecords;
     int		nMaxRecords;
+
     int		*panRecOffset;
     int		*panRecSize;
+    char  *panRecLoaded;
+    int   panRecAllLoaded;
 
     double	adBoundsMin[4];
     double	adBoundsMax[4];
@@ -100,6 +106,8 @@ typedef	struct {
 typedef SHPInfo * SHPHandle;
 #endif
 
+
+
 typedef	struct
 {
 #ifdef SWIG
@@ -174,12 +182,11 @@ typedef struct {
   int tilelayerindex;
 } msTiledSHPLayerInfo;
 
-
 /* shapefileObj function prototypes  */
-MS_DLL_EXPORT int msSHPOpenFile(shapefileObj *shpfile, char *mode, char *filename);
-MS_DLL_EXPORT int msSHPCreateFile(shapefileObj *shpfile, char *filename, int type);
-MS_DLL_EXPORT void msSHPCloseFile(shapefileObj *shpfile);
-MS_DLL_EXPORT int msSHPWhichShapes(shapefileObj *shpfile, rectObj rect, int debug);
+MS_DLL_EXPORT int msShapefileOpen(shapefileObj *shpfile, char *mode, char *filename, int log_failures);
+MS_DLL_EXPORT int msShapefileCreate(shapefileObj *shpfile, char *filename, int type);
+MS_DLL_EXPORT void msShapefileClose(shapefileObj *shpfile);
+MS_DLL_EXPORT int msShapefileWhichShapes(shapefileObj *shpfile, rectObj rect, int debug);
 
 /* SHP/SHX function prototypes */
 MS_DLL_EXPORT SHPHandle msSHPOpen( const char * pszShapeFile, const char * pszAccess );
@@ -191,6 +198,12 @@ MS_DLL_EXPORT void msSHPReadShape( SHPHandle psSHP, int hEntity, shapeObj *shape
 MS_DLL_EXPORT int msSHPReadPoint(SHPHandle psSHP, int hEntity, pointObj *point );
 MS_DLL_EXPORT int msSHPWriteShape( SHPHandle psSHP, shapeObj *shape );
 MS_DLL_EXPORT int msSHPWritePoint(SHPHandle psSHP, pointObj *point );
+/* SHX reading */
+MS_DLL_EXPORT int msSHXLoadAll( SHPHandle psSHP );
+MS_DLL_EXPORT int msSHXLoadPage( SHPHandle psSHP, int shxBufferPage );
+MS_DLL_EXPORT int msSHXReadOffset( SHPHandle psSHP, int hEntity );
+MS_DLL_EXPORT int msSHXReadSize( SHPHandle psSHP, int hEntity );
+
 
 /* tiledShapefileObj function prototypes are in mapserver.h */
 
