@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: mapsymbol.c 8859 2009-04-02 01:25:31Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  symbolObj related functions.
@@ -34,7 +35,7 @@
 #include "mapcopy.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapsymbol.c 7017 2007-11-08 15:19:29Z Assefa $")
+MS_CVSID("$Id: mapsymbol.c 8859 2009-04-02 01:25:31Z sdlime $")
 
 extern int msyylex(void); /* lexer globals */
 extern void msyyrestart(FILE *);
@@ -127,11 +128,12 @@ void initSymbol(symbolObj *s)
   s->transparent = MS_FALSE;
   s->transparentcolor = 0;
   s->patternlength = 0; /* solid line */
-  s->sizex = 0;
-  s->sizey = 0;
+  s->sizex = 1;
+  s->sizey = 1;
   s->filled = MS_FALSE;
   s->numpoints=0;
   s->img = NULL;
+  s->renderer_cache=NULL;
   s->imagepath = NULL;
   s->name = NULL;
   s->gap = 0;
@@ -154,6 +156,9 @@ int msFreeSymbol(symbolObj *s) {
   
   if(s->name) free(s->name);
   if(s->img) gdImageDestroy(s->img);
+#ifdef USE_AGG
+  if(s->renderer_cache) msFreeSymbolCacheAGG(s->renderer_cache);
+#endif
   if(s->font) free(s->font);
   if(s->imagepath) free(s->imagepath);
   if(s->character) free(s->character);
@@ -288,7 +293,7 @@ int loadSymbol(symbolObj *s, char *symbolpath)
 	switch(msyylex()) {  
 	case(END):
 	  if(s->patternlength < 2) {
-	    msSetError(MS_SYMERR, "Not enough pattern elements. A minimum of 2 are required", "loadSymbol()");
+	    msSetError(MS_SYMERR, "Not enough pattern elements. A minimum of 2 are required.", "loadSymbol()");
 	    return(-1);
 	  }	  
 	  done = MS_TRUE;
@@ -311,9 +316,14 @@ int loadSymbol(symbolObj *s, char *symbolpath)
       break;
     case(POINTS):
       done = MS_FALSE;
+      s->sizex = s->sizey = 0;
       for(;;) {
 	switch(msyylex()) { 
 	case(END):
+          if(s->numpoints < 1) {
+            msSetError(MS_SYMERR, "Not enough points are defined", "loadSymbol()");
+            return(-1);
+          }
 	  done = MS_TRUE;
 	  break;
 	case(MS_NUMBER):
@@ -626,11 +636,12 @@ int msLoadSymbolSet(symbolSetObj *symbolset, mapObj *map)
 
 int loadSymbolSet(symbolSetObj *symbolset, mapObj *map)
 {
-/* char old_path[MS_PATH_LENGTH]; */
-/* char *symbol_path; */
   int status=1;
   char szPath[MS_MAXPATHLEN], *pszSymbolPath=NULL;
 
+  int foundSymbolSetToken=MS_FALSE; 
+  int token;
+
   if(!symbolset) {
     msSetError(MS_SYMERR, "Symbol structure unallocated.", "loadSymbolSet()");
     return(-1);
@@ -657,7 +668,14 @@ int loadSymbolSet(symbolSetObj *symbolset, mapObj *map)
   ** Read the symbol file
   */
   for(;;) {
-    switch(msyylex()) {
+    token = msyylex(); 
+
+    if(!foundSymbolSetToken && token != SYMBOLSET) { 
+      msSetError(MS_IDENTERR, "First token must be SYMBOLSET, this doesn't look like a symbol file.", "msLoadSymbolSet()"); 
+      return(-1); 
+    }
+
+    switch(token) {
     case(END):
     case(EOF):      
       status = 0;
@@ -673,6 +691,7 @@ int loadSymbolSet(symbolSetObj *symbolset, mapObj *map)
           symbolset->numsymbols++;
       break;
     case(SYMBOLSET):
+      foundSymbolSetToken = MS_TRUE;
       break;
     default:
       msSetError(MS_IDENTERR, "Parsing error near (%s):(line %d)", "loadSymbolSet()", msyytext, msyylineno);
