@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maptemplate.h 6745 2007-08-30 13:52:07Z warmerdam $
+ * $Id: maptemplate.h 8855 2009-04-01 03:18:19Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Template processing related declarations.
@@ -33,22 +33,24 @@
 #include "mapserver.h"
 #include "maphash.h"
 
-#define IDSIZE 128
+#define IDPATTERN "^[0-9A-Za-z]{1,63}$"
+#define IDSIZE 64
 #define TEMPLATE_TYPE(s)  (((strncmp("http://", s, 7) == 0) || (strncmp("https://", s, 8) == 0) || (strncmp("ftp://", s, 6)) == 0)  ? MS_URL : MS_FILE)
 #define MAXZOOM 25
 #define MINZOOM -25
 
-enum coordSources {NONE, FROMIMGPNT, FROMIMGBOX, FROMIMGSHAPE, FROMREFPNT, FROMUSERPNT, FROMUSERBOX, FROMUSERSHAPE, FROMBUF, FROMSCALE};
+enum coordSources {NONE, FROMIMGPNT, FROMIMGBOX, FROMIMGSHAPE, FROMREFPNT, FROMUSERPNT, FROMUSERBOX, FROMUSERSHAPE, FROMBUF, FROMSCALE, FROMTILE};
 
-enum modes {BROWSE, ZOOMIN, ZOOMOUT, MAP, LEGEND, REFERENCE, SCALEBAR, COORDINATE, 
+enum modes {BROWSE, ZOOMIN, ZOOMOUT, MAP, LEGEND, LEGENDICON, REFERENCE, SCALEBAR, COORDINATE, 
             QUERY, QUERYMAP, NQUERY, NQUERYMAP, 
             ITEMQUERY, ITEMQUERYMAP, ITEMNQUERY, ITEMNQUERYMAP, 
             FEATUREQUERY, FEATUREQUERYMAP, FEATURENQUERY, FEATURENQUERYMAP, 
-            ITEMFEATUREQUERY, ITEMFEATUREQUERYMAP,ITEMFEATURENQUERY,ITEMFEATURENQUERYMAP,
-            INDEXQUERY,INDEXQUERYMAP};
+            ITEMFEATUREQUERY, ITEMFEATUREQUERYMAP, ITEMFEATURENQUERY, ITEMFEATURENQUERYMAP,
+            INDEXQUERY, INDEXQUERYMAP, TILE, OWS};
 
-/*! \srtuct mapservObj
- *  \brief Global structur used by template and mapserv
+
+/* struct mapservObj
+ * Global structure used by templates and mapserver CGI interface.
  * 
  * This structur was created to seperate template functionality
  * from the main mapserv file. Instead of moving all template
@@ -58,65 +60,61 @@ enum modes {BROWSE, ZOOMIN, ZOOMOUT, MAP, LEGEND, REFERENCE, SCALEBAR, COORDINAT
 */
 typedef struct
 {
-   /* should the query and/or map be saved */
-   int SaveMap, SaveQuery;
+  /* should the query and/or map be saved */
+  int savemap, savequery;
 
   cgiRequestObj *request;
 
-   mapObj *Map;
+  int sendheaders; /* should mime-type header be output, default will be MS_TRUE */
 
-   char **Layers;
+  mapObj *map;
+
+  char **Layers;
+  char *icon; /* layer:class combination that defines a legend icon */
+    
+  int NumLayers; /* number of layers specfied by a use */
+  int MaxLayers; /* Allocated size of Layers[] array */
+
+  layerObj *resultlayer;
    
-   /* number of layers specfied by a use */
-   int NumLayers;
-   /* Allocated size of Layers[] array */
-   int MaxLayers;
+  int UseShapes; /* are results of a query to be used in calculating an extent of some sort */
 
-   layerObj *ResultLayer;
+  shapeObj SelectShape, resultshape;
+
+  rectObj RawExt;
+
+  pointObj mappnt;
+  
+  double fZoom, Zoom;     
+  int ZoomDirection; /* whether zooming in or out, default is pan or 0 */
+
+  int Mode; /* can be BROWSE, QUERY, etc. */
    
-   int UseShapes; /* are results of a query to be used in calculating an extent of some sort */
-
-
-   shapeObj SelectShape, ResultShape;
-
-   rectObj RawExt;
-
-   pointObj MapPnt;
-
-   /* default for browsing */
-   double fZoom, Zoom;
+  int TileMode; /* can be GMAP, VE */
+  char *TileCoords; /* for GMAP: 0 0 1; for VE: 013021023 */
    
-   /* whether zooming in or out, default is pan or 0 */
-   int ZoomDirection; 
-
-   /* can be BROWSE, QUERY, etc. */
-   int Mode; 
+  char Id[IDSIZE]; /* big enough for time + pid */
    
-   /* big enough for time + pid */
-   char Id[IDSIZE]; 
+  int CoordSource;
+  double ScaleDenom; /* used to create a map extent around a point */
+
+  int ImgRows, ImgCols;
+  rectObj ImgExt; /* Existing image's mapextent */
+  rectObj ImgBox;
    
-   int CoordSource;
-   double ScaleDenom; /* used to create a map extent around a point */
+  pointObj RefPnt;
+  pointObj ImgPnt;
 
-   int ImgRows, ImgCols;
-   rectObj ImgExt; /* Existing image's mapextent */
-   rectObj ImgBox;
-   
-   pointObj RefPnt;
-   pointObj ImgPnt;
-
-   double Buffer;
-
-
-   
-   /* 
-    ** variables for multiple query results processing 
-    */
-   int RN; /* overall result number */
-   int LRN; /* result number within a layer */
-   int NL; /* total number of layers with results */
-   int NR; /* total number or results */
-   int NLR; /* number of results in a layer */
+  double Buffer;
+ 
+  /* 
+  ** variables for multiple query results processing 
+  */
+  int RN; /* overall result number */
+  int LRN; /* result number within a layer */
+  int NL; /* total number of layers with results */
+  int NR; /* total number or results */
+  int NLR; /* number of results in a layer */
 } mapservObj;
    
    
@@ -139,16 +137,14 @@ MS_DLL_EXPORT int setExtent(mapservObj *msObj);
 
 MS_DLL_EXPORT int msReturnPage(mapservObj* msObj, char* , int, char **);
 MS_DLL_EXPORT int msReturnURL(mapservObj* msObj, char*, int);
-MS_DLL_EXPORT int msReturnQuery(mapservObj* msObj, char* pszMimeType, char **papszBuffer);
-
-MS_DLL_EXPORT int msReturnTemplateQuery(mapservObj *msObj, char* pszMimeType, 
-                          char **papszBuffer);
+MS_DLL_EXPORT int msReturnNestedTemplateQuery(mapservObj* msObj, char* pszMimeType, char **papszBuffer);
+MS_DLL_EXPORT int msReturnTemplateQuery(mapservObj *msObj, char* pszMimeType,  char **papszBuffer);
 
 MS_DLL_EXPORT int msRedirect(char* url);
 
 MS_DLL_EXPORT char *generateLegendTemplate(mapservObj *msObj);
 
-MS_DLL_EXPORT int msGenerateImages(mapservObj *msObj, char *szQuery, int bReturnOnError);
+MS_DLL_EXPORT int msGenerateImages(mapservObj *msObj, int bQueryMap, int bReturnOnError);
 
 
 MS_DLL_EXPORT char *msProcessTemplate(mapObj *map, int bGenerateImages, 
