@@ -1,9 +1,10 @@
 /******************************************************************************
+ * $Id: maptree.c 7528 2008-04-25 21:32:01Z pramsey $
  *
  * Project:  MapServer
- * Purpose:  .qix spatial index implementation.  
+ * Purpose:  .qix spatial index implementation.  Derived from shapelib, and 
+ *           relicensed with permission of Frank Warmerdam (shapelib author).
  * Author:   Steve Lime
- *           Derived from code in shapelib.
  *
  ******************************************************************************
  * Copyright (c) 1996-2005 Regents of the University of Minnesota.
@@ -30,7 +31,7 @@
 #include "mapserver.h"
 #include "maptree.h"
 
-MS_CVSID("$Id: maptree.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: maptree.c 7528 2008-04-25 21:32:01Z pramsey $")
 
 /* -------------------------------------------------------------------- */
 /*      If the following is 0.5, nodes will be split in half.  If it    */
@@ -767,10 +768,14 @@ void msFilterTreeSearch(shapefileObj *shp, char *status, rectObj search_rect)
   int i;
   rectObj shape_rect;
 
-  for(i=0;i<shp->numshapes;i++) { /* for each shape */
-    if(msGetBit(status, i)) {
-      if(msSHPReadBounds(shp->hSHP, i, &shape_rect) == MS_SUCCESS)
-	if(msRectOverlap(&shape_rect, &search_rect) != MS_TRUE) msSetBit(status, i, 0);
+  i = msGetNextBit(status, 0, shp->numshapes);
+  while(i >= 0) {
+    if(msSHPReadBounds(shp->hSHP, i, &shape_rect) == MS_SUCCESS) {
+	    if(msRectOverlap(&shape_rect, &search_rect) != MS_TRUE) {
+	      msSetBit(status, i, 0);
+      }
     }
+    i = msGetNextBit(status, i+1, shp->numshapes);
   }
+
 }
