@@ -1,9 +1,14 @@
 /******************************************************************************
+ * $Id: maputil.c 8979 2009-04-30 04:09:00Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Various utility functions ... a real hodgepodge.
  * Author:   Steve Lime and the MapServer team.
  *
+ * Notes: Some code (notably msAlphaBlend()) are directly derived from GD. See
+ * the mapserver/GD-COPYING file for the GD license.  Use of this code in this
+ * manner is compatible with the MapServer license.
+ *
  ******************************************************************************
  * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
@@ -30,6 +35,7 @@
 #include <time.h>
 
 #include "mapserver.h"
+#include "maptime.h"
 #include "mapparser.h"
 #include "mapthread.h"
 #include "mapfile.h"
@@ -43,7 +49,7 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: maputil.c 6991 2007-11-01 19:02:59Z sdlime $")
+MS_CVSID("$Id: maputil.c 8979 2009-04-30 04:09:00Z sdlime $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -60,7 +66,7 @@ extern char *msyystring;
 static int bindIntegerAttribute(int *attribute, char *value)
 {
   if(!value || strlen(value) == 0) return MS_FAILURE;
-  *attribute = atoi(value);
+  *attribute = MS_NINT(atof(value)); /*use atof instead of atoi as a fix for bug 2394*/
   return MS_SUCCESS;
 }
 
@@ -113,7 +119,7 @@ static int bindColorAttribute(colorObj *attribute, char *value)
 /*
 ** Function to bind various layer properties to shape attributes.
 */
-int msBindLayerToShape(layerObj *layer, shapeObj *shape)
+int msBindLayerToShape(layerObj *layer, shapeObj *shape, int querymapMode)
 {
   int i, j;
   labelObj *label; /* for brevity */
@@ -141,15 +147,20 @@ int msBindLayerToShape(layerObj *layer, shapeObj *shape)
 
         if(style->bindings[MS_STYLE_BINDING_SIZE].index != -1) {
           style->size = 1;
-          bindIntegerAttribute(&style->size, shape->values[style->bindings[MS_STYLE_BINDING_SIZE].index]);
+          bindDoubleAttribute(&style->size, shape->values[style->bindings[MS_STYLE_BINDING_SIZE].index]);
         }
 
-        if(style->bindings[MS_STYLE_BINDING_COLOR].index != -1) {
+        if(style->bindings[MS_STYLE_BINDING_WIDTH].index != -1) {
+          style->width = 1;
+          bindDoubleAttribute(&style->width, shape->values[style->bindings[MS_STYLE_BINDING_WIDTH].index]);
+        }
+
+        if(style->bindings[MS_STYLE_BINDING_COLOR].index != -1 && (querymapMode != MS_TRUE)) {
           MS_INIT_COLOR(style->color, -1,-1,-1);
           bindColorAttribute(&style->color, shape->values[style->bindings[MS_STYLE_BINDING_COLOR].index]);
         }
 
-        if(style->bindings[MS_STYLE_BINDING_OUTLINECOLOR].index != -1) {
+        if(style->bindings[MS_STYLE_BINDING_OUTLINECOLOR].index != -1 && (querymapMode != MS_TRUE)) {
           MS_INIT_COLOR(style->outlinecolor, -1,-1,-1);
           bindColorAttribute(&style->outlinecolor, shape->values[style->bindings[MS_STYLE_BINDING_OUTLINECOLOR].index]);
         }
@@ -167,7 +178,7 @@ int msBindLayerToShape(layerObj *layer, shapeObj *shape)
 
       if(label->bindings[MS_LABEL_BINDING_SIZE].index != -1) {
         label->size = 1;
-        bindIntegerAttribute(&label->size, shape->values[label->bindings[MS_LABEL_BINDING_SIZE].index]);
+        bindDoubleAttribute(&label->size, shape->values[label->bindings[MS_LABEL_BINDING_SIZE].index]);
       }
 
       if(label->bindings[MS_LABEL_BINDING_COLOR].index != -1) {
@@ -332,7 +343,7 @@ int msEvalContext(mapObj *map, layerObj *layer, char *context)
 int msEvalExpression(expressionObj *expression, int itemindex, char **items, int numitems)
 {
   int i;
-  char *tmpstr=NULL;
+  char *tmpstr=NULL, *tmpstr2=NULL;
   int status;
   int expresult;  /* result of logical expression parsing operation */
   
@@ -358,9 +369,11 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
     tmpstr = strdup(expression->string);
 
     for(i=0; i<expression->numitems; i++) {
-        items[expression->indexes[i]] = msReplaceSubstring( items[expression->indexes[i]], "\'", "\\\'");
-        items[expression->indexes[i]] = msReplaceSubstring( items[expression->indexes[i]], "\"", "\\\"");
-        tmpstr = msReplaceSubstring(tmpstr, expression->items[i], items[expression->indexes[i]]);
+      tmpstr2 = strdup(items[expression->indexes[i]]);
+      tmpstr2 = msReplaceSubstring(tmpstr2, "\'", "\\\'");
+      tmpstr2 = msReplaceSubstring(tmpstr2, "\"", "\\\"");
+      tmpstr = msReplaceSubstring(tmpstr, expression->items[i], tmpstr2);
+      free(tmpstr2);
     }
     msAcquireLock( TLOCK_PARSER );
     msyystate = MS_TOKENIZE_EXPRESSION;
@@ -369,15 +382,14 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
     expresult = msyyresult;
     msReleaseLock( TLOCK_PARSER );
 
-    if (status != 0)
-    {
-        msSetError(MS_PARSEERR, "Failed to parse expression: %s", "msEvalExpression", tmpstr);
-        free(tmpstr);
-        return MS_FALSE;
+    if (status != 0) {
+      msSetError(MS_PARSEERR, "Failed to parse expression: %s", "msEvalExpression", tmpstr);
+      free(tmpstr);
+      return MS_FALSE;
     }
+
     free(tmpstr);
-    return expresult;
-    
+    return expresult;    
     break;
   case(MS_REGEX):
     if(itemindex == -1) {
@@ -428,9 +440,42 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
  * }
  */
 
-int msShapeGetClass(layerObj *layer, shapeObj *shape, double scaledenom)
+int *msAllocateValidClassGroups(layerObj *lp, int *nclasses)
 {
-  int i;
+    int *classgroup = NULL;
+    int nvalidclass = 0, i=0;
+
+    if (!lp || !lp->classgroup || lp->numclasses <=0 || !nclasses)
+      return NULL;
+
+    classgroup = (int *)malloc(sizeof(int)*lp->numclasses);       
+    nvalidclass = 0;
+    for (i=0; i<lp->numclasses; i++)
+    {
+        if (lp->class[i]->group && strcasecmp(lp->class[i]->group, lp->classgroup) == 0)
+        {
+            classgroup[nvalidclass] = i;
+            nvalidclass++;
+        }
+    }
+    if (nvalidclass > 0)
+    {
+        classgroup = (int *)realloc(classgroup, sizeof(int)*nvalidclass);
+        *nclasses = nvalidclass;
+        return classgroup;
+    }
+
+    if (classgroup)
+      msFree(classgroup);
+    
+    return NULL;
+        
+}       
+
+int msShapeGetClass(layerObj *layer, shapeObj *shape, double scaledenom, int *classgroup, int numclasses)
+{
+  int i, iclass;
+
 
   /* INLINE features do not work with expressions, allow the classindex */
   /* value set prior to calling this function to carry through. */
@@ -447,19 +492,37 @@ int msShapeGetClass(layerObj *layer, shapeObj *shape, double scaledenom)
     return(shape->classindex);
   }
 
-  for(i=0; i<layer->numclasses; i++) {
-    
-    if(scaledenom > 0) {  /* verify scaledenom here  */
-      if((layer->class[i]->maxscaledenom > 0) && (scaledenom > layer->class[i]->maxscaledenom))
-        continue; /* can skip this one, next class */
-      if((layer->class[i]->minscaledenom > 0) && (scaledenom <= layer->class[i]->minscaledenom))
-        continue; /* can skip this one, next class */
-    }
+  if (layer->numclasses > 0)
+  {
 
-    if(layer->class[i]->status != MS_DELETE && msEvalExpression(&(layer->class[i]->expression), layer->classitemindex, shape->values, layer->numitems) == MS_TRUE)
-      return(i);
+      if (classgroup == NULL || numclasses <=0)
+        numclasses = layer->numclasses;
+
+      for(i=0; i<numclasses; i++) 
+      {
+          if (classgroup)
+            iclass = classgroup[i];
+          else
+            iclass = i;
+
+          if (iclass < 0 || iclass >= layer->numclasses)        
+            continue; /*this should never happen but just in case*/
+
+          if(scaledenom > 0) {  /* verify scaledenom here  */
+              if((layer->class[iclass]->maxscaledenom > 0) && (scaledenom > layer->class[iclass]->maxscaledenom))
+                continue; /* can skip this one, next class */
+              if((layer->class[iclass]->minscaledenom > 0) && (scaledenom <= layer->class[iclass]->minscaledenom))
+                continue; /* can skip this one, next class */
+          }
+
+          if(layer->class[iclass]->status != MS_DELETE && 
+             msEvalExpression(&(layer->class[iclass]->expression), 
+                              layer->classitemindex, shape->values, layer->numitems) == MS_TRUE)
+          {
+              return(iclass);
+          }
+      }
   }
-
   return(-1); /* no match */
 }
 
@@ -579,6 +642,10 @@ int msSaveImage(mapObj *map, imageObj *img, char *filename)
 {
     int nReturnVal = -1;
     char szPath[MS_MAXPATHLEN];
+    struct mstimeval starttime, endtime;
+
+    if(map && map->debug >= MS_DEBUGLEVEL_TUNING) 
+        msGettimeofday(&starttime, NULL);
 
     if (img)
     {
@@ -596,12 +663,12 @@ int msSaveImage(mapObj *map, imageObj *img, char *filename)
         else if( MS_DRIVER_AGG(img->format) )
         {
             if(map != NULL && filename != NULL )
-                nReturnVal = msSaveImageAGG(img->img.gd, 
+                nReturnVal = msSaveImageAGG(img, 
                                            msBuildPath(szPath, map->mappath, 
                                                        filename), 
                                            img->format );
             else
-                nReturnVal = msSaveImageAGG(img->img.gd, filename, img->format);
+                nReturnVal = msSaveImageAGG(img, filename, img->format);
         }
 #endif
         else if( MS_DRIVER_IMAGEMAP(img->format) )
@@ -655,6 +722,13 @@ int msSaveImage(mapObj *map, imageObj *img, char *filename)
                        "msSaveImage()"); 
     }
 
+    if(map && map->debug >= MS_DEBUGLEVEL_TUNING) {
+      msGettimeofday(&endtime, NULL);
+      msDebug("msSaveImage() total time: %.3fs\n", 
+              (endtime.tv_sec+endtime.tv_usec/1.0e6)-
+              (starttime.tv_sec+starttime.tv_usec/1.0e6) );
+    }
+
     return nReturnVal;
 }
 
@@ -679,7 +753,7 @@ unsigned char *msSaveImageBuffer(imageObj* image, int *size_ptr, outputFormatObj
 #ifdef USE_AGG
     else if( MS_DRIVER_AGG(image->format) )
     {
-        return msSaveImageBufferAGG(image->img.gd, size_ptr, format);
+        return msSaveImageBufferAGG(image, size_ptr, format);
     }
 #endif   
 	
@@ -1391,6 +1465,173 @@ void  msTransformShape(shapeObj *shape, rectObj extent, double cellsize,
     msTransformShapeToPixel(shape, extent, cellsize);
 }
 
+/*
+** Helper functions supplied as part of bug #2868 solution. Consider moving these to
+** mapprimitive.c for more general use.
+*/
+
+/* vector difference */
+static pointObj point_diff(const pointObj a, const pointObj b) {
+  pointObj retv = {a.x-b.x,a.y-b.y
+#ifdef USE_POINT_Z_M
+    ,a.z-b.z,a.m-b.m
+#endif
+  };
+  return retv;
+}
+
+/* vector sum */
+static pointObj point_sum(const pointObj a, const pointObj b) {
+  pointObj retv = {a.x+b.x,a.y+b.y
+#ifdef USE_POINT_Z_M
+    ,a.z+b.z,a.m+b.m
+#endif
+  };
+  return retv;
+}
+
+/* vector multiply */
+static pointObj point_mul(const pointObj a, double b) {
+  pointObj retv= {a.x*b,a.y*b
+#ifdef USE_POINT_Z_M
+    ,a.z*b,a.m*b
+#endif
+  };
+  return retv;
+}
+
+/* vector ??? */
+static double point_abs2(const pointObj a) {
+#ifdef USE_POINT_Z_M
+  return a.x*a.x+a.y*a.y+a.z*a.z+a.m*a.m;
+#else
+  return a.x*a.x+a.y*a.y;
+#endif
+}
+
+/* vector normal */
+static pointObj point_norm(const pointObj a) {
+  double lenmul;
+  pointObj retv;
+
+#ifdef USE_POINT_Z_M
+  if (a.x==0 && a.y==0 && a.z==0 && a.m==0) 
+#else
+  if (a.x==0 && a.y==0) 
+#endif
+    return a;
+
+  lenmul=1.0/sqrt(point_abs2(a));  /* this seems to be the costly operation */
+
+  retv.x = a.x*lenmul;
+  retv.y = a.y*lenmul;
+#ifdef USE_POINT_Z_M
+  retv.z = a.z*lenmul;
+  retv.m = a.m*lenmul;
+#endif
+
+  return retv;
+}
+
+/* rotate a vector 90 degrees */
+static pointObj point_rotz90(const pointObj a) {
+  double nx=-1.0*a.y, ny=a.x;
+  pointObj retv=a;
+  retv.x=nx; retv.y=ny;
+  return retv;
+}
+
+/* vector cross product (warning: z and m dimensions are ignored!) */
+static double point_cross(const pointObj a, const pointObj b) {
+  return a.x*b.y-a.y*b.x;
+}
+
+/* 
+** For offset corner point calculation 1/sin() is used
+** to avoid 1/0 division (and long spikes) we define a 
+** limit for sin().
+*/
+#define CURVE_SIN_LIMIT 0.3
+
+shapeObj *msOffsetPolyline(shapeObj *p, double offsetx, double offsety) {
+  int i, j, first,idx;
+
+  shapeObj *ret = (shapeObj*)malloc(sizeof(shapeObj));
+  msInitShape(ret);
+  ret->numlines = p->numlines;
+  ret->line=(lineObj*)malloc(sizeof(lineObj)*ret->numlines);
+  for(i=0;i<ret->numlines;i++) {
+    ret->line[i].numpoints=p->line[i].numpoints;
+    ret->line[i].point=(pointObj*)malloc(sizeof(pointObj)*ret->line[i].numpoints);
+  }
+
+  if(offsety == -99) { /* complex calculations */
+    for (i = 0; i < p->numlines; i++) {
+      pointObj old_pt, old_diffdir, old_offdir;
+      idx=0;
+      first = 1;
+
+      /* saved metrics of the last processed point */
+      if (p->line[i].numpoints>0)
+        old_pt=p->line[i].point[0];
+      for(j=1; j<p->line[i].numpoints; j++) {
+        const pointObj pt = p->line[i].point[j]; /* place of the point */
+        const pointObj diffdir = point_norm(point_diff(pt,old_pt)); /* direction of the line */
+        const pointObj offdir = point_rotz90(diffdir); /* direction where the distance between the line and the offset is measured */
+        pointObj offpt; /* this will be the corner point of the offset line */
+
+        /* offset line points computation */
+        if(first == 1) { /* first point */
+          first = 0;
+          offpt = point_sum(old_pt,point_mul(offdir,offsetx));
+        } else { /* middle points */
+          /* curve is the angle of the last and the current line's direction (supplementary angle of the shape's inner angle) */
+          double sin_curve = point_cross(diffdir,old_diffdir);
+          double cos_curve = point_cross(old_offdir,diffdir);
+          if ((-1.0)*CURVE_SIN_LIMIT < sin_curve && sin_curve < CURVE_SIN_LIMIT) {
+            /* do not calculate 1/sin, instead use a corner point approximation: average of the last and current offset direction and length */
+
+            /* 
+	    ** TODO: fair for obtuse inner angles, however, positive and negative
+            ** acute inner angles would need special handling - similar to LINECAP
+            ** to avoid drawing of long spikes
+            */
+            offpt = point_sum(old_pt, point_mul(point_sum(offdir, old_offdir),0.5*offsetx));
+          } else {
+            double base_shift = -1.0*(1.0+cos_curve)/sin_curve;
+            offpt = point_sum(old_pt, point_mul(point_sum(point_mul(diffdir,base_shift),offdir), offsetx));
+	  }
+        }
+        ret->line[i].point[idx]=offpt;
+        idx++;
+        old_pt=pt; old_diffdir=diffdir; old_offdir=offdir;
+      }
+	            
+      /* last point */
+      if(first == 0) {
+        pointObj offpt=point_sum(old_pt,point_mul(old_offdir,offsetx));
+        ret->line[i].point[idx]=offpt;
+        idx++;
+      }
+            
+      if(idx != p->line[i].numpoints) {
+        /* printf("shouldn't happen :(\n"); */
+        ret->line[i].numpoints=idx;
+        ret->line=realloc(ret->line,ret->line[i].numpoints*sizeof(pointObj));
+      }
+    }
+  } else { /* normal offset (eg. drop shadow) */
+    for (i = 0; i < p->numlines; i++) {
+      for(j=1; j<p->line[i].numpoints; j++) {
+        ret->line[i].point[j-1].x=p->line[i].point[j-1].x+offsetx;
+        ret->line[i].point[j-1].y=p->line[i].point[j-1].y+offsety;
+      }
+    }
+  }
+
+  return ret;
+}
+
 /*
 -------------------------------------------------------------------------------
  msSetup()
