@@ -1,9 +1,7 @@
 /**********************************************************************
- * $Id: mapwfs.c 6579 2007-08-14 14:36:38Z sdlime $
+ * $Id: mapwfs.c 8575 2009-02-18 15:18:37Z aboudreault $
  *
- * Name:     mapwfs.c
  * Project:  MapServer
- * Language: C
  * Purpose:  WFS server implementation
  * Author:   Daniel Morissette, DM Solutions Group (morissette@dmsolutions.ca)
  *
@@ -30,7 +28,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapwfs.c 6579 2007-08-14 14:36:38Z sdlime $")
+MS_CVSID("$Id: mapwfs.c 8575 2009-02-18 15:18:37Z aboudreault $")
 
 #if defined(USE_WFS_SVR)
 
@@ -45,20 +43,31 @@ MS_CVSID("$Id: mapwfs.c 6579 2007-08-14 14:36:38Z sdlime $")
 ** Report current MapServer error in XML exception format.
 */
 
-static int msWFSException(mapObj *map, const char *wmtversion) 
+int msWFSException(mapObj *map, const char *locator, const char *code,
+                   const char *version )
 {
     char *schemalocation = NULL;
     /* In WFS, exceptions are always XML.
     */
-    msIO_printf("Content-type: text/xml%c%c",10,10);
 
-    msIO_printf("<ServiceExceptionReport\n");
+    if( version == NULL )
+      version = "1.0.0";
+
+    if( msOWSParseVersionString(version) >= OWS_1_1_0 )
+      return msWFSException11( map, code, locator, version );
+
+    msIO_printf("Content-type: text/xml%c%c",10,10);
+    msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), "FO", "encoding", OWS_NOERR,
+                "<?xml version='1.0' encoding=\"%s\" ?>\n", "ISO-8859-1");
+
+    msIO_printf("<ServiceExceptionReport ");
+    msIO_printf("version=\"1.2.0\" ");
     msIO_printf("xmlns=\"http://www.opengis.net/ogc\" ");
     msIO_printf("xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ");
     schemalocation = msEncodeHTMLEntities(msOWSGetSchemasLocation(map));
-    msIO_printf("xsi:schemaLocation=\"http://www.opengis.net/ogc %s/wms/1.1.1/OGC-exception.xsd\">\n", schemalocation);
+    msIO_printf("xsi:schemaLocation=\"http://www.opengis.net/ogc %s/wfs/1.0.0/OGC-exception.xsd\">\n", schemalocation);
     free(schemalocation);
-    msIO_printf("  <ServiceException>\n");
+    msIO_printf("  <ServiceException code=\"%s\" locator=\"%s\">\n", code, locator);
     /* Optional <Locator> element currently unused. */
     /* msIO_printf("    <Message>\n"); */
     msWriteErrorXML(stdout);
@@ -121,7 +130,7 @@ static void msWFSPrintRequestCap(const char *wmtver, const char *request,
 ** Returns true (1) is this layer meets the requirements to be served as
 ** a WFS feature type.
 */
-static int msWFSIsLayerSupported(layerObj *lp)
+int msWFSIsLayerSupported(layerObj *lp)
 {
     /* In order to be supported, lp->type must be specified, even for 
     ** layers that are OGR, SDE, SDO, etc connections.
@@ -131,7 +140,8 @@ static int msWFSIsLayerSupported(layerObj *lp)
         (lp->type == MS_LAYER_POINT ||
          lp->type == MS_LAYER_LINE ||
          lp->type == MS_LAYER_POLYGON ) &&
-        lp->connectiontype != MS_WMS )
+        lp->connectiontype != MS_WMS && 
+        lp->connectiontype != MS_GRATICULE)
     {
         return 1; /* true */
     }
@@ -195,6 +205,9 @@ static const char *msWFSGetGeomType(layerObj *lp)
 int msWFSDumpLayer(mapObj *map, layerObj *lp)
 {
    rectObj ext;
+   const char *pszWfsSrs = NULL;
+   projectionObj poWfs;
+   int result = 0;
    
    msIO_printf("    <FeatureType>\n");
 
@@ -232,35 +245,39 @@ int msWFSDumpLayer(mapObj *map, layerObj *lp)
    if (msOWSGetEPSGProj(&(map->projection),&(map->web.metadata),"FO",MS_TRUE) != NULL)
    {
        /* Map has a SRS.  Use it for all layers. */
-       msOWSPrintEncodeParam(stdout, "(at least one of) MAP.PROJECTION, LAYER.PROJECTION or wfs_srs metadata", 
-                  msOWSGetEPSGProj(&(map->projection),&(map->web.metadata),"FO",MS_TRUE),
-                  OWS_WARN, "        <SRS>%s</SRS>\n", NULL);
+       pszWfsSrs = msOWSGetEPSGProj(&(map->projection),&(map->web.metadata), "FO", MS_TRUE);
    }
    else
    {
        /* Map has no SRS.  Use layer SRS or produce a warning. */
-       msOWSPrintEncodeParam(stdout, "(at least one of) MAP.PROJECTION, LAYER.PROJECTION or wfs_srs metadata", 
-                  msOWSGetEPSGProj(&(lp->projection), &(lp->metadata), "FO", MS_TRUE),
-                  OWS_WARN, "        <SRS>%s</SRS>\n", NULL);
+       pszWfsSrs = msOWSGetEPSGProj(&(map->projection),&(map->web.metadata), "FO", MS_TRUE);
    }
 
+   msOWSPrintEncodeParam(stdout, "(at least one of) MAP.PROJECTION, LAYER.PROJECTION or wfs_srs metadata", 
+                  pszWfsSrs, OWS_WARN, "        <SRS>%s</SRS>\n", NULL);
+
    /* If layer has no proj set then use map->proj for bounding box. */
    if (msOWSGetLayerExtent(map, lp, "FO", &ext) == MS_SUCCESS)
    {
+       msInitProjection(&poWfs);
+       if (pszWfsSrs != NULL)
+           result = msLoadProjectionString(&(poWfs), (char *)pszWfsSrs);
+       
        if(lp->projection.numargs > 0) 
        {
            msOWSPrintLatLonBoundingBox(stdout, "        ", &(ext), 
-                                       &(lp->projection), OWS_WFS);
+                                       &(lp->projection), &(poWfs), OWS_WFS);
        } 
        else 
        {
            msOWSPrintLatLonBoundingBox(stdout, "        ", &(ext), 
-                                       &(map->projection), OWS_WFS);
+                                       &(map->projection), &(poWfs), OWS_WFS);
        }
+       msFreeProjection(&poWfs);
    }
    else
    {
-       msIO_printf("<!-- WARNING: Mandatory LatLongBoundingBox could not be established for this layer.  Consider setting LAYER.EXTENT or wfs_extent metadata. -->\n");
+       msIO_printf("<!-- WARNING: Optional LatLongBoundingBox could not be established for this layer.  Consider setting LAYER.EXTENT or wfs_extent metadata. Also check that your data exists in the DATA statement -->\n");
    }
 
    msOWSPrintURLType(stdout, &(lp->metadata), "FO", "metadataurl", 
@@ -271,7 +288,7 @@ int msWFSDumpLayer(mapObj *map, layerObj *lp)
 
    if (msOWSLookupMetadata(&(lp->metadata), "OFG", "featureid") == NULL)
    {
-       msIO_fprintf(stdout, "<!-- WARNING: Required Feature Id attribute (fid) not specified for this feature type. Make sure you set one of wfs_featureid, ows_feature_id or gml_featureid metadata. -->\n");
+       msIO_fprintf(stdout, "<!-- WARNING: Required Feature Id attribute (fid) not specified for this feature type. Make sure you set one of wfs_featureid, ows_featureid or gml_featureid metadata. -->\n");
    }
 
    msIO_printf("    </FeatureType>\n");
@@ -279,41 +296,75 @@ int msWFSDumpLayer(mapObj *map, layerObj *lp)
    return MS_SUCCESS;
 }
 
-
-
-
 /*
 ** msWFSGetCapabilities()
 */
-int msWFSGetCapabilities(mapObj *map, const char *wmtver, cgiRequestObj *req) 
+int msWFSGetCapabilities(mapObj *map, wfsParamsObj *wfsparams, cgiRequestObj *req) 
 {
   char *script_url=NULL, *script_url_encoded;
-  int i;
+  const char *updatesequence=NULL;
+  const char *wmtver=NULL;
+  char tmpString[OWS_VERSION_MAXLEN];
+  int wfsSupportedVersions[] = {OWS_1_1_0, OWS_1_0_0};
+  int wfsNumSupportedVersions = 2;
+
+  int i=0, tmpInt=0;
+
+  
+  /* negotiate version */
+  tmpInt = msOWSNegotiateVersion(msOWSParseVersionString(wfsparams->pszVersion), wfsSupportedVersions, 
+                                 wfsNumSupportedVersions);
+
+  /* set result as string and carry on */
+  if (wfsparams->pszVersion)
+    msFree(wfsparams->pszVersion);
+  wfsparams->pszVersion = strdup(msOWSGetVersionString(tmpInt, tmpString));
+
+  if( wfsparams->pszVersion == NULL ||  strncmp(wfsparams->pszVersion,"1.1",3) == 0 )
+    return msWFSGetCapabilities11( map, wfsparams, req );
 
   /* Decide which version we're going to return... only 1.0.0 for now */
-  wmtver = "1.0.0";
+  wmtver = strdup("1.0.0");
 
   /* We need this server's onlineresource. */
   if ((script_url=msOWSGetOnlineResource(map, "FO", "onlineresource", req)) == NULL ||
       (script_url_encoded = msEncodeHTMLEntities(script_url)) == NULL)
   {
-      return msWFSException(map, wmtver);
+      msSetError(MS_WFSERR, "Server URL not found", "msWFSGetCapabilities()");
+      return msWFSException(map, "mapserv", "NoApplicableCode", wmtver);
   }
 
-  msIO_printf("Content-type: text/xml%c%c",10,10); 
+  updatesequence = msOWSLookupMetadata(&(map->web.metadata), "FO", "updatesequence");
 
+  if (!updatesequence)
+    updatesequence = strdup("0");
+
+  if (wfsparams->pszUpdateSequence != NULL) {
+      i = msOWSNegotiateUpdateSequence(wfsparams->pszUpdateSequence, updatesequence);
+      if (i == 0) { /* current */
+          msSetError(MS_WFSERR, "UPDATESEQUENCE parameter (%s) is equal to server (%s)", "msWFSGetCapabilities()", wfsparams->pszUpdateSequence, updatesequence);
+          return msWFSException(map, "updatesequence", "CurrentUpdateSequence", wmtver);
+      }
+      if (i > 0) { /* invalid */
+          msSetError(MS_WFSERR, "UPDATESEQUENCE parameter (%s) is higher than server (%s)", "msWFSGetCapabilities()", wfsparams->pszUpdateSequence, updatesequence);
+          return msWFSException(map, "updatesequence", "InvalidUpdateSequence", wmtver);
+      }
+  }
+
+  msIO_printf("Content-type: text/xml%c%c",10,10);
   msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), "FO", "encoding", OWS_NOERR,
                 "<?xml version='1.0' encoding=\"%s\" ?>\n",
                 "ISO-8859-1");
 
+
   msIO_printf("<WFS_Capabilities \n"
          "   version=\"%s\" \n"
-         "   updateSequence=\"0\" \n"
+         "   updateSequence=\"%s\" \n"
          "   xmlns=\"http://www.opengis.net/wfs\" \n"
          "   xmlns:ogc=\"http://www.opengis.net/ogc\" \n"
          "   xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
          "   xsi:schemaLocation=\"http://www.opengis.net/wfs %s/wfs/%s/WFS-capabilities.xsd\">\n",
-         wmtver,
+         wmtver, updatesequence,
          msOWSGetSchemasLocation(map), wmtver);
 
   /* Report MapServer Version Information */
@@ -390,6 +441,9 @@ int msWFSGetCapabilities(mapObj *map, const char *wmtver, cgiRequestObj *req)
       layerObj *lp;
       lp = GET_LAYER(map, i);
 
+      if (lp->status == MS_DELETE)
+         continue;
+
       /* List only vector layers in which DUMP=TRUE */
       if (msWFSIsLayerSupported(lp))
       {
@@ -674,14 +728,20 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
     }
   } 
 
+  /*set the output format to gml3 for wfs1.1*/
+  if(paramsObj->pszVersion == NULL || strncmp(paramsObj->pszVersion,"1.1",3) == 0 )
+    outputformat = OWS_SFE_SCHEMA;
+
   if (paramsObj->pszOutputFormat) {
-    if(strcasecmp(paramsObj->pszOutputFormat, "XMLSCHEMA") == 0)
+    if(strcasecmp(paramsObj->pszOutputFormat, "XMLSCHEMA") == 0 ||
+       strstr(paramsObj->pszOutputFormat, "gml/2")!= NULL)
       outputformat = OWS_DEFAULT_SCHEMA;
-    else if(strcasecmp(paramsObj->pszOutputFormat, "SFE_XMLSCHEMA") == 0)
+    else if(strcasecmp(paramsObj->pszOutputFormat, "SFE_XMLSCHEMA") == 0 ||
+            strstr(paramsObj->pszOutputFormat, "gml/3")!= NULL)
       outputformat = OWS_SFE_SCHEMA;
     else {
       msSetError(MS_WFSERR, "Unsupported DescribeFeatureType outputFormat (%s).", "msWFSDescribeFeatureType()", paramsObj->pszOutputFormat);
-      return msWFSException(map, paramsObj->pszVersion);
+      return msWFSException(map, "outputformat", "InvalidParameterValue", paramsObj->pszVersion);
     }
   }
 
@@ -690,7 +750,7 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
     for (i=0; i<numlayers; i++) {
       if (msGetLayerIndex(map, layers[i]) < 0) {
 	      msSetError(MS_WFSERR, "Invalid typename (%s).", "msWFSDescribeFeatureType()", layers[i]);/* paramsObj->pszTypeName); */
-        return msWFSException(map, paramsObj->pszVersion);
+              return msWFSException(map, "typename", "InvalidParameterValue", paramsObj->pszVersion);
       }
     }
   }
@@ -743,7 +803,7 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
   encoded = msEncodeHTMLEntities( msOWSGetSchemasLocation(map) );
   if(outputformat == OWS_SFE_SCHEMA) /* reference GML 3.1.1 schema */
     msIO_printf("\n  <import namespace=\"http://www.opengis.net/gml\"\n"
-		"          schemaLocation=\"%s/gml/3.1.1/base/feature.xsd\" />\n", encoded);
+		"          schemaLocation=\"%s/gml/3.1.1/base/gml.xsd\" />\n", encoded);
   else /* default GML 2.1.x schema */
     msIO_printf("\n  <import namespace=\"http://www.opengis.net/gml\"\n"
 	      "          schemaLocation=\"%s/gml/2.1.2/feature.xsd\" />\n", encoded);
@@ -762,8 +822,10 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
     }
   }
 
-  /* output definition for the default feature container, can't use wfs:FeatureCollection with GML3 */
-  if(outputformat == OWS_SFE_SCHEMA) {
+  /* output definition for the default feature container, can't use wfs:FeatureCollection with GML3:
+     kept here so that the behaviour with whs1.0 and gml3 output is preserved.
+     We can use the wfs:FeatureCollection for wfs1.1*/
+  if(outputformat == OWS_SFE_SCHEMA && strncmp(paramsObj->pszVersion,"1.1",3) != 0) {
     value = msOWSLookupMetadata(&(map->web.metadata), "FO", "feature_collection");
     if(value) collection_name = value;
     
@@ -960,7 +1022,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       msSetError(MS_WFSERR, 
                  "Incomplete WFS request: TYPENAME parameter missing", 
                  "msWFSGetFeature()");
-      return msWFSException(map, paramsObj->pszVersion);
+      return msWFSException(map, "typename", "MissingParameterValue", paramsObj->pszVersion);
   }
 
 
@@ -986,7 +1048,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     
     if (layers==NULL || numlayers < 1) {
       msSetError(MS_WFSERR, "At least one type name required in TYPENAME parameter.", "msWFSGetFeature()");
-      return msWFSException(map, paramsObj->pszVersion);
+      return msWFSException(map, "typename", "InvalidParameterValue", paramsObj->pszVersion);
     }
     
     tokens = msStringSplit(layers[0], ':', &n);
@@ -1028,7 +1090,8 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	
 	if (msWFSIsLayerSupported(lp) && lp->name && strcasecmp(lp->name, layers[k]) == 0) {
 	  const char *pszThisLayerSRS;
-	  
+          char szBuf[32];
+          rectObj ext;
 	  bLayerFound = MS_TRUE;
 	  
 	  lp->status = MS_ON;
@@ -1046,7 +1109,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	    msSetError(MS_WFSERR, 
 		       "Server config error: SRS must be set at least at the map or at the layer level.",
 		       "msWFSGetFeature()");
-	    return msWFSException(map, paramsObj->pszVersion);
+	    return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
 	  }
 
 	  /* Keep track of output SRS.  If different from value */
@@ -1059,10 +1122,31 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	    msSetError(MS_WFSERR, 
 		       "Invalid GetFeature Request: All TYPENAMES in a single GetFeature request must have been advertized in the same SRS.  Please check the capabilities and reformulate your request.",
 		       "msWFSGetFeature()");
-	    return msWFSException(map, paramsObj->pszVersion);
+	    return msWFSException(map, "typename", "InvalidParameterValue", paramsObj->pszVersion);
 	  }
-	}
-	
+
+          /* set the map extent to the layer extent */
+
+          /* get the extent of the layer (bbox will get further filtered later */
+          /* if the client specifies BBOX or a spatial filter */
+
+          if (msOWSGetLayerExtent(map, lp, "FO", &ext) == MS_SUCCESS) {
+            sprintf(szBuf, "init=epsg:%.10s", pszMapSRS+5);
+
+            if (szBuf != NULL) {
+              if (msLoadProjectionString(&(map->projection), szBuf) != 0) {
+                 msSetError(MS_WFSERR, "msLoadProjectionString() failed: %s", "msWFSGetFeature()", szBuf);
+                 return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
+              }
+            }
+
+            if (msProjectionsDiffer(&map->projection, &lp->projection) == MS_TRUE) {
+               msProjectRect(&lp->projection, &map->projection, &(ext));
+            }
+
+            bbox = map->extent = ext;
+          }
+        }
       }
 
       if (!bLayerFound) {
@@ -1072,7 +1156,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	msSetError(MS_WFSERR, 
 		   "TYPENAME '%s' doesn't exist in this server.  Please check the capabilities and reformulate your request.",
 		   "msWFSGetFeature()", layers[k]);
-	return msWFSException(map, paramsObj->pszVersion);
+	return msWFSException(map, "typename", "InvalidParameterValue", paramsObj->pszVersion);
       }
     }
   }
@@ -1087,13 +1171,19 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       outputformat = OWS_GML3;
       output_schema_format = "SFE_XMLSCHEMA";
     } else {
-      msSetError(MS_WFSERR, 
-		 "Unsupported GetFeature outputFormat (%s). Only GML2 and GML3 are supported.", 
-		 "msWFSDescribeFeatureType()", paramsObj->pszOutputFormat);
-      return msWFSException(map, paramsObj->pszVersion);
+      outputformat = OWS_GML2;
+      output_schema_format = "XMLSCHEMA";
     }
   }
-
+  else
+  {
+      /*set the output format using the version if available*/
+      if(paramsObj->pszVersion == NULL || strncmp(paramsObj->pszVersion,"1.1",3) == 0 )
+      {
+          outputformat = OWS_GML3;
+          output_schema_format = "text/xml; subtype=gml/3.1.1";
+      }
+  }
   /* else if (strcasecmp(names[i], "PROPERTYNAME") == 0) */
   /* { */
   /*  */
@@ -1122,7 +1212,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     tokens = msStringSplit(paramsObj->pszBbox, ',', &n);
     if (tokens==NULL || n != 4) {
       msSetError(MS_WFSERR, "Wrong number of arguments for BBOX.", "msWFSGetFeature()");
-      return msWFSException(map, paramsObj->pszVersion);
+      return msWFSException(map, "bbox", "InvalidParameterValue", paramsObj->pszVersion);
     }
     bbox.minx = atof(tokens[0]);
     bbox.miny = atof(tokens[1]);
@@ -1144,6 +1234,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       FilterEncodingNode *psNode = NULL;
       int iLayerIndex =1;
       char **paszFilter = NULL;
+      errorObj   *ms_error;
       
       /* -------------------------------------------------------------------- */
       /*      Validate the parameters. When a FILTER parameter is given,      */
@@ -1157,21 +1248,21 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	msSetError(MS_WFSERR, 
                    "Required TYPENAME parameter missing for GetFeature with a FILTER parameter.", 
                    "msWFSGetFeature()");
-	return msWFSException(map, paramsObj->pszVersion);
+	return msWFSException(map, "typename", "MissingParameterValue", paramsObj->pszVersion);
       }
        
       if (bBBOXSet) {
 	msSetError(MS_WFSERR, 
                    "BBOX parameter and FILTER parameter are mutually exclusive in GetFeature.", 
                    "msWFSGetFeature()");
-	return msWFSException(map, paramsObj->pszVersion);
+	return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
       }
 
       if (bFeatureIdSet) {
 	msSetError(MS_WFSERR, 
                    "FEATUREID parameter and FILTER parameter are mutually exclusive in GetFeature.", 
                    "msWFSGetFeature()");
-	return msWFSException(map, paramsObj->pszVersion);
+	return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
       }
 
       /* -------------------------------------------------------------------- */
@@ -1191,7 +1282,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	if (tokens == NULL || nFilters <=0 || nFilters != numlayers) {
 	  msSetError(MS_WFSERR, "Wrong number of FILTER attributes",
 		     "msWFSGetFeature()");
-	  return msWFSException(map, paramsObj->pszVersion);
+	  return msWFSException(map, "filter", "InvalidParameterValue", paramsObj->pszVersion);
 	}
          
 	paszFilter = (char **)malloc(sizeof(char *)*nFilters);
@@ -1215,7 +1306,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	iLayerIndex = msGetLayerIndex(map, layers[i]);
 	if (iLayerIndex < 0) {
 	  msSetError(MS_WFSERR, "Invalid Typename in GetFeature : %s", "msWFSGetFeature()", layers[i]);
-	  return msWFSException(map, paramsObj->pszVersion);
+	  return msWFSException(map, "typename", "InvalidParameterValue", paramsObj->pszVersion);
 	}
 	psNode = FLTParseFilterEncoding(paszFilter[i]);
 	
@@ -1223,11 +1314,24 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	  msSetError(MS_WFSERR, 
 		     "Invalid or Unsupported FILTER in GetFeature : %s", 
 		     "msWFSGetFeature()", pszFilter);
-	  return msWFSException(map, paramsObj->pszVersion);
+	  return msWFSException(map, "filter", "InvalidParameterValue", paramsObj->pszVersion);
 	}
 
-	if( FLTApplyFilterToLayer(psNode, map, iLayerIndex, MS_FALSE) != MS_SUCCESS )
-	  return msWFSException(map, paramsObj->pszVersion);
+        /*preparse the filter for gml aliases*/
+        FLTPreParseFilterForAlias(psNode, map, iLayerIndex, "G");
+
+        /* run filter.  If no results are found, do not throw exception */
+        /* this is a null result */
+        if( FLTApplyFilterToLayer(psNode, map, iLayerIndex, MS_FALSE) != MS_SUCCESS ) 
+        {
+            ms_error = msGetErrorObj();
+	
+            if(ms_error->code != MS_NOTFOUND)
+            {
+                msSetError(MS_WFSERR, "FLTApplyFilterToLayer() failed", "msWFSGetFeature()", pszFilter);
+                return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
+            }
+        }
 
         FLTFreeFilterEncodingNode( psNode );
         psNode = NULL;
@@ -1293,7 +1397,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
                        msFreeCharArray(tokens, nTokens);
                      if (tokens1)
                        msFreeCharArray(tokens1, nTokens1);
-                     return msWFSException(map, paramsObj->pszVersion);
+                     return msWFSException(map, "featureid", "InvalidParameterValue", paramsObj->pszVersion);
                  }
                  if (tokens1)
                     msFreeCharArray(tokens1, nTokens1);
@@ -1318,8 +1422,10 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
                      }
                      psNode = FLTCreateFeatureIdFilterEncoding(aFIDValues[j]);
 
-                     if( FLTApplyFilterToLayer(psNode, map, lp->index, MS_FALSE) != MS_SUCCESS )
-                       return msWFSException(map, paramsObj->pszVersion);
+                     if( FLTApplyFilterToLayer(psNode, map, lp->index, MS_FALSE) != MS_SUCCESS ) {
+                       msSetError(MS_WFSERR, "FLTApplyFilterToLayer() failed", "msWFSGetFeature");
+                       return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
+                     }
 
                      FLTFreeFilterEncodingNode( psNode );
                      psNode = NULL;
@@ -1342,7 +1448,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
                      msFree(aFIDLayers);
                      msFree(aFIDValues);
                  }
-                 return msWFSException(map, paramsObj->pszVersion);
+                 return msWFSException(map, "featureid", "InvalidParameterValue", paramsObj->pszVersion);
              }
          }
 
@@ -1368,8 +1474,10 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       char szBuf[32];
       sprintf(szBuf, "init=epsg:%.10s", pszOutputSRS+5);
       
-      if (msLoadProjectionString(&(map->projection), szBuf) != 0)
-	return msWFSException(map, paramsObj->pszVersion);
+      if (msLoadProjectionString(&(map->projection), szBuf) != 0) {
+        msSetError(MS_WFSERR, "msLoadProjectionString() failed", "msWFSGetFeature()");
+	return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
+      }
     }
 
     /*
@@ -1382,8 +1490,10 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	errorObj   *ms_error;
 	ms_error = msGetErrorObj();
 	
-	if(ms_error->code != MS_NOTFOUND) 
-	  return msWFSException(map, paramsObj->pszVersion);
+	if(ms_error->code != MS_NOTFOUND) {
+          msSetError(MS_WFSERR, "ms_error->code not found", "msWFSGetFeature()");
+	  return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
+        }
       }
     }
 
@@ -1393,7 +1503,8 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 
     if ((script_url=msOWSGetOnlineResource(map,"FO","onlineresource",req)) ==NULL ||
         (script_url_encoded = msEncodeHTMLEntities(script_url)) == NULL) {
-      return msWFSException(map, paramsObj->pszVersion);
+      msSetError(MS_WFSERR, "Server URL not found", "msWFSGetFeature()");
+      return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
     }
 
     /*
@@ -1451,15 +1562,29 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 		  "                       %s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s\">\n",
 		  encoded_schema, encoded, user_namespace_uri_encoded, 
 		  script_url_encoded, encoded, encoded_typename, output_schema_format);
-    } else { 
-      msIO_printf("<%s:%s\n"
+    } 
+    else 
+    {
+      if(paramsObj->pszVersion && strncmp(paramsObj->pszVersion,"1.1",3) == 0 )
+      {
+      msIO_printf("<wfs:FeatureCollection\n"
+		  "   xmlns:%s=\"%s\"\n"  
+		  "   xmlns:gml=\"http://www.opengis.net/gml\"\n"
+		  "   xmlns:wfs=\"http://www.opengis.net/wfs\"\n"
+		  "   xmlns:ogc=\"http://www.opengis.net/ogc\"\n"
+      "   xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n",
+                  user_namespace_prefix, user_namespace_uri_encoded);
+      }
+      else
+      {
+         msIO_printf("<%s:%s\n"
       "   version=\"1.0.0\"\n"
 		  "   xmlns:%s=\"%s\"\n"  
 		  "   xmlns:gml=\"http://www.opengis.net/gml\"\n"
 		  "   xmlns:ogc=\"http://www.opengis.net/ogc\"\n"
       "   xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n",
       user_namespace_prefix, collection_name, user_namespace_prefix, user_namespace_uri_encoded);
-
+      }
       /* any additional namespaces */
       for(i=0; i<namespaceList->numnamespaces; i++) {
         if(namespaceList->namespaces[i].uri) {
@@ -1471,7 +1596,11 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
         }
       }
 
-		  msIO_printf("   xsi:schemaLocation=\"%s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s\">\n",
+      if(paramsObj->pszVersion && strncmp(paramsObj->pszVersion,"1.1",3) == 0 )
+        msIO_printf("   xsi:schemaLocation=\"%s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s  http://www.opengis.net/wfs http://schemas.opengis.net/wfs/1.1.0/wfs.xsd\">\n",
+		  user_namespace_uri_encoded, script_url_encoded, encoded, encoded_typename, output_schema_format);
+      else
+        msIO_printf("   xsi:schemaLocation=\"%s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s\">\n",
 		  user_namespace_uri_encoded, script_url_encoded, encoded, encoded_typename, output_schema_format);
     }
 
@@ -1497,7 +1626,12 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     if(outputformat == OWS_GML2)
       msIO_printf("</wfs:FeatureCollection>\n\n");
     else
-      msIO_printf("</%s:%s>\n\n", user_namespace_prefix, collection_name);
+    {
+      if(paramsObj->pszVersion && strncmp(paramsObj->pszVersion,"1.1",3) == 0)
+        msIO_printf("</wfs:FeatureCollection>\n\n", user_namespace_prefix, collection_name);
+      else
+        msIO_printf("</%s:%s>\n\n", user_namespace_prefix, collection_name);
+    }
 
     /*
     ** Done! Now a bit of clean-up.
@@ -1571,7 +1705,7 @@ int msWFSDispatch(mapObj *map, cgiRequestObj *requestobj)
                  "Incomplete WFS request: VERSION parameter missing", 
                  "msWFSDispatch()");
        
-      returnvalue = msWFSException(map, paramsObj->pszVersion);
+      returnvalue = msWFSException11(map, "version", "MissingParameterValue", "1.1.0");
       msWFSFreeParamsObj(paramsObj);
       free(paramsObj);
       paramsObj = NULL;
@@ -1583,7 +1717,7 @@ int msWFSDispatch(mapObj *map, cgiRequestObj *requestobj)
       msSetError(MS_WFSERR, 
                  "Incomplete WFS request: REQUEST parameter missing", 
                  "msWFSDispatch()");
-      returnvalue = msWFSException(map, paramsObj->pszVersion);
+      returnvalue = msWFSException(map, "request", "MissingParameterValue", paramsObj->pszVersion);
       msWFSFreeParamsObj(paramsObj);
       free(paramsObj);
       paramsObj = NULL;
@@ -1596,7 +1730,7 @@ int msWFSDispatch(mapObj *map, cgiRequestObj *requestobj)
                  "Incomplete WFS request: SERVICE parameter missing", 
                  "msWFSDispatch()");
        
-      returnvalue = msWFSException(map, paramsObj->pszVersion);
+      returnvalue = msWFSException(map, "service", "MissingParameterValue", paramsObj->pszVersion);
       msWFSFreeParamsObj(paramsObj);
       free(paramsObj);
       paramsObj = NULL;
@@ -1605,7 +1739,8 @@ int msWFSDispatch(mapObj *map, cgiRequestObj *requestobj)
 
   if ((status = msOWSMakeAllLayersUnique(map)) != MS_SUCCESS)
   {
-      returnvalue = msWFSException(map, paramsObj->pszVersion);
+      msSetError(MS_WFSERR, "msOWSMakeAllLayersUnique() failed", "msWFSDispatch()");
+      returnvalue = msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
       msWFSFreeParamsObj(paramsObj);
       free(paramsObj);
       paramsObj = NULL;
@@ -1616,8 +1751,7 @@ int msWFSDispatch(mapObj *map, cgiRequestObj *requestobj)
   */
   if (strcasecmp(paramsObj->pszRequest, "GetCapabilities") == 0 ) 
   {
-      returnvalue = 
-        msWFSGetCapabilities(map, paramsObj->pszVersion, requestobj);
+      returnvalue = msWFSGetCapabilities(map, paramsObj, requestobj);
       msWFSFreeParamsObj(paramsObj);
       free(paramsObj);
       paramsObj = NULL;
@@ -1628,12 +1762,13 @@ int msWFSDispatch(mapObj *map, cgiRequestObj *requestobj)
   ** Validate VERSION against the versions that we support... we don't do this
   ** for GetCapabilities in order to allow version negociation.
   */
-  if (strcmp(paramsObj->pszVersion, "1.0.0") != 0)
+  if (strcmp(paramsObj->pszVersion, "1.0.0") != 0 && 
+      strcmp(paramsObj->pszVersion, "1.1.0") != 0)
   {
       msSetError(MS_WFSERR, 
                  "WFS Server does not support VERSION %s.", 
                  "msWFSDispatch()", paramsObj->pszVersion);
-      returnvalue = msWFSException(map, paramsObj->pszVersion);
+      returnvalue = msWFSException11(map, "version", "InvalidParameterValue","1.1.0");
       msWFSFreeParamsObj(paramsObj);
       free(paramsObj);
       paramsObj = NULL;
@@ -1658,14 +1793,14 @@ int msWFSDispatch(mapObj *map, cgiRequestObj *requestobj)
       /* Unsupported WFS request */
       msSetError(MS_WFSERR, "Unsupported WFS request: %s", "msWFSDispatch()",
                  paramsObj->pszRequest);
-      returnvalue = msWFSException(map, paramsObj->pszVersion);
+      returnvalue = msWFSException(map, "request", "InvalidParameterValue", paramsObj->pszVersion);
   }
   else if (strcasecmp(paramsObj->pszService, "WFS") == 0)
   {
       /* Invalid WFS request */
       msSetError(MS_WFSERR, "Invalid WFS request: %s", "msWFSDispatch()",
                  paramsObj->pszRequest);
-      returnvalue = msWFSException(map, paramsObj->pszVersion);
+      returnvalue = msWFSException(map, "request", "InvalidParameterValue", paramsObj->pszVersion);
   }
 
   /* This was not detected as a WFS request... let MapServer handle it */
@@ -1711,6 +1846,8 @@ void msWFSFreeParamsObj(wfsParamsObj *wfsparams)
     {
         if (wfsparams->pszVersion)
           free(wfsparams->pszVersion);
+        if (wfsparams->pszUpdateSequence)
+          free(wfsparams->pszUpdateSequence);
         if (wfsparams->pszRequest)
           free(wfsparams->pszRequest);
         if (wfsparams->pszService)
@@ -1721,6 +1858,8 @@ void msWFSFreeParamsObj(wfsParamsObj *wfsparams)
           free(wfsparams->pszFilter);
         if (wfsparams->pszFeatureId)
           free(wfsparams->pszFeatureId);
+        if (wfsparams->pszOutputFormat)
+          free(wfsparams->pszOutputFormat);
     }
 }
 
@@ -1746,6 +1885,9 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
                 if (strcasecmp(request->ParamNames[i], "VERSION") == 0)
                   wfsparams->pszVersion = strdup(request->ParamValues[i]);
 
+                else if (strcasecmp(request->ParamNames[i], "UPDATESEQUENCE") == 0)
+                  wfsparams->pszUpdateSequence = strdup(request->ParamValues[i]);
+
                 else if (strcasecmp(request->ParamNames[i], "REQUEST") == 0)
                   wfsparams->pszRequest = strdup(request->ParamValues[i]);
                 
@@ -1772,11 +1914,11 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
             }
         }
         /* version is optional is the GetCapabilities. If not */
-        /* provided, set it. */
+        /* provided, set it. Default it to 1.1.0*/
         if (wfsparams->pszVersion == NULL &&
             wfsparams->pszRequest && 
             strcasecmp(wfsparams->pszRequest, "GetCapabilities") == 0)
-          wfsparams->pszVersion = strdup("1.0.0");
+          wfsparams->pszVersion = strdup("1.1.0");
     }
 /* -------------------------------------------------------------------- */
 /*      Parse the post request. It is assumed to be an XML document.    */
@@ -1812,6 +1954,11 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
                     else if(strcasecmp(psOperation->pszValue,"GetCapabilities")==0)
                     {
                         psGetCapabilities = psOperation;
+                        pszValue = (char*)CPLGetXMLValue(psGetFeature,  "updateSequence", NULL);
+
+                        if (pszValue)
+                            wfsparams->pszUpdateSequence = strdup(pszValue);
+
                         break;
                     }
                     else if(strcasecmp(psOperation->pszValue,
