@@ -1,4 +1,5 @@
 /**********************************************************************
+ * $Id: mapwfslayer.c 8754 2009-03-10 13:24:04Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  Implementation of WFS CONNECTIONTYPE - client to WFS servers
@@ -36,7 +37,7 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: mapwfslayer.c 6373 2007-07-24 02:37:18Z dmorissette $")
+MS_CVSID("$Id: mapwfslayer.c 8754 2009-03-10 13:24:04Z dmorissette $")
 
 #define WFS_V_0_0_14  14
 #define WFS_V_1_0_0  100
@@ -516,6 +517,7 @@ int msPrepareWFSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
     char *pszHashFileName = NULL;
     int bPostRequest = 0;
     wfsParamsObj *psParams = NULL;
+    char *pszHTTPCookieData = NULL;
     
 
     if (lp->connectiontype != MS_WFS || lp->connection == NULL)
@@ -575,6 +577,44 @@ int msPrepareWFSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
         nTimeout = atoi(pszTmp);
     }
 
+/*------------------------------------------------------------------
+ * Check to see if there's a HTTP Cookie to forward
+ * If Cookie differ between the two connection, it's NOT OK to merge
+ * the connection
+ * ------------------------------------------------------------------ */
+    if ((pszTmp = msOWSLookupMetadata(&(lp->metadata), 
+                                      "MO", "http_cookie")) != NULL)
+    {
+        if(strcasecmp(pszTmp, "forward") == 0)
+        {
+            pszTmp= msLookupHashTable(&(map->web.metadata),"http_cookie_data");
+            if(pszTmp != NULL)
+            {
+                pszHTTPCookieData = strdup(pszTmp);
+            }
+        }
+        else
+        {
+            pszHTTPCookieData = strdup(pszTmp);
+        }
+    }
+    else if ((pszTmp = msOWSLookupMetadata(&(map->web.metadata), 
+                                           "MO", "http_cookie")) != NULL)
+    {
+        if(strcasecmp(pszTmp, "forward") == 0)
+        {
+            pszTmp= msLookupHashTable(&(map->web.metadata),"http_cookie_data");
+            if(pszTmp != NULL)
+            {
+                pszHTTPCookieData = strdup(pszTmp);
+            }
+        }
+        else
+        {
+            pszHTTPCookieData = strdup(pszTmp);
+        }
+    }
+
 /* ------------------------------------------------------------------
  * If nLayerId == -1 then we need to figure it
  * ------------------------------------------------------------------ */
@@ -623,6 +663,8 @@ int msPrepareWFSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
         msOWSBuildURLFilename(map->web.imagepath, 
                               pszHashFileName,".tmp.gml");
     free(pszHashFileName);
+    pasReqInfo[(*numRequests)].pszHTTPCookieData = pszHTTPCookieData;
+    pszHTTPCookieData = NULL;
     pasReqInfo[(*numRequests)].nStatus = 0;
     pasReqInfo[(*numRequests)].nTimeout = nTimeout;
     pasReqInfo[(*numRequests)].bbox = bbox;
@@ -790,6 +832,11 @@ int msWFSLayerOpen(layerObj *lp,
     /* way we work with layers right now the bbox is unlikely to change */
     /* between now and the time whichshapes() would have been called by */
     /* the MapServer core. */
+#ifdef USE_PROJ
+    if((lp->map->projection.numargs > 0) && (lp->projection.numargs > 0))
+        msProjectRect(&lp->map->projection, &lp->projection, &psInfo->rect); /* project the searchrect to source coords */
+#endif
+
     if (msWFSLayerWhichShapes(lp, psInfo->rect) == MS_FAILURE)
         status = MS_FAILURE;
     
