@@ -1,5 +1,5 @@
 /*****************************************************************************
- * $Id: mapwmslayer.c 7004 2007-11-05 19:45:26Z dmorissette $
+ * $Id: mapwmslayer.c 8609 2009-02-25 03:10:32Z assefa $
  *
  * Project:  MapServer
  * Purpose:  Implementation of WMS CONNECTIONTYPE - client to WMS servers
@@ -49,6 +49,7 @@ int msInitWmsParamsObj(wmsParamsObj *wmsparams)
     wmsparams->onlineresource = NULL;
     wmsparams->params = msCreateHashTable();
     wmsparams->numparams=0;
+    wmsparams->httpcookiedata = NULL;
 
     return MS_SUCCESS;
 }
@@ -66,6 +67,8 @@ void msFreeWmsParamsObj(wmsParamsObj *wmsparams)
     msFreeHashTable(wmsparams->params);
     wmsparams->params = NULL;
 
+    msFree(wmsparams->httpcookiedata);
+
     wmsparams->numparams=0;
 }
 
@@ -82,7 +85,33 @@ static int msSetWMSParamString(wmsParamsObj *psWMSParams,
     if (urlencode)
     {
         char *pszTmp;
-        pszTmp = msEncodeUrl(value);
+
+        /*
+         *  Special case handling for characters the WMS specification
+         *  says should not be encoded, when they occur in certain
+         *  parameters.
+         *  
+         *  TODO: WMS 1.3 removes SRS and FORMAT from the set of 
+         *        exceptional cases.
+         */
+        if( strcmp(name,"LAYERS") == 0 ||
+            strcmp(name,"STYLES") == 0 ||
+            strcmp(name,"BBOX") == 0 )
+        {
+            pszTmp = msEncodeUrlExcept(value,',');
+        }
+        else if ( strcmp(name,"SRS") == 0 )
+        {
+            pszTmp = msEncodeUrlExcept(value,':');
+        }
+        else if ( strcmp(name,"FORMAT") == 0 )
+        {
+            pszTmp = msEncodeUrlExcept(value,'/');
+        }
+        else {
+            pszTmp = msEncodeUrl(value);
+        }
+
         msInsertHashTable(psWMSParams->params, name, pszTmp);
         msFree(pszTmp);
     }
@@ -169,6 +198,7 @@ static char *msBuildURLFromWMSParams(wmsParamsObj *wmsparams)
     /* Get rid of trailing '&'*/
     pszURL[nLen-1] = '\0';
 
+
     return pszURL;
 }
 
@@ -196,6 +226,7 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
 {
     const char *pszOnlineResource, *pszVersion, *pszName, *pszFormat;
     const char *pszFormatList, *pszStyle, *pszStyleList, *pszTime;
+    const char *pszBgColor, *pszTransparent;
     const char *pszSLD=NULL, *pszStyleSLDBody=NULL, *pszVersionKeyword=NULL;
     const char *pszSLDBody=NULL, *pszSLDURL = NULL;
     char *pszSLDGenerated = NULL;
@@ -215,6 +246,8 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
     pszTime =           msOWSLookupMetadata(&(lp->metadata), "MO", "time");
     pszSLDBody =        msOWSLookupMetadata(&(lp->metadata), "MO", "sld_body");
     pszSLDURL =         msOWSLookupMetadata(&(lp->metadata), "MO", "sld_url");
+    pszBgColor =        msOWSLookupMetadata(&(lp->metadata), "MO", "bgcolor");
+    pszTransparent =    msOWSLookupMetadata(&(lp->metadata), "MO", "transparent");
 
     if (pszOnlineResource==NULL || pszVersion==NULL || pszName==NULL)
     {
@@ -229,7 +262,7 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
 
     psWMSParams->onlineresource = strdup(pszOnlineResource);
 
-    if (strncmp(pszVersion, "1.0.7", 5) < 0)
+    if (strncmp(pszVersion, "1.0.7", 5) < 0) 
         pszVersionKeyword = "WMTVER";
     else
         pszVersionKeyword = "VERSION";
@@ -237,8 +270,6 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
     msSetWMSParamString(psWMSParams, pszVersionKeyword, pszVersion, MS_FALSE);
     msSetWMSParamString(psWMSParams, "SERVICE", "WMS",     MS_FALSE);
     msSetWMSParamString(psWMSParams, "LAYERS",  pszName,   MS_TRUE);
-    msSetWMSParamString(psWMSParams, "TRANSPARENT", "TRUE",MS_FALSE);
-
 
     if (pszFormat==NULL && pszFormatList==NULL)
     {
@@ -279,7 +310,7 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
 #endif
 #ifdef USE_GD_WBMP
                 || strcasecmp(papszTok[i], "WBMP")
-                || strcasecmp(papszTok[i], "image/wbmp")
+                || strcasecmp(papszTok[i], "image/vnd.wap.wbmp")
 #endif
                 )
             {
@@ -364,7 +395,11 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
     {
         if (strcasecmp(pszSLDBody, "AUTO") == 0)
         {
-            pszSLDGenerated = msSLDGenerateSLD(map, lp->index);
+            if (pszVersion && strncmp(pszVersion, "1.3.0", 5) == 0)
+              pszSLDGenerated = msSLDGenerateSLD(map, lp->index, "1.1.0");
+            else
+              pszSLDGenerated = msSLDGenerateSLD(map, lp->index, NULL);
+
             if (pszSLDGenerated)
             {
                 msSetWMSParamString(psWMSParams, "SLD_BODY", 
@@ -384,6 +419,20 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
         msSetWMSParamString(psWMSParams, "SLD", pszSLDURL, MS_TRUE);
     }	
 
+    if (pszBgColor)
+    {
+        msSetWMSParamString(psWMSParams, "BGCOLOR", pszBgColor, MS_TRUE);
+    }
+
+    if (pszTransparent)
+    {
+        msSetWMSParamString(psWMSParams, "TRANSPARENT", pszTransparent, MS_TRUE);
+    }
+    else 
+    {
+        msSetWMSParamString(psWMSParams, "TRANSPARENT", "TRUE", MS_TRUE);
+    }
+
     return MS_SUCCESS;
 }
 
@@ -408,9 +457,9 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
 {
 #ifdef USE_WMS_LYR
     char *pszEPSG = NULL;
-    const char *pszVersion, *pszTmp, *pszRequestParam, *pszExceptionsParam;
+    const char *pszVersion, *pszTmp, *pszRequestParam, *pszExceptionsParam, *pszQueryLayers=NULL;
     rectObj bbox;
-    int nVersion=0;
+    int nVersion=OWS_VERSION_NOTSET;
     
     if (lp->connectiontype != MS_WMS)
     {
@@ -482,8 +531,12 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
          strstr(psWMSParams->onlineresource, "query_layers=") == NULL &&
          msLookupHashTable(psWMSParams->params, "QUERY_LAYERS") == NULL)
     {
-        msSetError(MS_WMSCONNERR, "WMS Connection String must contain the QUERY_LAYERS parameter to support GetFeatureInfo requests (with name in uppercase).", "msBuildWMSLayerURL()");
-        return MS_FAILURE;
+        pszQueryLayers = msOWSLookupMetadata(&(lp->metadata), "MO", "name");
+
+        if (pszQueryLayers == NULL) {
+             msSetError(MS_WMSCONNERR, "wms_name not set or WMS Connection String must contain the QUERY_LAYERS parameter to support GetFeatureInfo requests (with name in uppercase).", "msBuildWMSLayerURL()");
+             return MS_FAILURE;
+        }
     }
 
 
@@ -586,7 +639,17 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
         }
     }
 
+/* ------------------------------------------------------------------
+ * Adjust for MapServer EXTENT being center of pixel and WMS BBOX being 
+ * edge of pixel (#2843), and then reproject if needed.
+ * ------------------------------------------------------------------ */
     bbox = map->extent;
+
+    bbox.minx -= map->cellsize * 0.5;
+    bbox.maxx += map->cellsize * 0.5;
+    bbox.miny -= map->cellsize * 0.5;
+    bbox.maxy += map->cellsize * 0.5;
+
     if (msProjectionsDiffer(&(map->projection), &(lp->projection)))
     {
         msProjectRect(&(map->projection), &(lp->projection), &bbox);
@@ -615,7 +678,7 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
  *   FORMAT
  *   TRANSPARENT
  *   STYLES
- *   QUERY_LAYERS (for queriable layers only)
+ *   QUERY_LAYERS (for queryable layers only)
  * ------------------------------------------------------------------ */
 
     if (nRequestType == WMS_GETFEATUREINFO)
@@ -647,7 +710,11 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
         msSetWMSParamInt(   psWMSParams, "Y",       nClickY);
  
         msSetWMSParamString(psWMSParams, "EXCEPTIONS", pszExceptionsParam, MS_FALSE);
-        msSetWMSParamString(psWMSParams, "INFO_FORMAT",pszInfoFormat, MS_TRUE);
+        msSetWMSParamString(psWMSParams, "INFO_FORMAT", pszInfoFormat, MS_TRUE);
+
+        if (pszQueryLayers) { /* not set in CONNECTION string */
+          msSetWMSParamString(psWMSParams, "QUERY_LAYERS", pszQueryLayers, MS_FALSE);
+        }
 
         /* If FEATURE_COUNT <= 0 then don't pass this parameter */
         /* The spec states that FEATURE_COUNT must be greater than zero */
@@ -747,11 +814,19 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
                              httpRequestObj *pasReqInfo, int *numRequests) 
 {
 #ifdef USE_WMS_LYR
-    char *pszURL = NULL;
+    char *pszURL = NULL, *pszHTTPCookieData = NULL;
     const char *pszTmp;
     rectObj bbox;
     int nTimeout, bOkToMerge, bForceSeparateRequest;
     wmsParamsObj sThisWMSParams;
+    
+    char    *pszProxyHost=NULL;
+    long     nProxyPort=0;
+    char    *pszProxyUsername=NULL, *pszProxyPassword=NULL;
+    char    *pszHttpAuthUsername=NULL, *pszHttpAuthPassword=NULL;
+    enum MS_HTTP_AUTH_TYPE eHttpAuthType = MS_BASIC;
+    enum MS_HTTP_AUTH_TYPE eProxyAuthType = MS_BASIC;
+    enum MS_HTTP_PROXY_TYPE eProxyType = MS_HTTP;
 
     if (lp->connectiontype != MS_WMS)
         return MS_FAILURE;
@@ -825,6 +900,112 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
     {
         nTimeout = atoi(pszTmp);
     }
+    
+/* ------------------------------------------------------------------
+ * Check for authentication and proxying metadata. If the metadata is not found
+ * in the layer metadata, check the map-level metadata.
+ * ------------------------------------------------------------------ */
+    if ((pszTmp = msOWSLookupMetadata2(&(lp->metadata), &(map->web.metadata),
+                                       "MO", "proxy_host")) != NULL)
+    {
+        pszProxyHost = strdup(pszTmp);
+    }
+    
+    if ((pszTmp = msOWSLookupMetadata2(&(lp->metadata), &(map->web.metadata),
+                                       "MO", "proxy_port")) != NULL)
+    {
+        nProxyPort = atol(pszTmp);
+    }
+    
+    if ((pszTmp = msOWSLookupMetadata2(&(lp->metadata), &(map->web.metadata),
+                                       "MO", "proxy_type")) != NULL)
+    {
+        
+        if (strcasecmp(pszTmp, "HTTP") == 0)
+            eProxyType = MS_HTTP;
+        else if (strcasecmp(pszTmp, "SOCKS5") == 0)
+            eProxyType = MS_SOCKS5;
+        else
+        {
+            msSetError(MS_WMSERR, "Invalid proxy_type metadata '%s' specified",
+                       "msPrepareWMSLayerRequest()", pszTmp);
+            return MS_FAILURE;
+        }
+    }
+    
+    if ((pszTmp = msOWSLookupMetadata2(&(lp->metadata), &(map->web.metadata),
+                                       "MO", "proxy_auth_type")) != NULL)
+    {
+        if (strcasecmp(pszTmp, "BASIC") == 0)
+            eProxyAuthType = MS_BASIC;
+        else if (strcasecmp(pszTmp, "DIGEST") == 0)
+            eProxyAuthType = MS_DIGEST;
+        else if (strcasecmp(pszTmp, "NTLM") == 0)
+            eProxyAuthType = MS_NTLM;
+        else if (strcasecmp(pszTmp, "ANY") == 0)
+            eProxyAuthType = MS_ANY;
+        else if (strcasecmp(pszTmp, "ANYSAFE") == 0)
+            eProxyAuthType = MS_ANYSAFE;
+        else
+        {
+            msSetError(MS_WMSERR, "Invalid proxy_auth_type metadata '%s' specified",
+                       "msPrepareWMSLayerRequest()", pszTmp);
+            return MS_FAILURE;
+        }
+    }
+    
+    
+    if ((pszTmp = msOWSLookupMetadata2(&(lp->metadata), &(map->web.metadata),
+                                       "MO", "proxy_username")) != NULL)
+    {
+        pszProxyUsername = strdup(pszTmp);
+    }
+    
+    if ((pszTmp = msOWSLookupMetadata2(&(lp->metadata), &(map->web.metadata),
+                                       "MO", "proxy_password")) != NULL)
+    {
+        pszProxyPassword = msDecryptStringTokens(map, pszTmp);
+        if (pszProxyPassword == NULL) {
+            return(MS_FAILURE);  /* An error should already have been produced */
+        }
+    }
+    
+    if ((pszTmp = msOWSLookupMetadata2(&(lp->metadata), &(map->web.metadata),
+                                       "MO", "auth_type")) != NULL)
+    {
+        if (strcasecmp(pszTmp, "BASIC") == 0)
+            eHttpAuthType = MS_BASIC;
+        else if (strcasecmp(pszTmp, "DIGEST") == 0)
+            eHttpAuthType = MS_DIGEST;
+        else if (strcasecmp(pszTmp, "NTLM") == 0)
+            eHttpAuthType = MS_NTLM;
+        else if (strcasecmp(pszTmp, "ANY") == 0)
+            eHttpAuthType = MS_ANY;
+        else if (strcasecmp(pszTmp, "ANYSAFE") == 0)
+            eHttpAuthType = MS_ANYSAFE;
+        else
+        {
+            msSetError(MS_WMSERR, "Invalid auth_type metadata '%s' specified",
+                       "msPrepareWMSLayerRequest()", pszTmp);
+            return MS_FAILURE;
+        }
+    }
+    
+    if ((pszTmp = msOWSLookupMetadata2(&(lp->metadata), &(map->web.metadata),
+                                       "MO", "auth_username")) != NULL)
+    {
+        pszHttpAuthUsername = strdup(pszTmp);
+    }
+    
+    if ((pszTmp = msOWSLookupMetadata2(&(lp->metadata), &(map->web.metadata),
+                                       "MO", "auth_password")) != NULL)
+    {
+        pszHttpAuthPassword = msDecryptStringTokens(map, pszTmp);
+        if (pszHttpAuthPassword == NULL) {
+            return(MS_FAILURE);  /* An error should already have been produced */
+        }
+    }
+    
 
 /* ------------------------------------------------------------------
  * Check if layer can be merged with previous WMS layer requests
@@ -870,6 +1051,56 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
         }
     }
 
+/*------------------------------------------------------------------
+ * Check to see if there's a HTTP Cookie to forward
+ * If Cookie differ between the two connection, it's NOT OK to merge
+ * the connection
+ * ------------------------------------------------------------------ */
+    if ((pszTmp = msOWSLookupMetadata(&(lp->metadata), 
+                                      "MO", "http_cookie")) != NULL)
+    {
+        if(strcasecmp(pszTmp, "forward") == 0)
+        {
+            pszTmp= msLookupHashTable(&(map->web.metadata),"http_cookie_data");
+            if(pszTmp != NULL)
+            {
+                pszHTTPCookieData = strdup(pszTmp);
+            }
+        }
+        else
+        {
+            pszHTTPCookieData = strdup(pszTmp);
+        }
+    }
+    else if ((pszTmp = msOWSLookupMetadata(&(map->web.metadata), 
+                                           "MO", "http_cookie")) != NULL)
+    {
+        if(strcasecmp(pszTmp, "forward") == 0)
+        {
+            pszTmp= msLookupHashTable(&(map->web.metadata),"http_cookie_data");
+            if(pszTmp != NULL)
+            {
+                pszHTTPCookieData = strdup(pszTmp);
+            }
+        }
+        else
+        {
+            pszHTTPCookieData = strdup(pszTmp);
+        }
+    }
+
+    if(bOkToMerge && pszHTTPCookieData != sThisWMSParams.httpcookiedata)
+    {
+        if(pszHTTPCookieData == NULL || sThisWMSParams.httpcookiedata == NULL)
+        {
+            bOkToMerge = MS_FALSE;
+        }
+        if(strcmp(pszHTTPCookieData, sThisWMSParams.httpcookiedata) != 0)
+        {
+            bOkToMerge = MS_FALSE;
+        }
+    }
+
     if (bOkToMerge)
     {
         /* Merge both requests into sThisWMSParams
@@ -912,7 +1143,6 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
  * ------------------------------------------------------------------ */
     pszURL = msBuildURLFromWMSParams(&sThisWMSParams);
 
-
     if (bOkToMerge && (*numRequests)>0)
     {
 /* ------------------------------------------------------------------
@@ -941,6 +1171,8 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
                        "msPrepareWMSLayerRequest()");
             return MS_FAILURE;
         }
+        pasReqInfo[(*numRequests)].pszHTTPCookieData = pszHTTPCookieData;
+        pszHTTPCookieData = NULL;
         pasReqInfo[(*numRequests)].pszOutputFile =msTmpFile(map->mappath,
                                                             map->web.imagepath,
                                                             "img.tmp");
@@ -948,7 +1180,17 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
         pasReqInfo[(*numRequests)].nTimeout = nTimeout;
         pasReqInfo[(*numRequests)].bbox = bbox;
         pasReqInfo[(*numRequests)].debug = lp->debug;
-
+        
+        pasReqInfo[(*numRequests)].pszProxyAddress  = pszProxyHost;
+        pasReqInfo[(*numRequests)].nProxyPort       = nProxyPort;
+        pasReqInfo[(*numRequests)].eProxyType       = eProxyType;
+        pasReqInfo[(*numRequests)].eProxyAuthType   = eProxyAuthType;
+        pasReqInfo[(*numRequests)].pszProxyUsername = pszProxyUsername;
+        pasReqInfo[(*numRequests)].pszProxyPassword = pszProxyPassword;
+        pasReqInfo[(*numRequests)].eHttpAuthType    = eHttpAuthType;
+        pasReqInfo[(*numRequests)].pszHttpUsername  = pszHttpAuthUsername;
+        pasReqInfo[(*numRequests)].pszHttpPassword  = pszHttpAuthPassword;
+        
         (*numRequests)++;
     }
 
