@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: shp2img.c 8489 2009-02-03 20:35:24Z tomkralidis $
  *
  * Project:  MapServer
  * Purpose:  Commandline .map rendering utility, mostly for testing.
@@ -27,8 +28,9 @@
  ****************************************************************************/
 
 #include "mapserver.h"
+#include "maptime.h"
 
-MS_CVSID("$Id: shp2img.c 6694 2007-08-23 13:31:08Z sdlime $")
+MS_CVSID("$Id: shp2img.c 8489 2009-02-03 20:35:24Z tomkralidis $")
 
 int main(int argc, char *argv[])
 {
@@ -40,8 +42,43 @@ int main(int argc, char *argv[])
   char **layers=NULL;
   int num_layers=0;
 
+  int layer_found=0;
+  char *invalid_layer=NULL;
+
   char *outfile=NULL; /* no -o sends image to STDOUT */
 
+  int iterations = 1;
+  int draws = 0;
+
+  for(i=1;i<argc;i++) { 
+    if (strcmp(argv[i],"-c") == 0) { /* user specified number of draws */
+      iterations = atoi(argv[i+1]);
+      printf("We will draw %d times...\n", iterations);
+      continue;
+    }
+
+    if(strcmp(argv[i], "-all_debug") == 0) /* global debug */
+    {
+        int debug_level = atoi(argv[++i]);
+
+        msSetGlobalDebugLevel(debug_level);
+
+        /* Send output to stderr by default */ 
+        if (msGetErrorFile() == NULL)
+            msSetErrorFile("stderr");
+
+        continue;
+    }
+    
+  }
+  
+  for(draws=0; draws<iterations; draws++) {
+
+  struct mstimeval requeststarttime, requestendtime;
+
+  if(msGetGlobalDebugLevel() >= MS_DEBUGLEVEL_TUNING) 
+      msGettimeofday(&requeststarttime, NULL);
+      
   if(argc > 1 && strcmp(argv[1], "-v") == 0) {
     printf("%s\n", msGetVersion());
     exit(0);
@@ -49,21 +86,26 @@ int main(int argc, char *argv[])
 
   /* ---- check the number of arguments, return syntax if not correct ---- */
   if( argc < 3 ) {
+    fprintf(stdout, "\nPurpose: convert a mapfile to an image\n\n");
     fprintf(stdout,
             "Syntax: shp2img -m mapfile [-o image] [-e minx miny maxx maxy] [-s sizex sizey]\n"
-            "               [-l layer1 \"[layers2...] ]\" [-i format] [-p n]\n"
-            "               [-all_debug n] [-map_debug n] [-layer_debug n]\n");
+            "               [-l \"layer1 [layers2...]\"] [-i format]\n"
+            "               [-all_debug n] [-map_debug n] [-layer_debug n] [-p n] [-c n] [-d layername datavalue]\n");
 
-    fprintf(stdout,"  -m mapfile: Map file to operate on - required.\n" );
-    fprintf(stdout,"  -i format: Override the IMAGETYPE value to pick output format.\n" );
+
+    fprintf(stdout,"  -m mapfile: Map file to operate on - required\n" );
+    fprintf(stdout,"  -i format: Override the IMAGETYPE value to pick output format\n" );
     fprintf(stdout,"  -o image: output filename (stdout if not provided)\n");
     fprintf(stdout,"  -e minx miny maxx maxy: extents to render\n");
     fprintf(stdout,"  -s sizex sizey: output image size\n");
-    fprintf(stdout,"  -l layers: layers to enable - make sure they are quoted and space seperated if more than one listed.\n" );
-    fprintf(stdout,"  -all_debug n: Set debug level for map and all layers.\n" );
-    fprintf(stdout,"  -map_debug n: Set map debug level.\n" );
-    fprintf(stdout,"  -layer_debug layer_name n: Set layer debug level.\n" );
+    fprintf(stdout,"  -l layers: layers to enable - make sure they are quoted and space seperated if more than one listed\n" );
+    fprintf(stdout,"  -all_debug n: Set debug level for map and all layers\n" );
+    fprintf(stdout,"  -map_debug n: Set map debug level\n" );
+    fprintf(stdout,"  -layer_debug layer_name n: Set layer debug level\n" );
+    fprintf(stdout,"  -c n: draw map n number of times\n" );
     fprintf(stdout,"  -p n: pause for n seconds after reading the map\n" );
+    fprintf(stdout,"  -d layername datavalue: change DATA value for layer\n" );
+
 
     exit(0);
   }
@@ -93,13 +135,14 @@ int main(int argc, char *argv[])
       }
     }
   }
-  
+
   if(!map) {
     fprintf(stderr, "Mapfile (-m) option not specified.\n");
     msCleanup();
     exit(0);
   }
 
+
   for(i=1;i<argc;i++) { /* Step though the user arguments */
 
     if(strcmp(argv[i],"-m") == 0) { /* skip it */
@@ -121,7 +164,7 @@ int main(int argc, char *argv[])
       outfile = argv[i+1];
       i+=1;
     }
-
+    
     if(strcmp(argv[i],"-i") == 0) { 
       outputFormatObj *format;
 
@@ -150,20 +193,18 @@ int main(int argc, char *argv[])
       i+=2;
     }
 
-    if(strcmp(argv[i], "-all_debug") == 0) /* debug */
+    if(strcmp(argv[i], "-all_debug") == 0) /* global debug */
     {
         int debug_level = atoi(argv[++i]);
 
+        /* msSetGlobalDebugLevel() already called. Just need to force debug
+         * level in map and all layers
+         */
         map->debug = debug_level;
         for(j=0; j<map->numlayers; j++) {
             GET_LAYER(map, j)->debug = debug_level;
         }
 
-        msSetGlobalDebugLevel(debug_level);
-
-        /* Send output to stderr by default */ 
-        if (msGetErrorFile() == NULL)
-            msSetErrorFile("stderr");
     }
     
     if(strcmp(argv[i], "-map_debug") == 0) /* debug */
@@ -213,9 +254,27 @@ int main(int argc, char *argv[])
     if(strcmp(argv[i],"-l") == 0) { /* load layer list */
       layers = msStringSplit(argv[i+1], ' ', &(num_layers));
 
+      for(j=0; j<num_layers; j++) { /* loop over -l */
+        layer_found=0;
+        for(k=0; k<map->numlayers; k++) {
+          if(GET_LAYER(map, k)->name && strcmp(GET_LAYER(map, k)->name, layers[j]) == 0) {
+            layer_found=1;
+            break;
+          }
+          else {
+            invalid_layer = strdup(layers[j]);
+          }
+        }
+        if (layer_found==0) {
+          fprintf(stderr, "Layer (-l) %s not found\n", invalid_layer);
+          msCleanup();
+          exit(0);
+        }
+      }
+
       for(j=0; j<map->numlayers; j++) {
 	if(GET_LAYER(map, j)->status == MS_DEFAULT)
-	  continue;
+          continue;
 	else {
 	  GET_LAYER(map, j)->status = MS_OFF;
 	  for(k=0; k<num_layers; k++) {
@@ -226,12 +285,13 @@ int main(int argc, char *argv[])
 	  }
 	}
       }
+
       msFreeCharArray(layers, num_layers);
 
       i+=1;
      }
   }
-
+  
   image = msDrawMap(map, MS_FALSE);
 
   if(!image) {
@@ -248,7 +308,16 @@ int main(int argc, char *argv[])
 
   msFreeImage(image);
   msFreeMap(map);
+
+  if(msGetGlobalDebugLevel() >= MS_DEBUGLEVEL_TUNING) {
+    msGettimeofday(&requestendtime, NULL);
+    msDebug("shp2img total time: %.3fs\n", 
+            (requestendtime.tv_sec+requestendtime.tv_usec/1.0e6)-
+            (requeststarttime.tv_sec+requeststarttime.tv_usec/1.0e6) );
+  }
+
   msCleanup();
 
+} /*   for(draws=0; draws<iterations; draws++) { */
   return(0);
 } /* ---- END Main Routine ---- */
